/*******************************************************************************
**
** Photivo
**
** Copyright (C) 2008,2009 Jos De Laender <jos.de_laender@telenet.be>
** Copyright (C) 2009-2011 Michael Munzert <mail@mm-log.com>
**
** This file is part of Photivo.
**
** Photivo is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 3
** as published by the Free Software Foundation.
**
** Photivo is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with Photivo.  If not, see <http://www.gnu.org/licenses/>.
**
*******************************************************************************
**
** Derived from wb_preset.c in ufraw with kind permission of Udi Fuchs.
**
*******************************************************************************/
// White balance presets for a wide variety of camera models.

#include "ptWhiteBalances.h"

#include <QObject>
#include <cstdlib>

// This has been autoconverted from wb_presets.c in ufraw.
// The 'tuning' stuff has not been adopted as there are imho
// tuning possibilities enough. And I want to 'kiss'.

// Also : Multipliers should be normalized, i.e. minimum value = 1.0.

const ptWhiteBalance ptWhiteBalances[] = {

// Four dummies to make sure that an index is larger than
// ptWhiteBalance_Camera/Auto/Manual/Spot
{ "","","",{0.0,0.0,0.0}},
{ "","","",{0.0,0.0,0.0}},
{ "","","",{0.0,0.0,0.0}},
{ "","","",{0.0,0.0,0.0}},

// Canon , PowerShot A630
{ "Canon",
  "PowerShot A630",
  QObject::tr("Daylight"),
  { 1.831422, 1, 1.245671, 0 } },
{ "Canon",
  "PowerShot A630",
  QObject::tr("Cloudy"),
  { 1.669924, 1, 1.326299, 0 } },
{ "Canon",
  "PowerShot A630",
  QObject::tr("Tungsten"),
  { 1.696768, 1, 1.268658, 0 } },
{ "Canon",
  "PowerShot A630",
  QObject::tr("Fluorescent"),
  { 1.869859, 1, 1.209110, 0 } },
{ "Canon",
  "PowerShot A630",
  QObject::tr("FluorescentHigh"),
  { 1.855491, 1, 1.206855, 0 } },

// Canon , PowerShot A710 IS
{ "Canon",
  "PowerShot A710 IS",
  QObject::tr("Daylight"),
  { 1.683007, 1, 1.893246, 0 } },
{ "Canon",
  "PowerShot A710 IS",
  QObject::tr("Cloudy"),
  { 1.871320, 1, 1.718648, 0 } },
{ "Canon",
  "PowerShot A710 IS",
  QObject::tr("Tungsten"),
  { 1.268692, 1, 2.707944, 0 } },
{ "Canon",
  "PowerShot A710 IS",
  QObject::tr("Fluorescent"),
  { 1.589857, 1, 2.051819, 0 } },
{ "Canon",
  "PowerShot A710 IS",
  QObject::tr("DaylightFluorescent"),
  { 1.820287, 1, 1.820287, 0 } },
{ "Canon",
  "PowerShot A710 IS",
  QObject::tr("Underwater"),
  { 2.926108, 1, 1.376847, 0 } },

// Canon , PowerShot A720 IS
{ "Canon",
  "PowerShot A720 IS",
  QObject::tr("Daylight"),
  { 2.059371, 1, 1.975553, 0 } },
{ "Canon",
  "PowerShot A720 IS",
  QObject::tr("Cloudy"),
  { 2.262722, 1, 1.818935, 0 } },
{ "Canon",
  "PowerShot A720 IS",
  QObject::tr("Tungsten"),
  { 1.632258, 1, 2.974194, 0 } },
{ "Canon",
  "PowerShot A720 IS",
  QObject::tr("Fluorescent"),
  { 1.902728, 1, 2.365362, 0 } },
{ "Canon",
  "PowerShot A720 IS",
  QObject::tr("FluorescentHigh"),
  { 2.243961, 1, 1.935990, 0 } },
{ "Canon",
  "PowerShot A720 IS",
  QObject::tr("Underwater"),
  { 2.303465, 1, 1.882915, 0 } },
{ "Canon",
  "PowerShot A720 IS",
  QObject::tr("Flash"),
  { 2.151586, 1, 1.864865, 0 } },

// Canon , PowerShot G2
{ "Canon",
  "PowerShot G2",
  QObject::tr("Daylight"),
  { 2.011483, 1, 1.299522, 0 } },
{ "Canon",
  "PowerShot G2",
  QObject::tr("Cloudy"),
  { 2.032505, 1, 1.285851, 0 } },
{ "Canon",
  "PowerShot G2",
  QObject::tr("Tungsten"),
  { 1.976008, 1, 1.332054, 0 } },
{ "Canon",
  "PowerShot G2",
  QObject::tr("Fluorescent"),
  { 2.022010, 1, 1.295694, 0 } },
{ "Canon",
  "PowerShot G2",
  QObject::tr("FluorescentHigh"),
  { 2.029637, 1, 1.286807, 0 } },
{ "Canon",
  "PowerShot G2",
  QObject::tr("Flash"),
  { 2.153576, 1, 1.140680, 0 } },

// Canon , PowerShot G3
{ "Canon",
  "PowerShot G3",
  QObject::tr("Daylight"),
  { 1.858513, 1, 1.387290, 0 } },
{ "Canon",
  "PowerShot G3",
  QObject::tr("Cloudy"),
  { 1.951132, 1, 1.305125, 0 } },
{ "Canon",
  "PowerShot G3",
  QObject::tr("Tungsten"),
  { 1.128386, 1, 2.313310, 0 } },
{ "Canon",
  "PowerShot G3",
  QObject::tr("Fluorescent"),
  { 1.715573, 1, 2.194337, 0 } },
{ "Canon",
  "PowerShot G3",
  QObject::tr("FluorescentHigh"),
  { 2.580563, 1, 1.496164, 0 } },
{ "Canon",
  "PowerShot G3",
  QObject::tr("Flash"),
  { 2.293173, 1, 1.187416, 0 } },

// Canon , PowerShot G5
{ "Canon",
  "PowerShot G5",
  QObject::tr("Daylight"),
  { 1.639521, 1, 1.528144, 0 } },
{ "Canon",
  "PowerShot G5",
  QObject::tr("Cloudy"),
  { 1.702153, 1, 1.462919, 0 } },
{ "Canon",
  "PowerShot G5",
  QObject::tr("Tungsten"),
  { 1.135071, 1, 2.374408, 0 } },
{ "Canon",
  "PowerShot G5",
  QObject::tr("Fluorescent"),
  { 1.660281, 1, 2.186462, 0 } },
{ "Canon",
  "PowerShot G5",
  QObject::tr("FluorescentHigh"),
  { 1.463297, 1, 1.764140, 0 } },
{ "Canon",
  "PowerShot G5",
  QObject::tr("Flash"),
  { 1.603593, 1, 1.562874, 0 } },

// Canon , PowerShot G6
{ "Canon",
  "PowerShot G6",
  QObject::tr("Daylight"),
  { 1.769704, 1, 1.637931, 0 } },
{ "Canon",
  "PowerShot G6",
  QObject::tr("Cloudy"),
  { 2.062731, 1, 1.442804, 0 } },
{ "Canon",
  "PowerShot G6",
  QObject::tr("Tungsten"),
  { 1.077106, 1, 2.721234, 0 } },
{ "Canon",
  "PowerShot G6",
  QObject::tr("Fluorescent"),
  { 1.914922, 1, 2.142670, 0 } },
{ "Canon",
  "PowerShot G6",
  QObject::tr("FluorescentHigh"),
  { 2.543677, 1, 1.650587, 0 } },
{ "Canon",
  "PowerShot G6",
  QObject::tr("Flash"),
  { 2.285322, 1, 1.333333, 0 } },

// Canon , PowerShot G9
{ "Canon",
  "PowerShot G9",
  QObject::tr("Daylight"),
  { 2.089552, 1, 1.786452, 0 } },
{ "Canon",
  "PowerShot G9",
  QObject::tr("Cloudy"),
  { 2.208716, 1, 1.660550, 0 } },
{ "Canon",
  "PowerShot G9",
  QObject::tr("Tungsten"),
  { 1.533493, 1, 2.586124, 0 } },
{ "Canon",
  "PowerShot G9",
  QObject::tr("Fluorescent"),
  { 2.065668, 1, 1.829493, 0 } },
{ "Canon",
  "PowerShot G9",
  QObject::tr("FluorescentHigh"),
  { 2.237601, 1, 1.668974, 0 } },
{ "Canon",
  "PowerShot G9",
  QObject::tr("Flash"),
  { 2.461538, 1, 1.498834, 0 } },
{ "Canon",
  "PowerShot G9",
  QObject::tr("Underwater"),
  { 2.237327, 1, 1.661290, 0 } },

// Canon , PowerShot G10
{ "Canon",
  "PowerShot G10",
  QObject::tr("Daylight"),
  { 1.598980, 1, 1.830612, 0 } },
{ "Canon",
  "PowerShot G10",
  QObject::tr("Cloudy"),
  { 1.738120, 1, 1.722281, 0 } },
{ "Canon",
  "PowerShot G10",
  QObject::tr("Tungsten"),
  { 1, 1.035550, 3.569954, 0 } },
{ "Canon",
  "PowerShot G10",
  QObject::tr("Fluorescent"),
  { 1.341633, 1, 2.434263, 0 } },
{ "Canon",
  "PowerShot G10",
  QObject::tr("FluorescentHigh"),
  { 1.749171, 1, 1.907182, 0 } },
{ "Canon",
  "PowerShot G10",
  QObject::tr("Flash"),
  { 1.926829, 1, 1.501591, 0 } },
{ "Canon",
  "PowerShot G10",
  QObject::tr("Underwater"),
  { 1.822314, 1, 1.841942, 0 } },

// Canon , PowerShot G11
{ "Canon",
  "PowerShot G11",
  QObject::tr("Daylight"),
  { 1.721591, 1, 2.097727, 0 } },
{ "Canon",
  "PowerShot G11",
  QObject::tr("Cloudy"),
  { 1.910936, 1, 1.856821, 0 } },
{ "Canon",
  "PowerShot G11",
  QObject::tr("Tungsten"),
  { 1.380435, 1, 3.576087, 0 } },
{ "Canon",
  "PowerShot G11",
  QObject::tr("Fluorescent"),
  { 1.649143, 1, 2.693714, 0 } },
{ "Canon",
  "PowerShot G11",
  QObject::tr("FluorescentHigh"),
  { 2.008168, 1, 1.961494, 0 } },
{ "Canon",
  "PowerShot G11",
  QObject::tr("Flash"),
  { 1.985556, 1, 1.703333, 0 } },
{ "Canon",
  "PowerShot G11",
  QObject::tr("Underwater"),
  { 2.225624, 1, 1.577098, 0 } },

// Canon , PowerShot G12
{ "Canon",
  "PowerShot G12",
  QObject::tr("Daylight"),
  { 1.785877, 1, 2.042141, 0 } },
{ "Canon",
  "PowerShot G12",
  QObject::tr("Cloudy"),
  { 1.804323, 1, 2.021615, 0 } },
{ "Canon",
  "PowerShot G12",
  QObject::tr("Tungsten"),
  { 1.310127, 1, 3.170886, 0 } },
{ "Canon",
  "PowerShot G12",
  QObject::tr("Fluorescent"),
  { 1.771139, 1, 2.064262, 0 } },
{ "Canon",
  "PowerShot G12",
  QObject::tr("FluorescentHigh"),
  { 1.806122, 1, 2.032880, 0 } },
{ "Canon",
  "PowerShot G12",
  QObject::tr("Flash"),
  { 2.102157, 1, 1.706016, 0 } },
{ "Canon",
  "PowerShot G12",
  QObject::tr("Underwater"),
  { 1.807650, 1, 2.112568, 0 } },

// Canon , PowerShot S3 IS
{ "Canon",
  "PowerShot S3 IS",
  QObject::tr("Daylight"),
  { 1.627271, 1, 1.823491, 0 } },
{ "Canon",
  "PowerShot S3 IS",
  QObject::tr("Cloudy"),
  { 1.794382, 1, 1.618412, 0 } },
{ "Canon",
  "PowerShot S3 IS",
  QObject::tr("Tungsten"),
  { 1, 1.192243, 4.546950, 0 } },
{ "Canon",
  "PowerShot S3 IS",
  QObject::tr("Flash"),
  { 1.884691, 1, 1.553869, 0 } },

// Canon , PowerShot S30
{ "Canon",
  "PowerShot S30",
  QObject::tr("Daylight"),
  { 1.741088, 1, 1.318949, 0 } },
{ "Canon",
  "PowerShot S30",
  QObject::tr("Cloudy"),
  { 1.766635, 1, 1.298969, 0 } },
{ "Canon",
  "PowerShot S30",
  QObject::tr("Tungsten"),
  { 1.498106, 1, 1.576705, 0 } },
{ "Canon",
  "PowerShot S30",
  QObject::tr("Fluorescent"),
  { 1.660075, 1, 1.394539, 0 } },
{ "Canon",
  "PowerShot S30",
  QObject::tr("FluorescentHigh"),
  { 1.753515, 1, 1.306467, 0 } },
{ "Canon",
  "PowerShot S30",
  QObject::tr("Flash"),
  { 2.141705, 1, 1.097926, 0 } },

// Canon , PowerShot S45
{ "Canon",
  "PowerShot S45",
  QObject::tr("Daylight"),
  { 2.325175, 1, 1.080420, 0 } },
{ "Canon",
  "PowerShot S45",
  QObject::tr("Cloudy"),
  { 2.145047, 1, 1.173349, 0 } },
{ "Canon",
  "PowerShot S45",
  QObject::tr("Tungsten"),
  { 1.213018, 1, 2.087574, 0 } },
{ "Canon",
  "PowerShot S45",
  QObject::tr("Fluorescent"),
  { 1.888183, 1, 1.822109, 0 } },
{ "Canon",
  "PowerShot S45",
  QObject::tr("FluorescentHigh"),
  { 2.964422, 1, 1.354511, 0 } },
{ "Canon",
  "PowerShot S45",
  QObject::tr("Flash"),
  { 2.534884, 1, 1.065663, 0 } },

// Canon , PowerShot S50
{ "Canon",
  "PowerShot S50",
  QObject::tr("Daylight"),
  { 1.772506, 1, 1.536496, 0 } },
{ "Canon",
  "PowerShot S50",
  QObject::tr("Cloudy"),
  { 1.831311, 1, 1.484223, 0 } },
{ "Canon",
  "PowerShot S50",
  QObject::tr("Tungsten"),
  { 1.185542, 1, 2.480723, 0 } },
{ "Canon",
  "PowerShot S50",
  QObject::tr("Fluorescent"),
  { 1.706410, 1, 2.160256, 0 } },
{ "Canon",
  "PowerShot S50",
  QObject::tr("FluorescentHigh"),
  { 1.562500, 1, 1.817402, 0 } },
{ "Canon",
  "PowerShot S50",
  QObject::tr("Flash"),
  { 1.776156, 1, 1.531630, 0 } },

// Canon , PowerShot S60
{ "Canon",
  "PowerShot S60",
  QObject::tr("Daylight"),
  { 1.759169, 1, 1.590465, 0 } },
{ "Canon",
  "PowerShot S60",
  QObject::tr("Cloudy"),
  { 1.903659, 1, 1.467073, 0 } },
{ "Canon",
  "PowerShot S60",
  QObject::tr("Tungsten"),
  { 1.138554, 1, 2.704819, 0 } },
{ "Canon",
  "PowerShot S60",
  QObject::tr("Fluorescent"),
  { 1.720721, 1, 2.185328, 0 } },
{ "Canon",
  "PowerShot S60",
  QObject::tr("FluorescentHigh"),
  { 2.877095, 1, 2.216480, 0 } },
{ "Canon",
  "PowerShot S60",
  QObject::tr("Flash"),
  { 2.182540, 1, 1.236773, 0 } },
{ "Canon",
  "PowerShot S60",
  QObject::tr("Underwater"),
  { 2.725369, 1, 1.240148, 0 } },

// Canon , PowerShot S70
{ "Canon",
  "PowerShot S70",
  QObject::tr("Daylight"),
  { 1.943834, 1, 1.456654, 0 } },
{ "Canon",
  "PowerShot S70",
  QObject::tr("Cloudy"),
  { 2.049939, 1, 1.382460, 0 } },
{ "Canon",
  "PowerShot S70",
  QObject::tr("Tungsten"),
  { 1.169492, 1, 2.654964, 0 } },
{ "Canon",
  "PowerShot S70",
  QObject::tr("Fluorescent"),
  { 1.993456, 1, 2.056283, 0 } },
{ "Canon",
  "PowerShot S70",
  QObject::tr("FluorescentHigh"),
  { 2.645914, 1, 1.565499, 0 } },
{ "Canon",
  "PowerShot S70",
  QObject::tr("Flash"),
  { 2.389189, 1, 1.147297, 0 } },
{ "Canon",
  "PowerShot S70",
  QObject::tr("Underwater"),
  { 3.110565, 1, 1.162162, 0 } },

// Canon , PowerShot S90
{ "Canon",
  "PowerShot S90",
  QObject::tr("Daylight"),
  { 1.955056, 1, 1.797753, 0 } },
{ "Canon",
  "PowerShot S90",
  QObject::tr("Cloudy"),
  { 1.945067, 1, 1.795964, 0 } },
{ "Canon",
  "PowerShot S90",
  QObject::tr("Tungsten"),
  { 2.000000, 1, 1.828018, 0 } },
{ "Canon",
  "PowerShot S90",
  QObject::tr("Fluorescent"),
  { 2.019473, 1, 1.841924, 0 } },
{ "Canon",
  "PowerShot S90",
  QObject::tr("FluorescentHigh"),
  { 2.009143, 1, 1.840000, 0 } },
{ "Canon",
  "PowerShot S90",
  QObject::tr("Flash"),
  { 2.045784, 1, 1.671692, 0 } },
{ "Canon",
  "PowerShot S90",
  QObject::tr("Underwater"),
  { 2.022297, 1, 1.830546, 0 } },

// Canon , PowerShot S95
{ "Canon",
  "PowerShot S95",
  QObject::tr("Daylight"),
  { 1.6182, 1, 2.1430, 0 } },
{ "Canon",
  "PowerShot S95",
  QObject::tr("Cloudy"),
  { 1.6920, 1, 2.0179, 0 } },
{ "Canon",
  "PowerShot S95",
  QObject::tr("Tungsten"),
  { 1.2314, 1, 3.1015, 0 } },
{ "Canon",
  "PowerShot S95",
  QObject::tr("Fluorescent"),
  { 1.6593, 1, 2.0940, 0 } },
{ "Canon",
  "PowerShot S95",
  QObject::tr("FluorescentHigh"),
  { 1.7272, 1, 1.9811, 0 } },
{ "Canon",
  "PowerShot S95",
  QObject::tr("Flash"),
  { 1.9955, 1, 1.7768, 0 } },
{ "Canon",
  "PowerShot S95",
  QObject::tr("Underwater"),
  { 1.7607, 1, 2.1224, 0 } },

// Canon , PowerShot S100
{ "Canon",
  "PowerShot S100",
  QObject::tr("Daylight"),
  { 2.077707, 1, 2.551592, 0 } },
{ "Canon",
  "PowerShot S100",
  QObject::tr("Cloudy"),
  { 2.276402, 1, 2.393742, 0 } },
{ "Canon",
  "PowerShot S100",
  QObject::tr("Tungsten"),
  { 1.267936, 1, 4.224012, 0 } },
{ "Canon",
  "PowerShot S100",
  QObject::tr("Fluorescent"),
  { 1.815115, 1, 3.093117, 0 } },
{ "Canon",
  "PowerShot S100",
  QObject::tr("FluorescentHigh"),
  { 2.398148, 1, 2.374339, 0 } },
{ "Canon",
  "PowerShot S100",
  QObject::tr("Flash"),
  { 2.615783, 1, 2.001294, 0 } },
{ "Canon",
  "PowerShot S100",
  QObject::tr("Underwater"),
  { 2.248391, 1, 2.338481, 0 } },

// Canon , PowerShot Pro1
{ "Canon",
  "PowerShot Pro1",
  QObject::tr("Daylight"),
  { 1.829238, 1, 1.571253, 0 } },
{ "Canon",
  "PowerShot Pro1",
  QObject::tr("Cloudy"),
  { 1.194139, 1, 2.755800, 0 } },
{ "Canon",
  "PowerShot Pro1",
  QObject::tr("Tungsten"),
  { 1.701416, 1, 2.218790, 0 } },
{ "Canon",
  "PowerShot Pro1",
  QObject::tr("Fluorescent"),
  { 2.014066, 1, 1.776215, 0 } },
{ "Canon",
  "PowerShot Pro1",
  QObject::tr("FluorescentHigh"),
  { 2.248663, 1, 1.227273, 0 } },
{ "Canon",
  "PowerShot Pro1",
  QObject::tr("Flash"),
  { 2.130081, 1, 1.422764, 0 } },

// Canon , PowerShot SX1 IS
{ "Canon",
  "PowerShot SX1 IS",
  QObject::tr("Daylight"),
  { 1.574586, 1, 2.114917, 0 } },
{ "Canon",
  "PowerShot SX1 IS",
  QObject::tr("Cloudy"),
  { 1.682628, 1, 2.015590, 0 } },
{ "Canon",
  "PowerShot SX1 IS",
  QObject::tr("Tungsten"),
  { 1.088836, 1, 3.056423, 0 } },
{ "Canon",
  "PowerShot SX1 IS",
  QObject::tr("Fluorescent"),
  { 1.398259, 1, 2.414581, 0 } },
{ "Canon",
  "PowerShot SX1 IS",
  QObject::tr("FluorescentHigh"),
  { 1.687500, 1, 2.025670, 0 } },
{ "Canon",
  "PowerShot SX1 IS",
  QObject::tr("Flash"),
  { 1.909699, 1, 1.795987, 0 } },

// Canon , EOS D60
{ "Canon",
  "EOS D60",
  QObject::tr("Daylight"),
  { 2.472594, 1, 1.225335, 0 } },
{ "Canon",
  "EOS D60",
  QObject::tr("Cloudy"),
  { 2.723926, 1, 1.137423, 0 } },
{ "Canon",
  "EOS D60",
  QObject::tr("Tungsten"),
  { 1.543054, 1, 1.907003, 0 } },
{ "Canon",
  "EOS D60",
  QObject::tr("Fluorescent"),
  { 1.957346, 1, 1.662322, 0 } },
{ "Canon",
  "EOS D60",
  QObject::tr("Flash"),
  { 2.829840, 1, 1.108508, 0 } },

// Canon , EOS 5D
{ "Canon",
  "EOS 5D",
  QObject::tr("Flash"),
  { 2.211914, 1, 1.260742, 0 } },
{ "Canon",
  "EOS 5D",
  QObject::tr("Fluorescent"),
  { 1.726054, 1, 2.088123, 0 } },
{ "Canon",
  "EOS 5D",
  QObject::tr("Tungsten"),
  { 1.373285, 1, 2.301006, 0 } },
{ "Canon",
  "EOS 5D",
  QObject::tr("Cloudy"),
  { 2.151367, 1, 1.321289, 0 } },
{ "Canon",
  "EOS 5D",
  QObject::tr("Shade"),
  { 2.300781, 1, 1.208008, 0 } },
{ "Canon",
  "EOS 5D",
  QObject::tr("Daylight"),
  { 1.988281, 1, 1.457031, 0 } },

// Canon , EOS 5D Mark II
{ "Canon",
  "EOS 5D Mark II",
  QObject::tr("Daylight"),
  { 2.216797, 1, 1.689453, 0 } },
{ "Canon",
  "EOS 5D Mark II",
  QObject::tr("Shade"),
  { 2.559570, 1, 1.397461, 0 } },
{ "Canon",
  "EOS 5D Mark II",
  QObject::tr("Cloudy"),
  { 2.392578, 1, 1.523438, 0 } },
{ "Canon",
  "EOS 5D Mark II",
  QObject::tr("Tungsten"),
  { 1.573115, 1, 2.646685, 0 } },
{ "Canon",
  "EOS 5D Mark II",
  QObject::tr("Fluorescent"),
  { 1.895833, 1, 2.461174, 0 } },
{ "Canon",
  "EOS 5D Mark II",
  QObject::tr("Flash"),
  { 2.409180, 1, 1.507813, 0 } },

// Canon , EOS 5D Mark III
{ "Canon",
  "EOS 5D Mark III",
  QObject::tr("Daylight"),
  { 2.060547, 1, 1.618164, 0 } },
{ "Canon",
  "EOS 5D Mark III",
  QObject::tr("Shade"),
  { 2.365234, 1, 1.381836, 0 } },
{ "Canon",
  "EOS 5D Mark III",
  QObject::tr("Cloudy"),
  { 2.216797, 1, 1.490234, 0 } },
{ "Canon",
  "EOS 5D Mark III",
  QObject::tr("Tungsten"),
  { 1.484375, 1, 2.398438, 0 } },
{ "Canon",
  "EOS 5D Mark III",
  QObject::tr("Fluorescent"),
  { 1.809570, 1, 2.300781, 0 } },
{ "Canon",
  "EOS 5D Mark III",
  QObject::tr("Flash"),
  { 2.291016, 1, 1.464844, 0 } },

// Canon , EOS 7D
{ "Canon",
  "EOS 7D",
  QObject::tr("Daylight"),
  { 2.093750, 1, 1.391602, 0 } },
{ "Canon",
  "EOS 7D",
  QObject::tr("Shade"),
  { 2.415039, 1, 1.178711, 0 } },
{ "Canon",
  "EOS 7D",
  QObject::tr("Cloudy"),
  { 2.250977, 1, 1.276367, 0 } },
{ "Canon",
  "EOS 7D",
  QObject::tr("Tungsten"),
  { 1.484375, 1, 2.093750, 0 } },
{ "Canon",
  "EOS 7D",
  QObject::tr("WhiteFluorescent"),
  { 1.868164, 1, 1.965820, 0 } },
{ "Canon",
  "EOS 7D",
  QObject::tr("Flash"),
  { 2.300781, 1, 1.270508, 0 } },

// Canon , EOS 10D
{ "Canon",
  "EOS 10D",
  QObject::tr("Daylight"),
  { 2.159856, 1, 1.218750, 0 } },
{ "Canon",
  "EOS 10D",
  QObject::tr("Shade"),
  { 2.533654, 1, 1.036058, 0 } },
{ "Canon",
  "EOS 10D",
  QObject::tr("Cloudy"),
  { 2.348558, 1, 1.116587, 0 } },
{ "Canon",
  "EOS 10D",
  QObject::tr("Tungsten"),
  { 1.431544, 1, 1.851040, 0 } },
{ "Canon",
  "EOS 10D",
  QObject::tr("Fluorescent"),
  { 1.891509, 1, 1.647406, 0 } },
{ "Canon",
  "EOS 10D",
  QObject::tr("Flash"),
  { 2.385817, 1, 1.115385, 0 } },

// Canon , EOS 20D
{ "Canon",
  "EOS 20D",
  QObject::tr("Daylight"),
  { 1.954680, 1, 1.478818, 0 } },
{ "Canon",
  "EOS 20D",
  QObject::tr("Shade"),
  { 2.248276, 1, 1.227586, 0 } },
{ "Canon",
  "EOS 20D",
  QObject::tr("Cloudy"),
  { 2.115271, 1, 1.336946, 0 } },
{ "Canon",
  "EOS 20D",
  QObject::tr("Tungsten"),
  { 1.368087, 1, 2.417044, 0 } },
{ "Canon",
  "EOS 20D",
  QObject::tr("Fluorescent"),
  { 1.752709, 1, 2.060098, 0 } },
{ "Canon",
  "EOS 20D",
  QObject::tr("Flash"),
  { 2.145813, 1, 1.293596, 0 } },

// Canon , EOS 30D
{ "Canon",
  "EOS 30D",
  QObject::tr("Daylight"),
  { 2.032227, 1, 1.537109, 0 } },
{ "Canon",
  "EOS 30D",
  QObject::tr("Shade"),
  { 2.354492, 1, 1.264648, 0 } },
{ "Canon",
  "EOS 30D",
  QObject::tr("Cloudy"),
  { 2.197266, 1, 1.389648, 0 } },
{ "Canon",
  "EOS 30D",
  QObject::tr("Tungsten"),
  { 1.411084, 1, 2.447477, 0 } },
{ "Canon",
  "EOS 30D",
  QObject::tr("Fluorescent"),
  { 1.761601, 1, 2.303913, 0 } },
{ "Canon",
  "EOS 30D",
  QObject::tr("Flash"),
  { 2.226562, 1, 1.347656, 0 } },

// Canon , EOS 40D
{ "Canon",
  "EOS 40D",
  QObject::tr("Daylight"),
  { 2.197266, 1, 1.438477, 0 } },
{ "Canon",
  "EOS 40D",
  QObject::tr("Shade"),
  { 2.546875, 1, 1.185547, 0 } },
{ "Canon",
  "EOS 40D",
  QObject::tr("Cloudy"),
  { 2.370117, 1, 1.290039, 0 } },
{ "Canon",
  "EOS 40D",
  QObject::tr("Tungsten"),
  { 1.510563, 1, 2.235915, 0 } },
{ "Canon",
  "EOS 40D",
  QObject::tr("Fluorescent"),
  { 2.019084, 1, 2.129771, 0 } },
{ "Canon",
  "EOS 40D",
  QObject::tr("Flash"),
  { 2.409180, 1, 1.260742, 0 } },

// Canon , EOS 50D
{ "Canon",
  "EOS 50D",
  QObject::tr("Daylight"),
  { 2.098633, 1, 1.402344, 0 } },
{ "Canon",
  "EOS 50D",
  QObject::tr("Shade"),
  { 2.386719, 1, 1.196289, 0 } },
{ "Canon",
  "EOS 50D",
  QObject::tr("Cloudy"),
  { 2.240234, 1, 1.285156, 0 } },
{ "Canon",
  "EOS 50D",
  QObject::tr("Tungsten"),
  { 1.542466, 1, 1.976256, 0 } },
{ "Canon",
  "EOS 50D",
  QObject::tr("WhiteFluorescent"),
  { 1.882805, 1, 1.876081, 0 } },
{ "Canon",
  "EOS 50D",
  QObject::tr("Flash"),
  { 2.275391, 1, 1.268555, 0 } },
{ "Canon",
  "EOS 50D",
  QObject::tr("5000K"),
  { 2.056641, 1, 1.438477, 0 } },
{ "Canon",
  "EOS 50D",
  QObject::tr("6500K"),
  { 2.311523, 1, 1.239258, 0 } },

// Canon , EOS 60D
{ "Canon",
  "EOS 60D",
  QObject::tr("Daylight"),
  { 2.1514, 1, 1.5420, 0 } },
{ "Canon",
  "EOS 60D",
  QObject::tr("Shade"),
  { 2.5039, 1, 1.3057, 0 } },
{ "Canon",
  "EOS 60D",
  QObject::tr("Cloudy"),
  { 2.3223, 1, 1.4160, 0 } },
{ "Canon",
  "EOS 60D",
  QObject::tr("Tungsten"),
  { 1.5215, 1, 2.3486, 0 } },
{ "Canon",
  "EOS 60D",
  QObject::tr("Fluorescent"),
  { 1.9248, 1, 2.1836, 0 } },
{ "Canon",
  "EOS 60D",
  QObject::tr("Flash"),
  { 2.1514, 1, 1.5420, 0 } },

// Canon , EOS 300D DIGITAL
{ "Canon",
  "EOS 300D DIGITAL",
  QObject::tr("Daylight"),
  { 2.072115, 1, 1.217548, 0 } },
{ "Canon",
  "EOS 300D DIGITAL",
  QObject::tr("Shade"),
  { 2.455529, 1, 1.026442, 0 } },
{ "Canon",
  "EOS 300D DIGITAL",
  QObject::tr("Cloudy"),
  { 2.254808, 1, 1.108173, 0 } },
{ "Canon",
  "EOS 300D DIGITAL",
  QObject::tr("Tungsten"),
  { 1.349057, 1, 1.896226, 0 } },
{ "Canon",
  "EOS 300D DIGITAL",
  QObject::tr("Fluorescent"),
  { 1.794664, 1, 1.711137, 0 } },
{ "Canon",
  "EOS 300D DIGITAL",
  QObject::tr("Flash"),
  { 2.326923, 1, 1.098558, 0 } },

// Canon , EOS DIGITAL REBEL
{ "Canon",
  "EOS DIGITAL REBEL",
  QObject::tr("Daylight"),
  { 2.072115, 1, 1.217548, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL",
  QObject::tr("Shade"),
  { 2.455529, 1, 1.026442, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL",
  QObject::tr("Cloudy"),
  { 2.254808, 1, 1.108173, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL",
  QObject::tr("Tungsten"),
  { 1.349057, 1, 1.896226, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL",
  QObject::tr("Fluorescent"),
  { 1.794664, 1, 1.711137, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL",
  QObject::tr("Flash"),
  { 2.326923, 1, 1.098558, 0 } },

// Canon , EOS Kiss Digital
{ "Canon",
  "EOS Kiss Digital",
  QObject::tr("Daylight"),
  { 2.072115, 1, 1.217548, 0 } },
{ "Canon",
  "EOS Kiss Digital",
  QObject::tr("Shade"),
  { 2.455529, 1, 1.026442, 0 } },
{ "Canon",
  "EOS Kiss Digital",
  QObject::tr("Cloudy"),
  { 2.254808, 1, 1.108173, 0 } },
{ "Canon",
  "EOS Kiss Digital",
  QObject::tr("Tungsten"),
  { 1.349057, 1, 1.896226, 0 } },
{ "Canon",
  "EOS Kiss Digital",
  QObject::tr("Fluorescent"),
  { 1.794664, 1, 1.711137, 0 } },
{ "Canon",
  "EOS Kiss Digital",
  QObject::tr("Flash"),
  { 2.326923, 1, 1.098558, 0 } },

// Canon , EOS 350D DIGITAL
{ "Canon",
  "EOS 350D DIGITAL",
  QObject::tr("Daylight"),
  { 2.3605, 1, 1.4450, 0 } },
{ "Canon",
  "EOS 350D DIGITAL",
  QObject::tr("Shade"),
  { 2.7888, 1, 1.2024, 0 } },
{ "Canon",
  "EOS 350D DIGITAL",
  QObject::tr("Cloudy"),
  { 2.5727, 1, 1.3075, 0 } },
{ "Canon",
  "EOS 350D DIGITAL",
  QObject::tr("Tungsten"),
  { 1.5343, 1, 2.2880, 0 } },
{ "Canon",
  "EOS 350D DIGITAL",
  QObject::tr("Fluorescent"),
  { 1.9754, 1, 1.9303, 0 } },
{ "Canon",
  "EOS 350D DIGITAL",
  QObject::tr("Flash"),
  { 2.6729, 1, 1.2613, 0 } },

// Canon , EOS DIGITAL REBEL XT
{ "Canon",
  "EOS DIGITAL REBEL XT",
  QObject::tr("Daylight"),
  { 2.3605, 1, 1.4450, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XT",
  QObject::tr("Shade"),
  { 2.7888, 1, 1.2024, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XT",
  QObject::tr("Cloudy"),
  { 2.5727, 1, 1.3075, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XT",
  QObject::tr("Tungsten"),
  { 1.5343, 1, 2.2880, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XT",
  QObject::tr("Fluorescent"),
  { 1.9754, 1, 1.9303, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XT",
  QObject::tr("Flash"),
  { 2.6729, 1, 1.2613, 0 } },

// Canon , EOS Kiss Digital N
{ "Canon",
  "EOS Kiss Digital N",
  QObject::tr("Daylight"),
  { 2.3605, 1, 1.4450, 0 } },
{ "Canon",
  "EOS Kiss Digital N",
  QObject::tr("Shade"),
  { 2.7888, 1, 1.2024, 0 } },
{ "Canon",
  "EOS Kiss Digital N",
  QObject::tr("Cloudy"),
  { 2.5727, 1, 1.3075, 0 } },
{ "Canon",
  "EOS Kiss Digital N",
  QObject::tr("Tungsten"),
  { 1.5343, 1, 2.2880, 0 } },
{ "Canon",
  "EOS Kiss Digital N",
  QObject::tr("Fluorescent"),
  { 1.9754, 1, 1.9303, 0 } },
{ "Canon",
  "EOS Kiss Digital N",
  QObject::tr("Flash"),
  { 2.6729, 1, 1.2613, 0 } },

// Canon , EOS 400D DIGITAL
{ "Canon",
  "EOS 400D DIGITAL",
  QObject::tr("Daylight"),
  { 2.300781, 1, 1.462891, 0 } },
{ "Canon",
  "EOS 400D DIGITAL",
  QObject::tr("Shade"),
  { 2.708984, 1, 1.208984, 0 } },
{ "Canon",
  "EOS 400D DIGITAL",
  QObject::tr("Cloudy"),
  { 2.509766, 1, 1.323242, 0 } },
{ "Canon",
  "EOS 400D DIGITAL",
  QObject::tr("Tungsten"),
  { 1.526123, 1, 2.330889, 0 } },
{ "Canon",
  "EOS 400D DIGITAL",
  QObject::tr("WhiteFluorescent"),
  { 1.910506, 1, 2.080739, 0 } },
{ "Canon",
  "EOS 400D DIGITAL",
  QObject::tr("Flash"),
  { 2.767578, 1, 1.203125, 0 } },

// Canon , EOS DIGITAL REBEL XTi
{ "Canon",
  "EOS DIGITAL REBEL XTi",
  QObject::tr("Daylight"),
  { 2.300781, 1, 1.462891, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XTi",
  QObject::tr("Shade"),
  { 2.708984, 1, 1.208984, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XTi",
  QObject::tr("Cloudy"),
  { 2.509766, 1, 1.323242, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XTi",
  QObject::tr("Tungsten"),
  { 1.526123, 1, 2.330889, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XTi",
  QObject::tr("WhiteFluorescent"),
  { 1.910506, 1, 2.080739, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XTi",
  QObject::tr("Flash"),
  { 2.767578, 1, 1.203125, 0 } },

// Canon , EOS Kiss Digital X
{ "Canon",
  "EOS Kiss Digital X",
  QObject::tr("Daylight"),
  { 2.300781, 1, 1.462891, 0 } },
{ "Canon",
  "EOS Kiss Digital X",
  QObject::tr("Shade"),
  { 2.708984, 1, 1.208984, 0 } },
{ "Canon",
  "EOS Kiss Digital X",
  QObject::tr("Cloudy"),
  { 2.509766, 1, 1.323242, 0 } },
{ "Canon",
  "EOS Kiss Digital X",
  QObject::tr("Tungsten"),
  { 1.526123, 1, 2.330889, 0 } },
{ "Canon",
  "EOS Kiss Digital X",
  QObject::tr("WhiteFluorescent"),
  { 1.910506, 1, 2.080739, 0 } },
{ "Canon",
  "EOS Kiss Digital X",
  QObject::tr("Flash"),
  { 2.767578, 1, 1.203125, 0 } },

// Canon , EOS 450D
{ "Canon",
  "EOS 450D",
  QObject::tr("Daylight"),
  { 2.216797, 1, 1.471680, 0 } },
{ "Canon",
  "EOS 450D",
  QObject::tr("Shade"),
  { 2.566406, 1, 1.241211, 0 } },
{ "Canon",
  "EOS 450D",
  QObject::tr("Cloudy"),
  { 2.386719, 1, 1.345703, 0 } },
{ "Canon",
  "EOS 450D",
  QObject::tr("Tungsten"),
  { 1.559034, 1, 2.170841, 0 } },
{ "Canon",
  "EOS 450D",
  QObject::tr("Fluorescent"),
  { 1.922857, 1, 1.996190, 0 } },
{ "Canon",
  "EOS 450D",
  QObject::tr("Flash"),
  { 2.456055, 1, 1.318359, 0 } },

// Canon , EOS DIGITAL REBEL XSi
{ "Canon",
  "EOS DIGITAL REBEL XSi",
  QObject::tr("Daylight"),
  { 2.216797, 1, 1.471680, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XSi",
  QObject::tr("Shade"),
  { 2.566406, 1, 1.241211, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XSi",
  QObject::tr("Cloudy"),
  { 2.386719, 1, 1.345703, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XSi",
  QObject::tr("Tungsten"),
  { 1.559034, 1, 2.170841, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XSi",
  QObject::tr("Fluorescent"),
  { 1.922857, 1, 1.996190, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XSi",
  QObject::tr("Flash"),
  { 2.456055, 1, 1.318359, 0 } },

// Canon , EOS Kiss Digital X2
{ "Canon",
  "EOS Kiss Digital X2",
  QObject::tr("Daylight"),
  { 2.216797, 1, 1.471680, 0 } },
{ "Canon",
  "EOS Kiss Digital X2",
  QObject::tr("Shade"),
  { 2.566406, 1, 1.241211, 0 } },
{ "Canon",
  "EOS Kiss Digital X2",
  QObject::tr("Cloudy"),
  { 2.386719, 1, 1.345703, 0 } },
{ "Canon",
  "EOS Kiss Digital X2",
  QObject::tr("Tungsten"),
  { 1.559034, 1, 2.170841, 0 } },
{ "Canon",
  "EOS Kiss Digital X2",
  QObject::tr("Fluorescent"),
  { 1.922857, 1, 1.996190, 0 } },
{ "Canon",
  "EOS Kiss Digital X2",
  QObject::tr("Flash"),
  { 2.456055, 1, 1.318359, 0 } },

// Canon , EOS 500D
{ "Canon",
  "EOS 500D",
  QObject::tr("Daylight"),
  { 2.023438, 1, 1.417969, 0 } },
{ "Canon",
  "EOS 500D",
  QObject::tr("Shade"),
  { 2.291016, 1, 1.217773, 0 } },
{ "Canon",
  "EOS 500D",
  QObject::tr("Cloudy"),
  { 2.156250, 1, 1.304687, 0 } },
{ "Canon",
  "EOS 500D",
  QObject::tr("Tungsten"),
  { 1.481347, 1, 1.976342, 0 } },
{ "Canon",
  "EOS 500D",
  QObject::tr("Fluorescent"),
  { 1.799224, 1, 1.824442, 0 } },
{ "Canon",
  "EOS 500D",
  QObject::tr("Flash"),
  { 2.207031, 1, 1.295898, 0 } },

// Canon , EOS REBEL T1i
{ "Canon",
  "EOS REBEL T1i",
  QObject::tr("Daylight"),
  { 2.023438, 1, 1.417969, 0 } },
{ "Canon",
  "EOS REBEL T1i",
  QObject::tr("Shade"),
  { 2.291016, 1, 1.217773, 0 } },
{ "Canon",
  "EOS REBEL T1i",
  QObject::tr("Cloudy"),
  { 2.156250, 1, 1.304687, 0 } },
{ "Canon",
  "EOS REBEL T1i",
  QObject::tr("Tungsten"),
  { 1.481347, 1, 1.976342, 0 } },
{ "Canon",
  "EOS REBEL T1i",
  QObject::tr("Fluorescent"),
  { 1.799224, 1, 1.824442, 0 } },
{ "Canon",
  "EOS REBEL T1i",
  QObject::tr("Flash"),
  { 2.207031, 1, 1.295898, 0 } },

// Canon , EOS Kiss Digital X3
{ "Canon",
  "EOS Kiss Digital X3",
  QObject::tr("Daylight"),
  { 2.023438, 1, 1.417969, 0 } },
{ "Canon",
  "EOS Kiss Digital X3",
  QObject::tr("Shade"),
  { 2.291016, 1, 1.217773, 0 } },
{ "Canon",
  "EOS Kiss Digital X3",
  QObject::tr("Cloudy"),
  { 2.156250, 1, 1.304687, 0 } },
{ "Canon",
  "EOS Kiss Digital X3",
  QObject::tr("Tungsten"),
  { 1.481347, 1, 1.976342, 0 } },
{ "Canon",
  "EOS Kiss Digital X3",
  QObject::tr("Fluorescent"),
  { 1.799224, 1, 1.824442, 0 } },
{ "Canon",
  "EOS Kiss Digital X3",
  QObject::tr("Flash"),
  { 2.207031, 1, 1.295898, 0 } },

// Canon , EOS 550D
{ "Canon",
  "EOS 550D",
  QObject::tr("Daylight"),
  { 2.1426, 1, 1.5488, 0 } },
{ "Canon",
  "EOS 550D",
  QObject::tr("Shade"),
  { 2.4619, 1, 1.3193, 0 } },
{ "Canon",
  "EOS 550D",
  QObject::tr("Cloudy"),
  { 2.3066, 1, 1.4258, 0 } },
{ "Canon",
  "EOS 550D",
  QObject::tr("Tungsten"),
  { 1.5264, 1, 2.3428, 0 } },
{ "Canon",
  "EOS 550D",
  QObject::tr("WhiteFluorescent"),
  { 1.9072, 1, 2.1973, 0 } },
{ "Canon",
  "EOS 550D",
  QObject::tr("Flash"),
  { 2.3701, 1, 1.4141, 0 } },

// Canon , EOS REBEL T2i
{ "Canon",
  "EOS REBEL T2i",
  QObject::tr("Daylight"),
  { 2.1426, 1, 1.5488, 0 } },
{ "Canon",
  "EOS REBEL T2i",
  QObject::tr("Shade"),
  { 2.4619, 1, 1.3193, 0 } },
{ "Canon",
  "EOS REBEL T2i",
  QObject::tr("Cloudy"),
  { 2.3066, 1, 1.4258, 0 } },
{ "Canon",
  "EOS REBEL T2i",
  QObject::tr("Tungsten"),
  { 1.5264, 1, 2.3428, 0 } },
{ "Canon",
  "EOS REBEL T2i",
  QObject::tr("WhiteFluorescent"),
  { 1.9072, 1, 2.1973, 0 } },
{ "Canon",
  "EOS REBEL T2i",
  QObject::tr("Flash"),
  { 2.3701, 1, 1.4141, 0 } },

// Canon , EOS Kiss Digital X4
{ "Canon",
  "EOS Kiss Digital X4",
  QObject::tr("Daylight"),
  { 2.1426, 1, 1.5488, 0 } },
{ "Canon",
  "EOS Kiss Digital X4",
  QObject::tr("Shade"),
  { 2.4619, 1, 1.3193, 0 } },
{ "Canon",
  "EOS Kiss Digital X4",
  QObject::tr("Cloudy"),
  { 2.3066, 1, 1.4258, 0 } },
{ "Canon",
  "EOS Kiss Digital X4",
  QObject::tr("Tungsten"),
  { 1.5264, 1, 2.3428, 0 } },
{ "Canon",
  "EOS Kiss Digital X4",
  QObject::tr("WhiteFluorescent"),
  { 1.9072, 1, 2.1973, 0 } },
{ "Canon",
  "EOS Kiss Digital X4",
  QObject::tr("Flash"),
  { 2.3701, 1, 1.4141, 0 } },

// Canon , EOS 600D
{ "Canon",
  "EOS 600D",
  QObject::tr("Daylight"),
  { 2.1602, 1, 1.5752, 0 } },
{ "Canon",
  "EOS 600D",
  QObject::tr("Shade"),
  { 2.5098, 1, 1.3418, 0 } },
{ "Canon",
  "EOS 600D",
  QObject::tr("Cloudy"),
  { 2.3330, 1, 1.4502, 0 } },
{ "Canon",
  "EOS 600D",
  QObject::tr("Tungsten"),
  { 1.5371, 1, 2.3594, 0 } },
{ "Canon",
  "EOS 600D",
  QObject::tr("WhiteFluorescent"),
  { 1.8789, 1, 2.2402, 0 } },
{ "Canon",
  "EOS 600D",
  QObject::tr("Flash"),
  { 2.4268, 1, 1.4297, 0 } },

// Canon , EOS REBEL T3i
{ "Canon",
  "EOS REBEL T3i",
  QObject::tr("Daylight"),
  { 2.1602, 1, 1.5752, 0 } },
{ "Canon",
  "EOS REBEL T3i",
  QObject::tr("Shade"),
  { 2.5098, 1, 1.3418, 0 } },
{ "Canon",
  "EOS REBEL T3i",
  QObject::tr("Cloudy"),
  { 2.3330, 1, 1.4502, 0 } },
{ "Canon",
  "EOS REBEL T3i",
  QObject::tr("Tungsten"),
  { 1.5371, 1, 2.3594, 0 } },
{ "Canon",
  "EOS REBEL T3i",
  QObject::tr("WhiteFluorescent"),
  { 1.8789, 1, 2.2402, 0 } },
{ "Canon",
  "EOS REBEL T3i",
  QObject::tr("Flash"),
  { 2.4268, 1, 1.4297, 0 } },

// Canon , EOS Kiss Digital X5
{ "Canon",
  "EOS Kiss Digital X5",
  QObject::tr("Daylight"),
  { 2.1602, 1, 1.5752, 0 } },
{ "Canon",
  "EOS Kiss Digital X5",
  QObject::tr("Shade"),
  { 2.5098, 1, 1.3418, 0 } },
{ "Canon",
  "EOS Kiss Digital X5",
  QObject::tr("Cloudy"),
  { 2.3330, 1, 1.4502, 0 } },
{ "Canon",
  "EOS Kiss Digital X5",
  QObject::tr("Tungsten"),
  { 1.5371, 1, 2.3594, 0 } },
{ "Canon",
  "EOS Kiss Digital X5",
  QObject::tr("WhiteFluorescent"),
  { 1.8789, 1, 2.2402, 0 } },
{ "Canon",
  "EOS Kiss Digital X5",
  QObject::tr("Flash"),
  { 2.4268, 1, 1.4297, 0 } },

// Canon , EOS 1000D
{ "Canon",
  "EOS 1000D",
  QObject::tr("Daylight"),
  { 2.183594, 1, 1.526367, 0 } },
{ "Canon",
  "EOS 1000D",
  QObject::tr("Shade"),
  { 2.553711, 1, 1.262695, 0 } },
{ "Canon",
  "EOS 1000D",
  QObject::tr("Cloudy"),
  { 2.365234, 1, 1.375977, 0 } },
{ "Canon",
  "EOS 1000D",
  QObject::tr("Tungsten"),
  { 1.470328, 1, 2.402126, 0 } },
{ "Canon",
  "EOS 1000D",
  QObject::tr("Fluorescent"),
  { 1.889648, 1, 2.133789, 0 } },
{ "Canon",
  "EOS 1000D",
  QObject::tr("Flash"),
  { 2.541830, 1, 1.769099, 0 } },

// Canon , EOS DIGITAL REBEL XS
{ "Canon",
  "EOS DIGITAL REBEL XS",
  QObject::tr("Daylight"),
  { 2.183594, 1, 1.526367, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XS",
  QObject::tr("Shade"),
  { 2.553711, 1, 1.262695, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XS",
  QObject::tr("Cloudy"),
  { 2.365234, 1, 1.375977, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XS",
  QObject::tr("Tungsten"),
  { 1.470328, 1, 2.402126, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XS",
  QObject::tr("Fluorescent"),
  { 1.889648, 1, 2.133789, 0 } },
{ "Canon",
  "EOS DIGITAL REBEL XS",
  QObject::tr("Flash"),
  { 2.541830, 1, 1.769099, 0 } },

// Canon , EOS Kiss Digital F
{ "Canon",
  "EOS Kiss Digital F",
  QObject::tr("Daylight"),
  { 2.183594, 1, 1.526367, 0 } },
{ "Canon",
  "EOS Kiss Digital F",
  QObject::tr("Shade"),
  { 2.553711, 1, 1.262695, 0 } },
{ "Canon",
  "EOS Kiss Digital F",
  QObject::tr("Cloudy"),
  { 2.365234, 1, 1.375977, 0 } },
{ "Canon",
  "EOS Kiss Digital F",
  QObject::tr("Tungsten"),
  { 1.470328, 1, 2.402126, 0 } },
{ "Canon",
  "EOS Kiss Digital F",
  QObject::tr("Fluorescent"),
  { 1.889648, 1, 2.133789, 0 } },
{ "Canon",
  "EOS Kiss Digital F",
  QObject::tr("Flash"),
  { 2.541830, 1, 1.769099, 0 } },

// Canon , EOS 1100D
{ "Canon",
  "EOS 1100D",
  QObject::tr("Daylight"),
  { 2.2559, 1, 1.4844, 0 } },
{ "Canon",
  "EOS 1100D",
  QObject::tr("Shade"),
  { 2.6455, 1, 1.2725, 0 } },
{ "Canon",
  "EOS 1100D",
  QObject::tr("Cloudy"),
  { 2.4443, 1, 1.3652, 0 } },
{ "Canon",
  "EOS 1100D",
  QObject::tr("Tungsten"),
  { 1.5654, 1, 2.2402, 0 } },
{ "Canon",
  "EOS 1100D",
  QObject::tr("WhiteFluorescent"),
  { 1.9541, 1, 2.0732, 0 } },
{ "Canon",
  "EOS 1100D",
  QObject::tr("Flash"),
  { 2.5283, 1, 1.3584, 0 } },

// Canon , EOS REBEL T3
{ "Canon",
  "EOS REBEL T3",
  QObject::tr("Daylight"),
  { 2.2559, 1, 1.4844, 0 } },
{ "Canon",
  "EOS REBEL T3",
  QObject::tr("Shade"),
  { 2.6455, 1, 1.2725, 0 } },
{ "Canon",
  "EOS REBEL T3",
  QObject::tr("Cloudy"),
  { 2.4443, 1, 1.3652, 0 } },
{ "Canon",
  "EOS REBEL T3",
  QObject::tr("Tungsten"),
  { 1.5654, 1, 2.2402, 0 } },
{ "Canon",
  "EOS REBEL T3",
  QObject::tr("WhiteFluorescent"),
  { 1.9541, 1, 2.0732, 0 } },
{ "Canon",
  "EOS REBEL T3",
  QObject::tr("Flash"),
  { 2.5283, 1, 1.3584, 0 } },

// Canon , EOS Kiss Digital X50
{ "Canon",
  "EOS Kiss Digital X50",
  QObject::tr("Daylight"),
  { 2.2559, 1, 1.4844, 0 } },
{ "Canon",
  "EOS Kiss Digital X50",
  QObject::tr("Shade"),
  { 2.6455, 1, 1.2725, 0 } },
{ "Canon",
  "EOS Kiss Digital X50",
  QObject::tr("Cloudy"),
  { 2.4443, 1, 1.3652, 0 } },
{ "Canon",
  "EOS Kiss Digital X50",
  QObject::tr("Tungsten"),
  { 1.5654, 1, 2.2402, 0 } },
{ "Canon",
  "EOS Kiss Digital X50",
  QObject::tr("WhiteFluorescent"),
  { 1.9541, 1, 2.0732, 0 } },
{ "Canon",
  "EOS Kiss Digital X50",
  QObject::tr("Flash"),
  { 2.5283, 1, 1.3584, 0 } },

// Canon , EOS-1DS
{ "Canon",
  "EOS-1DS",
  QObject::tr("Daylight"),
  { 1.6133, 1, 1.2500, 0 } },
{ "Canon",
  "EOS-1DS",
  QObject::tr("Shade"),
  { 1.8477, 1, 1.0684, 0 } },
{ "Canon",
  "EOS-1DS",
  QObject::tr("Cloudy"),
  { 1.7285, 1, 1.1504, 0 } },
{ "Canon",
  "EOS-1DS",
  QObject::tr("Tungsten"),
  { 1.1270, 1, 1.9199, 0 } },
{ "Canon",
  "EOS-1DS",
  QObject::tr("Fluorescent"),
  { 1.2012, 1, 1.7168, 0 } },
{ "Canon",
  "EOS-1DS",
  QObject::tr("Flash"),
  { 1.7793, 1, 1.1445, 0 } },

// Canon , EOS-1Ds Mark II
{ "Canon",
  "EOS-1Ds Mark II",
  QObject::tr("Daylight"),
  { 1.992188, 1, 1.503906, 0 } },
{ "Canon",
  "EOS-1Ds Mark II",
  QObject::tr("Shade"),
  { 2.300781, 1, 1.243164, 0 } },
{ "Canon",
  "EOS-1Ds Mark II",
  QObject::tr("Cloudy"),
  { 2.146484, 1, 1.363281, 0 } },
{ "Canon",
  "EOS-1Ds Mark II",
  QObject::tr("Tungsten"),
  { 1.395018, 1, 2.370107, 0 } },
{ "Canon",
  "EOS-1Ds Mark II",
  QObject::tr("Fluorescent"),
  { 1.854792, 1, 2.067764, 0 } },
{ "Canon",
  "EOS-1Ds Mark II",
  QObject::tr("Flash"),
  { 2.235352, 1, 1.297852, 0 } },

// Canon , EOS-1D Mark II
{ "Canon",
  "EOS-1D Mark II",
  QObject::tr("Cloudy"),
  { 2.093750, 1, 1.166016, 0 } },
{ "Canon",
  "EOS-1D Mark II",
  QObject::tr("Daylight"),
  { 1.957031, 1, 1.295898, 0 } },
{ "Canon",
  "EOS-1D Mark II",
  QObject::tr("Flash"),
  { 2.225586, 1, 1.172852, 0 } },
{ "Canon",
  "EOS-1D Mark II",
  QObject::tr("Fluorescent"),
  { 1.785853, 1, 1.785853, 0 } },
{ "Canon",
  "EOS-1D Mark II",
  QObject::tr("Shade"),
  { 2.220703, 1, 1.069336, 0 } },
{ "Canon",
  "EOS-1D Mark II",
  QObject::tr("Tungsten"),
  { 1.415480, 1, 2.160142, 0 } },

// Canon , EOS-1D Mark II N
{ "Canon",
  "EOS-1D Mark II N",
  QObject::tr("Cloudy"),
  { 2.183594, 1, 1.220703, 0 } },
{ "Canon",
  "EOS-1D Mark II N",
  QObject::tr("Daylight"),
  { 2.019531, 1, 1.349609, 0 } },
{ "Canon",
  "EOS-1D Mark II N",
  QObject::tr("Flash"),
  { 2.291016, 1, 1.149414, 0 } },
{ "Canon",
  "EOS-1D Mark II N",
  QObject::tr("Fluorescent"),
  { 1.802899, 1, 1.990338, 0 } },
{ "Canon",
  "EOS-1D Mark II N",
  QObject::tr("Shade"),
  { 2.337891, 1, 1.112305, 0 } },
{ "Canon",
  "EOS-1D Mark II N",
  QObject::tr("Tungsten"),
  { 1.408514, 1, 2.147645, 0 } },

// Canon , EOS-1D Mark IV
{ "Canon",
  "EOS-1D Mark IV",
  QObject::tr("Daylight"),
  { 2.040039, 1, 1.558594, 0 } },
{ "Canon",
  "EOS-1D Mark IV",
  QObject::tr("Shade"),
  { 2.342773, 1, 1.333008, 0 } },
{ "Canon",
  "EOS-1D Mark IV",
  QObject::tr("Cloudy"),
  { 2.188477, 1, 1.440430, 0 } },
{ "Canon",
  "EOS-1D Mark IV",
  QObject::tr("Tungsten"),
  { 1.458333, 1, 2.305254, 0 } },
{ "Canon",
  "EOS-1D Mark IV",
  QObject::tr("Fluorescent"),
  { 1.767892, 1, 2.205029, 0 } },
{ "Canon",
  "EOS-1D Mark IV",
  QObject::tr("Flash"),
  { 2.230469, 1, 1.423828, 0 } },

// FUJIFILM , FinePix E900
{ "FUJIFILM",
  "FinePix E900",
  QObject::tr("Daylight"),
  { 1.571875, 1, 1.128125, 0 } },
{ "FUJIFILM",
  "FinePix E900",
  QObject::tr("Shade"),
  { 1.668750, 1, 1.006250, 0 } },
{ "FUJIFILM",
  "FinePix E900",
  QObject::tr("DaylightFluorescent"),
  { 1.907609, 1, 1.016304, 0 } },
{ "FUJIFILM",
  "FinePix E900",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.654891, 1, 1.241848, 0 } },
{ "FUJIFILM",
  "FinePix E900",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.554348, 1, 1.519022, 0 } },
{ "FUJIFILM",
  "FinePix E900",
  QObject::tr("Incandescent"),
  { 1.037611, 1, 1.842920, 0 } },

// FUJIFILM , FinePix F700
{ "FUJIFILM",
  "FinePix F700",
  QObject::tr("Daylight"),
  { 1.725000, 1, 1.500000, 0 } },
{ "FUJIFILM",
  "FinePix F700",
  QObject::tr("Shade"),
  { 1.950000, 1, 1.325000, 0 } },
{ "FUJIFILM",
  "FinePix F700",
  QObject::tr("DaylightFluorescent"),
  { 2.032609, 1, 1.336957, 0 } },
{ "FUJIFILM",
  "FinePix F700",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.706522, 1, 1.663043, 0 } },
{ "FUJIFILM",
  "FinePix F700",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.684783, 1, 2.152174, 0 } },
{ "FUJIFILM",
  "FinePix F700",
  QObject::tr("Incandescent"),
  { 1.168142, 1, 2.477876, 0 } },

// FUJIFILM , FinePix HS20EXR
{ "FUJIFILM",
  "FinePix HS20EXR",
  QObject::tr("Daylight"),
  { 1.4107, 1, 1.9702, 0 } },
{ "FUJIFILM",
  "FinePix HS20EXR",
  QObject::tr("Shade"),
  { 1.5804, 1, 1.7440, 0 } },
{ "FUJIFILM",
  "FinePix HS20EXR",
  QObject::tr("DaylightFluorescent"),
  { 1.7292, 1, 1.7470, 0 } },
{ "FUJIFILM",
  "FinePix HS20EXR",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.4821, 1, 2.0476, 0 } },
{ "FUJIFILM",
  "FinePix HS20EXR",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.5625, 1, 2.5714, 0 } },
{ "FUJIFILM",
  "FinePix HS20EXR",
  QObject::tr("Incandescent"),
  { 1, 1.0633, 2.9430, 0 } },

// FUJIFILM , FinePix S100FS
{ "FUJIFILM",
  "FinePix S100FS",
  QObject::tr("Daylight"),
  { 1.702381, 1, 1.845238, 0 } },
{ "FUJIFILM",
  "FinePix S100FS",
  QObject::tr("Shade"),
  { 1.830357, 1, 1.601190, 0 } },
{ "FUJIFILM",
  "FinePix S100FS",
  QObject::tr("DaylightFluorescent"),
  { 1.895833, 1, 1.461309, 0 } },
{ "FUJIFILM",
  "FinePix S100FS",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.574405, 1, 1.818452, 0 } },
{ "FUJIFILM",
  "FinePix S100FS",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.663690, 1, 2.309524, 0 } },
{ "FUJIFILM",
  "FinePix S100FS",
  QObject::tr("Incandescent"),
  { 1.107143, 1, 2.815476, 0 } },

// FUJIFILM , FinePix S20Pro
{ "FUJIFILM",
  "FinePix S20Pro",
  QObject::tr("Daylight"),
  { 1.712500, 1, 1.500000, 0 } },
{ "FUJIFILM",
  "FinePix S20Pro",
  QObject::tr("Cloudy"),
  { 1.887500, 1, 1.262500, 0 } },
{ "FUJIFILM",
  "FinePix S20Pro",
  QObject::tr("DaylightFluorescent"),
  { 2.097826, 1, 1.304348, 0 } },
{ "FUJIFILM",
  "FinePix S20Pro",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.782609, 1, 1.619565, 0 } },
{ "FUJIFILM",
  "FinePix S20Pro",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.670213, 1, 2.063830, 0 } },
{ "FUJIFILM",
  "FinePix S20Pro",
  QObject::tr("Incandescent"),
  { 1.069565, 1, 2.486957, 0 } },

// FUJIFILM , FinePix S2Pro
{ "FUJIFILM",
  "FinePix S2Pro",
  QObject::tr("Daylight"),
  { 1.509804, 1, 1.401961, 0 } },
{ "FUJIFILM",
  "FinePix S2Pro",
  QObject::tr("Cloudy"),
  { 1.666667, 1, 1.166667, 0 } },
{ "FUJIFILM",
  "FinePix S2Pro",
  QObject::tr("Flash"),
  { 1, 1.014084, 2.542253, 0 } },
{ "FUJIFILM",
  "FinePix S2Pro",
  QObject::tr("DaylightFluorescent"),
  { 1.948718, 1, 1.230769, 0 } },
{ "FUJIFILM",
  "FinePix S2Pro",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.675214, 1, 1.572650, 0 } },
{ "FUJIFILM",
  "FinePix S2Pro",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.649573, 1, 2.094017, 0 } },

// FUJIFILM , FinePix S5000
{ "FUJIFILM",
  "FinePix S5000",
  QObject::tr("Incandescent"),
  { 1.212081, 1, 2.672364, 0 } },
{ "FUJIFILM",
  "FinePix S5000",
  QObject::tr("Fluorescent"),
  { 1.772316, 1, 2.349902, 0 } },
{ "FUJIFILM",
  "FinePix S5000",
  QObject::tr("Daylight"),
  { 1.860403, 1, 1.515946, 0 } },
{ "FUJIFILM",
  "FinePix S5000",
  QObject::tr("Flash"),
  { 2.202181, 1, 1.423284, 0 } },
{ "FUJIFILM",
  "FinePix S5000",
  QObject::tr("Cloudy"),
  { 2.036578, 1, 1.382513, 0 } },
{ "FUJIFILM",
  "FinePix S5000",
  QObject::tr("Shade"),
  { 2.357215, 1, 1.212016, 0 } },

// FUJIFILM , FinePix S5200
{ "FUJIFILM",
  "FinePix S5200",
  QObject::tr("Daylight"),
  { 1.587500, 1, 1.381250, 0 } },
{ "FUJIFILM",
  "FinePix S5200",
  QObject::tr("Shade"),
  { 1.946875, 1, 1.175000, 0 } },
{ "FUJIFILM",
  "FinePix S5200",
  QObject::tr("DaylightFluorescent"),
  { 1.948370, 1, 1.187500, 0 } },
{ "FUJIFILM",
  "FinePix S5200",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.682065, 1, 1.437500, 0 } },
{ "FUJIFILM",
  "FinePix S5200",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.595109, 1, 1.839674, 0 } },
{ "FUJIFILM",
  "FinePix S5200",
  QObject::tr("Incandescent"),
  { 1.077434, 1, 2.170354, 0 } },

// FUJIFILM , FinePix S5500
{ "FUJIFILM",
  "FinePix S5500",
  QObject::tr("Daylight"),
  { 1.712500, 1, 1.550000, 0 } },
{ "FUJIFILM",
  "FinePix S5500",
  QObject::tr("Shade"),
  { 1.912500, 1, 1.375000, 0 } },
{ "FUJIFILM",
  "FinePix S5500",
  QObject::tr("DaylightFluorescent"),
  { 1.978261, 1, 1.380435, 0 } },
{ "FUJIFILM",
  "FinePix S5500",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.673913, 1, 1.673913, 0 } },
{ "FUJIFILM",
  "FinePix S5500",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.663043, 1, 2.163043, 0 } },
{ "FUJIFILM",
  "FinePix S5500",
  QObject::tr("Incandescent"),
  { 1.115044, 1, 2.566372, 0 } },

// FUJIFILM , FinePix S5600
{ "FUJIFILM",
  "FinePix S5600",
  QObject::tr("Daylight"),
  { 1.587500, 1, 1.381250, 0 } },
{ "FUJIFILM",
  "FinePix S5600",
  QObject::tr("Shade"),
  { 1.946875, 1, 1.175000, 0 } },
{ "FUJIFILM",
  "FinePix S5600",
  QObject::tr("DaylightFluorescent"),
  { 1.948370, 1, 1.187500, 0 } },
{ "FUJIFILM",
  "FinePix S5600",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.682065, 1, 1.437500, 0 } },
{ "FUJIFILM",
  "FinePix S5600",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.595109, 1, 1.839674, 0 } },
{ "FUJIFILM",
  "FinePix S5600",
  QObject::tr("Incandescent"),
  { 1.077434, 1, 2.170354, 0 } },

// FUJIFILM , FinePix S6000fd
{ "FUJIFILM",
  "FinePix S6000fd",
  QObject::tr("Daylight"),
  { 1.511905, 1, 1.431548, 0 } },
{ "FUJIFILM",
  "FinePix S6000fd",
  QObject::tr("Shade"),
  { 1.699405, 1, 1.232143, 0 } },
{ "FUJIFILM",
  "FinePix S6000fd",
  QObject::tr("DaylightFluorescent"),
  { 1.866071, 1, 1.309524, 0 } },
{ "FUJIFILM",
  "FinePix S6000fd",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.568452, 1, 1.627976, 0 } },
{ "FUJIFILM",
  "FinePix S6000fd",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.598214, 1, 2.038691, 0 } },
{ "FUJIFILM",
  "FinePix S6000fd",
  QObject::tr("Incandescent"),
  { 1, 1.024390, 2.466463, 0 } },

// FUJIFILM , FinePix S6500fd
{ "FUJIFILM",
  "FinePix S6500fd",
  QObject::tr("Daylight"),
  { 1.398810, 1, 1.470238, 0 } },
{ "FUJIFILM",
  "FinePix S6500fd",
  QObject::tr("Shade"),
  { 1.580357, 1, 1.270833, 0 } },
{ "FUJIFILM",
  "FinePix S6500fd",
  QObject::tr("DaylightFluorescent"),
  { 1.735119, 1, 1.348214, 0 } },
{ "FUJIFILM",
  "FinePix S6500fd",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.455357, 1, 1.672619, 0 } },
{ "FUJIFILM",
  "FinePix S6500fd",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.482143, 1, 2.089286, 0 } },
{ "FUJIFILM",
  "FinePix S6500fd",
  QObject::tr("Incandescent"),
  { 1, 1.123746, 2.769231, 0 } },

// FUJIFILM , FinePix S7000
{ "FUJIFILM",
  "FinePix S7000",
  QObject::tr("Daylight"),
  { 1.900000, 1, 1.525000, 0 } },
{ "FUJIFILM",
  "FinePix S7000",
  QObject::tr("Shade"),
  { 2.137500, 1, 1.350000, 0 } },
{ "FUJIFILM",
  "FinePix S7000",
  QObject::tr("DaylightFluorescent"),
  { 2.315217, 1, 1.347826, 0 } },
{ "FUJIFILM",
  "FinePix S7000",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.902174, 1, 1.663043, 0 } },
{ "FUJIFILM",
  "FinePix S7000",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.836957, 1, 2.130435, 0 } },
{ "FUJIFILM",
  "FinePix S7000",
  QObject::tr("Incandescent"),
  { 1.221239, 1, 2.548673, 0 } },

// FUJIFILM , FinePix S9000
{ "FUJIFILM",
  "FinePix S9000",
  QObject::tr("Daylight"),
  { 1.618750, 1, 1.231250, 0 } },
{ "FUJIFILM",
  "FinePix S9000",
  QObject::tr("Cloudy"),
  { 1.700000, 1, 1.046875, 0 } },
{ "FUJIFILM",
  "FinePix S9000",
  QObject::tr("DaylightFluorescent"),
  { 1.902174, 1, 1.057065, 0 } },
{ "FUJIFILM",
  "FinePix S9000",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.633152, 1, 1.293478, 0 } },
{ "FUJIFILM",
  "FinePix S9000",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.546196, 1, 1.622283, 0 } },
{ "FUJIFILM",
  "FinePix S9000",
  QObject::tr("Incandescent"),
  { 1.064159, 1, 1.960177, 0 } },

// FUJIFILM , FinePix S9500
{ "FUJIFILM",
  "FinePix S9500",
  QObject::tr("Daylight"),
  { 1.618750, 1, 1.231250, 0 } },
{ "FUJIFILM",
  "FinePix S9500",
  QObject::tr("Cloudy"),
  { 1.700000, 1, 1.046875, 0 } },
{ "FUJIFILM",
  "FinePix S9500",
  QObject::tr("DaylightFluorescent"),
  { 1.902174, 1, 1.057065, 0 } },
{ "FUJIFILM",
  "FinePix S9500",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.633152, 1, 1.293478, 0 } },
{ "FUJIFILM",
  "FinePix S9500",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.546196, 1, 1.622283, 0 } },
{ "FUJIFILM",
  "FinePix S9500",
  QObject::tr("Incandescent"),
  { 1.064159, 1, 1.960177, 0 } },

// FUJIFILM , FinePix S9100
{ "FUJIFILM",
  "FinePix S9100",
  QObject::tr("Daylight"),
  { 1.506250, 1, 1.318750, 0 } },
{ "FUJIFILM",
  "FinePix S9100",
  QObject::tr("Cloudy"),
  { 1.587500, 1, 1.128125, 0 } },
{ "FUJIFILM",
  "FinePix S9100",
  QObject::tr("DaylightFluorescent"),
  { 1.777174, 1, 1.138587, 0 } },
{ "FUJIFILM",
  "FinePix S9100",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.521739, 1, 1.380435, 0 } },
{ "FUJIFILM",
  "FinePix S9100",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.437500, 1, 1.720109, 0 } },
{ "FUJIFILM",
  "FinePix S9100",
  QObject::tr("Incandescent"),
  { 1, 1.024943, 2.113379, 0 } },

// FUJIFILM , FinePix S9600
{ "FUJIFILM",
  "FinePix S9600",
  QObject::tr("Daylight"),
  { 1.534375, 1, 1.300000, 0 } },
{ "FUJIFILM",
  "FinePix S9600",
  QObject::tr("Shade"),
  { 1.615625, 1, 1.112500, 0 } },
{ "FUJIFILM",
  "FinePix S9600",
  QObject::tr("DaylightFluorescent"),
  { 1.809783, 1, 1.122283, 0 } },
{ "FUJIFILM",
  "FinePix S9600",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.551630, 1, 1.361413, 0 } },
{ "FUJIFILM",
  "FinePix S9600",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.467391, 1, 1.692935, 0 } },
{ "FUJIFILM",
  "FinePix S9600",
  QObject::tr("Incandescent"),
  { 1, 1.004444, 2.040000, 0 } },

// FUJIFILM , FinePix X100
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("Daylight"),
  { 1.4503, 1, 1.5033, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("Shade"),
  { 1.5861, 1, 1.2947, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("DaylightFluorescent"),
  { 1.8841, 1, 1.3179, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.6291, 1, 1.5927, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.5662, 1, 2.0265, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("Incandescent"),
  { 1, 1.0272, 2.4966, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("Underwater"),
  { 1.4603, 1, 1.5662, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("2500K"),
  { 1, 1.6503, 5.1858, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("2550K"),
  { 1, 1.5729, 4.7917, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("2650K"),
  { 1, 1.4313, 4.1090, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("2700K"),
  { 1, 1.3790, 3.8447, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("2800K"),
  { 1, 1.2797, 3.3814, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("2850K"),
  { 1, 1.2377, 3.1844, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("2950K"),
  { 1, 1.1660, 2.8571, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("3000K"),
  { 1, 1.1353, 2.7180, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("3100K"),
  { 1, 1.0824, 2.4731, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("3200K"),
  { 1, 1.0342, 2.2671, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("3300K"),
  { 1.0066, 1, 2.1060, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("3400K"),
  { 1.0430, 1, 2.0265, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("3600K"),
  { 1.1159, 1, 1.8907, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("3700K"),
  { 1.1457, 1, 1.8311, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("3800K"),
  { 1.1755, 1, 1.7781, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("4000K"),
  { 1.2351, 1, 1.6821, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("4200K"),
  { 1.2848, 1, 1.5993, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("4300K"),
  { 1.3079, 1, 1.5662, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("4500K"),
  { 1.3543, 1, 1.5000, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("4800K"),
  { 1.4172, 1, 1.4205, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("5000K"),
  { 1.4536, 1, 1.3742, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("5300K"),
  { 1.5033, 1, 1.3179, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("5600K"),
  { 1.5530, 1, 1.2715, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("5900K"),
  { 1.5927, 1, 1.2318, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("6300K"),
  { 1.6457, 1, 1.1887, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("6700K"),
  { 1.6921, 1, 1.1556, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("7100K"),
  { 1.7318, 1, 1.1291, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("7700K"),
  { 1.7881, 1, 1.0960, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("8300K"),
  { 1.8377, 1, 1.0728, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("9100K"),
  { 1.8940, 1, 1.0464, 0 } },
{ "FUJIFILM",
  "FinePix X100",
  QObject::tr("10000K"),
  { 1.9503, 1, 1.0298, 0 } },

// KODAK , P850 ZOOM
{ "KODAK",
  "P850 ZOOM",
  QObject::tr("Daylight"),
  { 1.859375, 1, 1.566406, 0 } },
{ "KODAK",
  "P850 ZOOM",
  QObject::tr("Cloudy"),
  { 1.960938, 1, 1.570313, 0 } },
{ "KODAK",
  "P850 ZOOM",
  QObject::tr("Shade"),
  { 2.027344, 1, 1.519531, 0 } },
{ "KODAK",
  "P850 ZOOM",
  QObject::tr("EveningSun"),
  { 1.679688, 1, 1.812500, 0 } },
{ "KODAK",
  "P850 ZOOM",
  QObject::tr("Tungsten"),
  { 1.140625, 1, 2.726563, 0 } },
{ "KODAK",
  "P850 ZOOM",
  QObject::tr("Fluorescent"),
  { 1.113281, 1, 2.949219, 0 } },

// KODAK , EASYSHARE Z1015 IS
{ "KODAK",
  "EASYSHARE Z1015 IS",
  QObject::tr("Daylight"),
  { 1.546875, 1, 2.082031, 0 } },
{ "KODAK",
  "EASYSHARE Z1015 IS",
  QObject::tr("Tungsten"),
  { 1, 1.024000, 3.384000, 0 } },
{ "KODAK",
  "EASYSHARE Z1015 IS",
  QObject::tr("Fluorescent"),
  { 1.562500, 1, 2.515625, 0 } },
{ "KODAK",
  "EASYSHARE Z1015 IS",
  QObject::tr("Shade"),
  { 1.820313, 1, 1.789062, 0 } },

// Leica Camera AG , M8 Digital Camera
{ "Leica Camera AG",
  "M8 Digital Camera",
  QObject::tr("Cloudy"),
  { 2.136719, 1, 1.168213, 0 } },
{ "Leica Camera AG",
  "M8 Digital Camera",
  QObject::tr("Daylight"),
  { 2.007996, 1, 1.268982, 0 } },
{ "Leica Camera AG",
  "M8 Digital Camera",
  QObject::tr("Flash"),
  { 2.164490, 1, 1.177795, 0 } },
{ "Leica Camera AG",
  "M8 Digital Camera",
  QObject::tr("Fluorescent"),
  { 1.655579, 1, 2.070374, 0 } },
{ "Leica Camera AG",
  "M8 Digital Camera",
  QObject::tr("Shade"),
  { 2.197754, 1, 1.111084, 0 } },
{ "Leica Camera AG",
  "M8 Digital Camera",
  QObject::tr("Tungsten"),
  { 1.160034, 1, 2.028381, 0 } },

// Leica , M9 Digital Camera
{ "Leica",
  "M9 Digital Camera",
  QObject::tr("Tungsten"),
  { 1.321288, 1, 2.077024, 0 } },
{ "Leica",
  "M9 Digital Camera",
  QObject::tr("Fluorescent"),
  { 1.673827, 1, 1.855043, 0 } },
{ "Leica",
  "M9 Digital Camera",
  QObject::tr("DaylightFluorescent"),
  { 2.224852, 1, 1.388000, 0 } },
{ "Leica",
  "M9 Digital Camera",
  QObject::tr("Daylight"),
  { 2.013733, 1, 1.364869, 0 } },
{ "Leica",
  "M9 Digital Camera",
  QObject::tr("Flash"),
  { 1.980652, 1, 1.331111, 0 } },
{ "Leica",
  "M9 Digital Camera",
  QObject::tr("Cloudy"),
  { 2.146728, 1, 1.252197, 0 } },
{ "Leica",
  "M9 Digital Camera",
  QObject::tr("Shade"),
  { 2.249268, 1, 1.179015, 0 } },

// Leica Camera AG , R8 - Digital Back DMR
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("Incandescent"),
  { 1, 1.109985, 2.430664, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("Fluorescent"),
  { 1.234985, 1, 1.791138, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("Daylight"),
  { 1.459961, 1, 1.184937, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("Flash"),
  { 1.395020, 1, 1.144897, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("Cloudy"),
  { 1.541992, 1, 1.052856, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("Shade"),
  { 1.644897, 1.033936, 1, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("2600K"),
  { 1, 1.220825, 2.999390, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("2700K"),
  { 1, 1.172607, 2.747192, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("2800K"),
  { 1, 1.129639, 2.527710, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("2900K"),
  { 1, 1.088867, 2.333130, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("3000K"),
  { 1, 1.049438, 2.156494, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("3100K"),
  { 1, 1.015503, 2.008423, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("3200K"),
  { 1.008789, 1, 1.904663, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("3300K"),
  { 1.032349, 1, 1.841187, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("3400K"),
  { 1.056763, 1, 1.780273, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("3500K"),
  { 1.081543, 1, 1.723755, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("3600K"),
  { 1.105591, 1, 1.673828, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("3700K"),
  { 1.128052, 1, 1.625732, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("3800K"),
  { 1.149536, 1, 1.580688, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("3900K"),
  { 1.170532, 1, 1.540527, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("4000K"),
  { 1.191040, 1, 1.504150, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("4100K"),
  { 1.209106, 1, 1.466919, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("4200K"),
  { 1.226807, 1, 1.433228, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("4300K"),
  { 1.244019, 1, 1.402466, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("4400K"),
  { 1.261108, 1, 1.374268, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("4500K"),
  { 1.276611, 1, 1.346924, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("4600K"),
  { 1.290771, 1, 1.320435, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("4700K"),
  { 1.304565, 1, 1.295898, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("4800K"),
  { 1.318115, 1, 1.273315, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("4900K"),
  { 1.331543, 1, 1.252441, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("5000K"),
  { 1.344360, 1, 1.233032, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("5200K"),
  { 1.365479, 1, 1.193970, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("5400K"),
  { 1.385498, 1, 1.160034, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("5600K"),
  { 1.404663, 1, 1.130127, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("5800K"),
  { 1.421387, 1, 1.102661, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("6000K"),
  { 1.435303, 1, 1.076782, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("6200K"),
  { 1.448608, 1, 1.053833, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("6400K"),
  { 1.461304, 1, 1.032959, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("6600K"),
  { 1.473511, 1, 1.014160, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("6800K"),
  { 1.488647, 1.003906, 1, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("7000K"),
  { 1.522705, 1.021118, 1, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("7200K"),
  { 1.555176, 1.037476, 1, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("7400K"),
  { 1.586182, 1.052979, 1, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("7600K"),
  { 1.615967, 1.067627, 1, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("7800K"),
  { 1.644409, 1.081665, 1, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("8000K"),
  { 1.671875, 1.094849, 1, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("8300K"),
  { 1.708740, 1.114624, 1, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("8600K"),
  { 1.743286, 1.133057, 1, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("8900K"),
  { 1.775879, 1.150269, 1.000000, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("9200K"),
  { 1.806274, 1.166382, 1.000000, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("9500K"),
  { 1.835449, 1.181519, 1.000000, 0 } },
{ "Leica Camera AG",
  "R8 - Digital Back DMR",
  QObject::tr("9800K"),
  { 1.862793, 1.195801, 1.000000, 0 } },

// Leica Camera AG , R9 - Digital Back DMR
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("Incandescent"),
  { 1, 1.109985, 2.430664, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("Fluorescent"),
  { 1.234985, 1, 1.791138, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("Daylight"),
  { 1.459961, 1, 1.184937, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("Flash"),
  { 1.395020, 1, 1.144897, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("Cloudy"),
  { 1.541992, 1, 1.052856, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("Shade"),
  { 1.644897, 1.033936, 1, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("2600K"),
  { 1, 1.220825, 2.999390, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("2700K"),
  { 1, 1.172607, 2.747192, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("2800K"),
  { 1, 1.129639, 2.527710, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("2900K"),
  { 1, 1.088867, 2.333130, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("3000K"),
  { 1, 1.049438, 2.156494, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("3100K"),
  { 1, 1.015503, 2.008423, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("3200K"),
  { 1.008789, 1, 1.904663, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("3300K"),
  { 1.032349, 1, 1.841187, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("3400K"),
  { 1.056763, 1, 1.780273, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("3500K"),
  { 1.081543, 1, 1.723755, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("3600K"),
  { 1.105591, 1, 1.673828, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("3700K"),
  { 1.128052, 1, 1.625732, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("3800K"),
  { 1.149536, 1, 1.580688, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("3900K"),
  { 1.170532, 1, 1.540527, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("4000K"),
  { 1.191040, 1, 1.504150, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("4100K"),
  { 1.209106, 1, 1.466919, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("4200K"),
  { 1.226807, 1, 1.433228, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("4300K"),
  { 1.244019, 1, 1.402466, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("4400K"),
  { 1.261108, 1, 1.374268, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("4500K"),
  { 1.276611, 1, 1.346924, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("4600K"),
  { 1.290771, 1, 1.320435, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("4700K"),
  { 1.304565, 1, 1.295898, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("4800K"),
  { 1.318115, 1, 1.273315, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("4900K"),
  { 1.331543, 1, 1.252441, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("5000K"),
  { 1.344360, 1, 1.233032, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("5200K"),
  { 1.365479, 1, 1.193970, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("5400K"),
  { 1.385498, 1, 1.160034, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("5600K"),
  { 1.404663, 1, 1.130127, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("5800K"),
  { 1.421387, 1, 1.102661, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("6000K"),
  { 1.435303, 1, 1.076782, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("6200K"),
  { 1.448608, 1, 1.053833, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("6400K"),
  { 1.461304, 1, 1.032959, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("6600K"),
  { 1.473511, 1, 1.014160, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("6800K"),
  { 1.488647, 1.003906, 1, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("7000K"),
  { 1.522705, 1.021118, 1, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("7200K"),
  { 1.555176, 1.037476, 1, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("7400K"),
  { 1.586182, 1.052979, 1, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("7600K"),
  { 1.615967, 1.067627, 1, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("7800K"),
  { 1.644409, 1.081665, 1, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("8000K"),
  { 1.671875, 1.094849, 1, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("8300K"),
  { 1.708740, 1.114624, 1, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("8600K"),
  { 1.743286, 1.133057, 1, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("8900K"),
  { 1.775879, 1.150269, 1.000000, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("9200K"),
  { 1.806274, 1.166382, 1.000000, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("9500K"),
  { 1.835449, 1.181519, 1.000000, 0 } },
{ "Leica Camera AG",
  "R9 - Digital Back DMR",
  QObject::tr("9800K"),
  { 1.862793, 1.195801, 1.000000, 0 } },

// LEICA , DIGILUX 2
{ "LEICA",
  "DIGILUX 2",
  QObject::tr("Daylight"),
  { 1.628906, 1, 1.488281, 0 } },
{ "LEICA",
  "DIGILUX 2",
  QObject::tr("Cloudy"),
  { 1.835938, 1, 1.343750, 0 } },
{ "LEICA",
  "DIGILUX 2",
  QObject::tr("Incandescent"),
  { 1.078125, 1, 2.203125, 0 } },
{ "LEICA",
  "DIGILUX 2",
  QObject::tr("Flash"),
  { 2.074219, 1, 1.304688, 0 } },
{ "LEICA",
  "DIGILUX 2",
  QObject::tr("BlackNWhite"),
  { 1.632812, 1, 1.550781, 0 } },

// LEICA , DIGILUX 3
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("Daylight"),
  { 1.942966, 1, 1.399240, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("Cloudy"),
  { 2.083650, 1, 1.247148, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("Shade"),
  { 2.296578, 1, 1.110266, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("Incandescent"),
  { 1.372624, 1, 2.079848, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("Flash"),
  { 2.095057, 1, 1.091255, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("2500K"),
  { 1.178707, 1, 2.756654, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("2600K"),
  { 1.212928, 1, 2.650190, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("2700K"),
  { 1.250951, 1, 2.539924, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("2800K"),
  { 1.285171, 1, 2.433460, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("2900K"),
  { 1.323194, 1, 2.326996, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("3000K"),
  { 1.361217, 1, 2.212928, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("3100K"),
  { 1.391635, 1, 2.197719, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("3200K"),
  { 1.429658, 1, 2.178707, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("3300K"),
  { 1.471483, 1, 2.167300, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("3400K"),
  { 1.509506, 1, 2.148289, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("3500K"),
  { 1.547529, 1, 2.133080, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("3600K"),
  { 1.574145, 1, 2.087453, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("3800K"),
  { 1.631179, 1, 1.992395, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("4000K"),
  { 1.684411, 1, 1.882129, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("4200K"),
  { 1.733840, 1, 1.790875, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("4400K"),
  { 1.790875, 1, 1.699620, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("4600K"),
  { 1.821293, 1, 1.615970, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("4800K"),
  { 1.832700, 1, 1.551331, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("5000K"),
  { 1.851711, 1, 1.490494, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("5300K"),
  { 1.889734, 1, 1.414449, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("5500K"),
  { 1.923954, 1, 1.361217, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("5800K"),
  { 1.954373, 1, 1.315589, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("6000K"),
  { 1.977186, 1, 1.277567, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("6300K"),
  { 2.049430, 1, 1.231939, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("6500K"),
  { 2.102662, 1, 1.193916, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("6800K"),
  { 2.155893, 1, 1.178707, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("7300K"),
  { 2.254753, 1, 1.133080, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("7800K"),
  { 2.319392, 1, 1.087452, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("8300K"),
  { 2.365019, 1, 1.045627, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("9000K"),
  { 2.429658, 1, 1.007605, 0 } },
{ "LEICA",
  "DIGILUX 3",
  QObject::tr("10000K"),
  { 2.680608, 1.057034, 1, 0 } },

// Minolta , DiMAGE 5
{ "Minolta",
  "DiMAGE 5",
  QObject::tr("Daylight"),
  { 2.023438, 1, 1.371094, 0 } },
{ "Minolta",
  "DiMAGE 5",
  QObject::tr("Incandescent"),
  { 1.113281, 1, 2.480469, 0 } },
{ "Minolta",
  "DiMAGE 5",
  QObject::tr("Fluorescent"),
  { 1.957031, 1, 2.058594, 0 } },
{ "Minolta",
  "DiMAGE 5",
  QObject::tr("Cloudy"),
  { 2.199219, 1, 1.300781, 0 } },

// Minolta , DiMAGE 7
{ "Minolta",
  "DiMAGE 7",
  QObject::tr("Cloudy"),
  { 2.082031, 1, 1.226562, 0 } },
{ "Minolta",
  "DiMAGE 7",
  QObject::tr("Daylight"),
  { 1.914062, 1, 1.527344, 0 } },
{ "Minolta",
  "DiMAGE 7",
  QObject::tr("Fluorescent"),
  { 1.917969, 1, 2.007812, 0 } },
{ "Minolta",
  "DiMAGE 7",
  QObject::tr("Tungsten"),
  { 1.050781, 1, 2.437500, 0 } },

// Minolta , DiMAGE 7i
{ "Minolta",
  "DiMAGE 7i",
  QObject::tr("Daylight"),
  { 1.441406, 1, 1.457031, 0 } },
{ "Minolta",
  "DiMAGE 7i",
  QObject::tr("Tungsten"),
  { 1, 1.333333, 3.572917, 0 } },
{ "Minolta",
  "DiMAGE 7i",
  QObject::tr("Fluorescent"),
  { 1.554688, 1, 2.230469, 0 } },
{ "Minolta",
  "DiMAGE 7i",
  QObject::tr("Cloudy"),
  { 1.550781, 1, 1.402344, 0 } },

// Minolta , DiMAGE 7Hi
{ "Minolta",
  "DiMAGE 7Hi",
  QObject::tr("Daylight"),
  { 1.609375, 1, 1.328125, 0 } },
{ "Minolta",
  "DiMAGE 7Hi",
  QObject::tr("Tungsten"),
  { 1, 1.137778, 2.768889, 0 } },
{ "Minolta",
  "DiMAGE 7Hi",
  QObject::tr("WhiteFluorescent"),
  { 1.664062, 1, 2.105469, 0 } },
{ "Minolta",
  "DiMAGE 7Hi",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.796875, 1, 1.734375, 0 } },
{ "Minolta",
  "DiMAGE 7Hi",
  QObject::tr("Cloudy"),
  { 1.730469, 1, 1.269531, 0 } },

// Minolta , DiMAGE A1
{ "Minolta",
  "DiMAGE A1",
  QObject::tr("Daylight"),
  { 1.808594, 1, 1.304688, 0 } },
{ "Minolta",
  "DiMAGE A1",
  QObject::tr("Tungsten"),
  { 1.062500, 1, 2.675781, 0 } },
{ "Minolta",
  "DiMAGE A1",
  QObject::tr("Fluorescent"),
  { 1.707031, 1, 2.039063, 0 } },
{ "Minolta",
  "DiMAGE A1",
  QObject::tr("Cloudy"),
  { 1.960938, 1, 1.339844, 0 } },
{ "Minolta",
  "DiMAGE A1",
  QObject::tr("Shade"),
  { 2.253906, 1, 1.199219, 0 } },
{ "Minolta",
  "DiMAGE A1",
  QObject::tr("Flash"),
  { 1.972656, 1, 1.265625, 0 } },

// Minolta , DiMAGE A2
{ "Minolta",
  "DiMAGE A2",
  QObject::tr("Cloudy"),
  { 2.203125, 1, 1.296875, 0 } },
{ "Minolta",
  "DiMAGE A2",
  QObject::tr("Daylight"),
  { 1.960938, 1, 1.402344, 0 } },
{ "Minolta",
  "DiMAGE A2",
  QObject::tr("Flash"),
  { 2.039062, 1, 1.332031, 0 } },
{ "Minolta",
  "DiMAGE A2",
  QObject::tr("Fluorescent"),
  { 1.722656, 1, 2.132812, 0 } },
{ "Minolta",
  "DiMAGE A2",
  QObject::tr("Shade"),
  { 2.367188, 1, 1.265625, 0 } },
{ "Minolta",
  "DiMAGE A2",
  QObject::tr("Tungsten"),
  { 1.097656, 1, 2.882812, 0 } },

// Minolta , DiMAGE Z2
{ "Minolta",
  "DiMAGE Z2",
  QObject::tr("Daylight"),
  { 1.843749, 1, 1.664062, 0 } },
{ "Minolta",
  "DiMAGE Z2",
  QObject::tr("Cloudy"),
  { 2.195312, 1, 1.449218, 0 } },
{ "Minolta",
  "DiMAGE Z2",
  QObject::tr("Tungsten"),
  { 1.097656, 1, 3.050780, 0 } },
{ "Minolta",
  "DiMAGE Z2",
  QObject::tr("Fluorescent"),
  { 1.796874, 1, 2.257810, 0 } },
{ "Minolta",
  "DiMAGE Z2",
  QObject::tr("Flash"),
  { 2.117186, 1, 1.472656, 0 } },

// Minolta , DiMAGE G500
{ "Minolta",
  "DiMAGE G500",
  QObject::tr("Daylight"),
  { 1.496094, 1, 1.121094, 0 } },
{ "Minolta",
  "DiMAGE G500",
  QObject::tr("Cloudy"),
  { 1.527344, 1, 1.105469, 0 } },
{ "Minolta",
  "DiMAGE G500",
  QObject::tr("Fluorescent"),
  { 1.382813, 1, 1.347656, 0 } },
{ "Minolta",
  "DiMAGE G500",
  QObject::tr("Tungsten"),
  { 1.042969, 1, 1.859375, 0 } },
{ "Minolta",
  "DiMAGE G500",
  QObject::tr("Flash"),
  { 1.647078, 1, 1.218159, 0 } },

// MINOLTA , DYNAX 5D
{ "MINOLTA",
  "DYNAX 5D",
  QObject::tr("Daylight"),
  { 1.757812, 1, 1.636719, 0 } },
{ "MINOLTA",
  "DYNAX 5D",
  QObject::tr("Shade"),
  { 2.011719, 1, 1.433594, 0 } },
{ "MINOLTA",
  "DYNAX 5D",
  QObject::tr("Cloudy"),
  { 1.886719, 1, 1.500000, 0 } },
{ "MINOLTA",
  "DYNAX 5D",
  QObject::tr("Tungsten"),
  { 1.023438, 1, 3.496094, 0 } },
{ "MINOLTA",
  "DYNAX 5D",
  QObject::tr("Fluorescent"),
  { 1.703125, 1, 2.582031, 0 } },
{ "MINOLTA",
  "DYNAX 5D",
  QObject::tr("Flash"),
  { 1.917969, 1, 1.457031, 0 } },

// MINOLTA , DYNAX 7D
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("Daylight"),
  { 1.621094, 1, 1.601562, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("Shade"),
  { 1.855469, 1, 1.402344, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("Cloudy"),
  { 1.738281, 1, 1.464844, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("Tungsten"),
  { 0.945312, 1, 3.292969, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("Fluorescent"),
  { 1.570312, 1, 2.453125, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("Flash"),
  { 1.890625, 1, 1.445312, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("2500K"),
  { 1, 1.207547, 4.801887, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("2600K"),
  { 1, 1.153153, 4.297297, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("2700K"),
  { 1, 1.089362, 3.829787, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("2800K"),
  { 1, 1.044898, 3.477551, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("2900K"),
  { 1, 1.007874, 3.173228, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("3000K"),
  { 1.031250, 1, 3.000000, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("3100K"),
  { 1.066406, 1, 2.875000, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("3200K"),
  { 1.109375, 1, 2.765625, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("3300K"),
  { 1.144531, 1, 2.648438, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("3400K"),
  { 1.175781, 1, 2.554688, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("3500K"),
  { 1.207031, 1, 2.468750, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("3600K"),
  { 1.242188, 1, 2.390625, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("3700K"),
  { 1.277344, 1, 2.312500, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("3800K"),
  { 1.304688, 1, 2.242188, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("3900K"),
  { 1.339844, 1, 2.179688, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("4000K"),
  { 1.363281, 1, 2.125000, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("4100K"),
  { 1.390625, 1, 2.078125, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("4200K"),
  { 1.421875, 1, 2.023438, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("4300K"),
  { 1.445312, 1, 1.976562, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("4400K"),
  { 1.476562, 1, 1.937500, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("4500K"),
  { 1.500000, 1, 1.894531, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("4600K"),
  { 1.527344, 1, 1.855469, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("4700K"),
  { 1.542969, 1, 1.824219, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("4800K"),
  { 1.566406, 1, 1.785156, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("4900K"),
  { 1.593750, 1, 1.757812, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("5000K"),
  { 1.609375, 1, 1.726562, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("5100K"),
  { 1.636719, 1, 1.699219, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("5200K"),
  { 1.656250, 1, 1.671875, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("5300K"),
  { 1.671875, 1, 1.644531, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("5400K"),
  { 1.691406, 1, 1.621094, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("5500K"),
  { 1.710938, 1, 1.601562, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("5600K"),
  { 1.726562, 1, 1.585938, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("5700K"),
  { 1.757812, 1, 1.558594, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("5800K"),
  { 1.765625, 1, 1.535156, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("5900K"),
  { 1.785156, 1, 1.515625, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("6000K"),
  { 1.792969, 1, 1.500000, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("6100K"),
  { 1.812500, 1, 1.484375, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("6200K"),
  { 1.835938, 1, 1.468750, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("6300K"),
  { 1.843750, 1, 1.453125, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("6400K"),
  { 1.863281, 1, 1.437500, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("6500K"),
  { 1.875000, 1, 1.421875, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("6600K"),
  { 1.894531, 1, 1.414062, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("6700K"),
  { 1.914062, 1, 1.398438, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("6800K"),
  { 1.925781, 1, 1.382812, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("6900K"),
  { 1.937500, 1, 1.375000, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("7000K"),
  { 1.945312, 1, 1.363281, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("7100K"),
  { 1.957031, 1, 1.347656, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("7200K"),
  { 1.976562, 1, 1.339844, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("7300K"),
  { 1.988281, 1, 1.324219, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("7400K"),
  { 2.000000, 1, 1.316406, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("7500K"),
  { 2.007812, 1, 1.304688, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("7600K"),
  { 2.023438, 1, 1.304688, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("7700K"),
  { 2.031250, 1, 1.289062, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("7800K"),
  { 2.046875, 1, 1.277344, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("7900K"),
  { 2.054688, 1, 1.277344, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("8000K"),
  { 2.062500, 1, 1.261719, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("8100K"),
  { 2.085938, 1, 1.253906, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("8200K"),
  { 2.085938, 1, 1.250000, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("8300K"),
  { 2.101562, 1, 1.234375, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("8400K"),
  { 2.109375, 1, 1.234375, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("8500K"),
  { 2.125000, 1, 1.226562, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("8600K"),
  { 2.132812, 1, 1.214844, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("8700K"),
  { 2.132812, 1, 1.207031, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("8800K"),
  { 2.148438, 1, 1.207031, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("8900K"),
  { 2.156250, 1, 1.195312, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("9000K"),
  { 2.171875, 1, 1.187500, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("9100K"),
  { 2.179688, 1, 1.187500, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("9200K"),
  { 2.179688, 1, 1.183594, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("9300K"),
  { 2.195312, 1, 1.175781, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("9400K"),
  { 2.203125, 1, 1.171875, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("9500K"),
  { 2.218750, 1, 1.164062, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("9600K"),
  { 2.218750, 1, 1.156250, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("9700K"),
  { 2.226562, 1, 1.152344, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("9800K"),
  { 2.226562, 1, 1.144531, 0 } },
{ "MINOLTA",
  "DYNAX 7D",
  QObject::tr("9900K"),
  { 2.242188, 1, 1.144531, 0 } },

// NIKON , COOLPIX P7100
{ "NIKON",
  "COOLPIX P7100",
  QObject::tr("DirectSunlight"),
  { 1.6500, 1, 2.1349, 0 } },
{ "NIKON",
  "COOLPIX P7100",
  QObject::tr("Incandescent"),
  { 1.1730, 1, 3.3705, 0 } },
{ "NIKON",
  "COOLPIX P7100",
  QObject::tr("WhiteFluorescent"),
  { 1.8456, 1, 2.9326, 0 } },
{ "NIKON",
  "COOLPIX P7100",
  QObject::tr("DayWhiteFluorescent"),
  { 1.8182, 1, 2.1310, 0 } },
{ "NIKON",
  "COOLPIX P7100",
  QObject::tr("DaylightFluorescent"),
  { 2.1271, 1, 1.7517, 0 } },
{ "NIKON",
  "COOLPIX P7100",
  QObject::tr("Cloudy"),
  { 1.8182, 1, 1.8534, 0 } },
{ "NIKON",
  "COOLPIX P7100",
  QObject::tr("Flash"),
  { 2.1349, 1, 1.6579, 0 } },

// NIKON , D1
{ "NIKON",
  "D1",
  QObject::tr("Incandescent"),
  { 1, 1.582583, 2.556096, 0 } },
{ "NIKON",
  "D1",
  QObject::tr("Fluorescent"),
  { 1, 1.077710, 1.672660, 0 } },
{ "NIKON",
  "D1",
  QObject::tr("DirectSunlight"),
  { 1.000000, 1.000000, 1, 0 } },
{ "NIKON",
  "D1",
  QObject::tr("Flash"),
  { 1.235772, 1.078231, 1, 0 } },
{ "NIKON",
  "D1",
  QObject::tr("Cloudy"),
  { 1.162116, 1.078231, 1, 0 } },
{ "NIKON",
  "D1",
  QObject::tr("Shade"),
  { 1.284963, 1.136201, 1, 0 } },

// NIKON , D1H
{ "NIKON",
  "D1H",
  QObject::tr("Incandescent"),
  { 1.363281, 1, 1.996094, 0 } },
{ "NIKON",
  "D1H",
  QObject::tr("Fluorescent"),
  { 1.925781, 1, 2.054688, 0 } },
{ "NIKON",
  "D1H",
  QObject::tr("DirectSunlight"),
  { 2.148438, 1, 1.246094, 0 } },
{ "NIKON",
  "D1H",
  QObject::tr("Flash"),
  { 2.347656, 1, 1.140625, 0 } },
{ "NIKON",
  "D1H",
  QObject::tr("Cloudy"),
  { 2.300781, 1, 1.128906, 0 } },
{ "NIKON",
  "D1H",
  QObject::tr("Shade"),
  { 2.628906, 1, 1.011719, 0 } },

// NIKON , D1X
{ "NIKON",
  "D1X",
  QObject::tr("Incandescent"),
  { 1.363281, 1, 1.996094, 0 } },
{ "NIKON",
  "D1X",
  QObject::tr("Fluorescent"),
  { 1.925781, 1, 2.054688, 0 } },
{ "NIKON",
  "D1X",
  QObject::tr("DirectSunlight"),
  { 2.148438, 1, 1.246094, 0 } },
{ "NIKON",
  "D1X",
  QObject::tr("Flash"),
  { 2.347656, 1, 1.140625, 0 } },
{ "NIKON",
  "D1X",
  QObject::tr("Cloudy"),
  { 2.300781, 1, 1.128906, 0 } },
{ "NIKON",
  "D1X",
  QObject::tr("Shade"),
  { 2.628906, 1, 1.011719, 0 } },

// NIKON , D2X
{ "NIKON",
  "D2X",
  QObject::tr("Incandescent"),
  { 0.92086, 1, 2.89928, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("Fluorescent"),
  { 1.42969, 1, 2.62891, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("DirectSunlight"),
  { 1.52734, 1, 1.69531, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("Flash"),
  { 1.66016, 1, 1.53125, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("Cloudy"),
  { 1.69141, 1, 1.54688, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("Shade"),
  { 1.98047, 1, 1.32812, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("2500K"),
  { 0.74203, 1, 3.67536, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("2550K"),
  { 0.76877, 1, 3.58859, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("2650K"),
  { 0.81529, 1, 3.42675, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("2700K"),
  { 0.83388, 1, 3.34528, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("2800K"),
  { 0.86486, 1, 3.17905, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("2850K"),
  { 0.87973, 1, 3.10309, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("2950K"),
  { 0.90780, 1, 2.96454, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("3000K"),
  { 0.92086, 1, 2.89928, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("3100K"),
  { 0.94465, 1, 2.77122, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("3200K"),
  { 0.96970, 1, 2.65530, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("3300K"),
  { 0.99611, 1, 2.55642, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("3400K"),
  { 1.01953, 1, 2.46484, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("3600K"),
  { 1.07422, 1, 2.34375, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("3700K"),
  { 1.09766, 1, 2.26172, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("3800K"),
  { 1.12500, 1, 2.18750, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("4000K"),
  { 1.17969, 1, 2.06250, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("4200K"),
  { 1.24219, 1, 1.96094, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("4300K"),
  { 1.27344, 1, 1.91797, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("4500K"),
  { 1.33594, 1, 1.83984, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("4800K"),
  { 1.42578, 1, 1.78906, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("5000K"),
  { 1.48438, 1, 1.74609, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("5300K"),
  { 1.55078, 1, 1.67578, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("5600K"),
  { 1.61328, 1, 1.61328, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("5900K"),
  { 1.67188, 1, 1.56250, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("6300K"),
  { 1.74219, 1, 1.50391, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("6700K"),
  { 1.80859, 1, 1.45703, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("7100K"),
  { 1.86719, 1, 1.41406, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("7700K"),
  { 1.94531, 1, 1.35547, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("8300K"),
  { 2.01562, 1, 1.30469, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("9100K"),
  { 2.09766, 1, 1.24609, 0 } },
{ "NIKON",
  "D2X",
  QObject::tr("10000K"),
  { 2.17578, 1, 1.18359, 0 } },

// NIKON , D3
{ "NIKON",
  "D3",
  QObject::tr("Daylight"),
  { 1.81640, 1, 1.35546, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("Flash"),
  { 2.03906, 1, 1.17187, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("Cloudy"),
  { 1.94921, 1, 1.22265, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("Shade"),
  { 2.24609, 1, 1.08593, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("Incandescent"),
  { 1.16796, 1, 2.31640, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("Fluorescent"),
  { 1.68750, 1, 2.10156, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("2500K"),
  { 1.00390, 1, 3.00000, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("2560K"),
  { 1.02343, 1, 2.89062, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("2630K"),
  { 1.04296, 1, 2.78125, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("2700K"),
  { 1.06640, 1, 2.67968, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("2780K"),
  { 1.09375, 1, 2.57812, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("2860K"),
  { 1.11718, 1, 2.47656, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("2940K"),
  { 1.14843, 1, 2.38281, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("3000K"),
  { 1.16796, 1, 2.31640, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("3030K"),
  { 1.17578, 1, 2.28906, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("3130K"),
  { 1.20703, 1, 2.19921, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("3230K"),
  { 1.24218, 1, 2.10937, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("3330K"),
  { 1.27734, 1, 2.02343, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("3450K"),
  { 1.31350, 1, 1.94140, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("3570K"),
  { 1.35156, 1, 1.85937, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("3700K"),
  { 1.39062, 1, 1.78125, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("3850K"),
  { 1.43359, 1, 1.70703, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("4000K"),
  { 1.47656, 1, 1.63281, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("4170K"),
  { 1.52343, 1, 1.56640, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("4350K"),
  { 1.60156, 1, 1.55078, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("4550K"),
  { 1.66406, 1, 1.51562, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("4760K"),
  { 1.72265, 1, 1.46093, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("5000K"),
  { 1.77734, 1, 1.40234, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("5200K"),
  { 1.81640, 1, 1.35546, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("5260K"),
  { 1.82812, 1, 1.34375, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("5560K"),
  { 1.87890, 1, 1.28515, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("5880K"),
  { 1.93359, 1, 1.23437, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("6000K"),
  { 1.94921, 1, 1.22265, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("6250K"),
  { 1.99218, 1, 1.19140, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("6400K"),
  { 2.03906, 1, 1.17187, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("6670K"),
  { 2.05468, 1, 1.15625, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("7140K"),
  { 2.12500, 1, 1.12500, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("7690K"),
  { 2.20312, 1, 1.09765, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("8000K"),
  { 2.24609, 1, 1.08593, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("8330K"),
  { 2.28906, 1, 1.07031, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("9090K"),
  { 2.38281, 1, 1.03515, 0 } },
{ "NIKON",
  "D3",
  QObject::tr("10000K"),
  { 2.48046, 1, 1.00000, 0 } },

// NIKON , D3S
{ "NIKON",
  "D3S",
  QObject::tr("Incandescent"),
  { 1.191406, 1, 2.242188, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("SodiumVaporFluorescent"),
  { 1.132812, 1, 2.511719, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.179688, 1, 1.996094, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("WhiteFluorescent"),
  { 1.394531, 1, 2.402344, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.703125, 1, 2.066406, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("DayWhiteFluorescent"),
  { 1.710938, 1, 1.390625, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("DaylightFluorescent"),
  { 1.941406, 1, 1.113281, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("HighTempMercuryVaporFluorescent"),
  { 2.289062, 1, 1.355469, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("DirectSunlight"),
  { 1.835938, 1, 1.359375, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("Flash"),
  { 2.035156, 1, 1.183594, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("Cloudy"),
  { 1.964844, 1, 1.226562, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("Shade"),
  { 2.253906, 1, 1.089844, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("2500K"),
  { 1.031250, 1, 2.851562, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("2560K"),
  { 1.050781, 1, 2.753906, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("2630K"),
  { 1.070312, 1, 2.656250, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("2700K"),
  { 1.093750, 1, 2.558594, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("2780K"),
  { 1.117188, 1, 2.468750, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("2860K"),
  { 1.144531, 1, 2.382812, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("2940K"),
  { 1.171875, 1, 2.300781, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("3030K"),
  { 1.199219, 1, 2.214844, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("3130K"),
  { 1.230469, 1, 2.125000, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("3230K"),
  { 1.265625, 1, 2.050781, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("3330K"),
  { 1.296875, 1, 1.984375, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("3450K"),
  { 1.335938, 1, 1.921875, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("3570K"),
  { 1.375000, 1, 1.843750, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("3700K"),
  { 1.414062, 1, 1.769531, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("3850K"),
  { 1.453125, 1, 1.695312, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("4000K"),
  { 1.500000, 1, 1.628906, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("4170K"),
  { 1.542969, 1, 1.562500, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("4350K"),
  { 1.621094, 1, 1.550781, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("4550K"),
  { 1.687500, 1, 1.511719, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("4760K"),
  { 1.742188, 1, 1.460938, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("5000K"),
  { 1.796875, 1, 1.402344, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("5260K"),
  { 1.847656, 1, 1.343750, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("5560K"),
  { 1.894531, 1, 1.292969, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("5880K"),
  { 1.949219, 1, 1.242188, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("6250K"),
  { 2.003906, 1, 1.199219, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("6670K"),
  { 2.066406, 1, 1.160156, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("7140K"),
  { 2.140625, 1, 1.132812, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("7690K"),
  { 2.214844, 1, 1.101562, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("8330K"),
  { 2.292969, 1, 1.070312, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("9090K"),
  { 2.390625, 1, 1.046875, 0 } },
{ "NIKON",
  "D3S",
  QObject::tr("10000K"),
  { 2.492188, 1, 1.003906, 0 } },

// NIKON , D3X
{ "NIKON",
  "D3X",
  QObject::tr("Incandescent"),
  { 1.367188, 1, 2.292969, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("Fluorescent"),
  { 1.984375, 1, 2.128906, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("DirectSunlight"),
  { 2.156250, 1, 1.417969, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("Flash"),
  { 2.402344, 1, 1.242187, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("Cloudy"),
  { 2.328125, 1, 1.289062, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("Shade"),
  { 2.707031, 1, 1.160156, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("2500K"),
  { 1.179688, 1, 2.898438, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("2560K"),
  { 1.203125, 1, 2.796875, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("2630K"),
  { 1.226563, 1, 2.699219, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("2700K"),
  { 1.253906, 1, 2.605469, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("2780K"),
  { 1.281250, 1, 2.519531, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("2860K"),
  { 1.312500, 1, 2.429688, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("2940K"),
  { 1.343750, 1, 2.347656, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("3030K"),
  { 1.378906, 1, 2.269531, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("3130K"),
  { 1.414063, 1, 2.187500, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("3230K"),
  { 1.453125, 1, 2.097656, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("3330K"),
  { 1.492187, 1, 2.015625, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("3450K"),
  { 1.539062, 1, 1.933594, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("3570K"),
  { 1.585937, 1, 1.859375, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("3700K"),
  { 1.636719, 1, 1.792969, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("3850K"),
  { 1.695312, 1, 1.734375, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("4000K"),
  { 1.753906, 1, 1.683594, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("4170K"),
  { 1.824219, 1, 1.636719, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("4350K"),
  { 1.902344, 1, 1.593750, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("4550K"),
  { 1.976562, 1, 1.554687, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("4760K"),
  { 2.042969, 1, 1.511719, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("5000K"),
  { 2.105469, 1, 1.460938, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("5260K"),
  { 2.167969, 1, 1.406250, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("5560K"),
  { 2.234375, 1, 1.351563, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("5880K"),
  { 2.304688, 1, 1.300781, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("6250K"),
  { 2.378906, 1, 1.257813, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("6670K"),
  { 2.464844, 1, 1.226562, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("7140K"),
  { 2.554687, 1, 1.199219, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("7690K"),
  { 2.652344, 1, 1.171875, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("8330K"),
  { 2.761719, 1, 1.144531, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("9090K"),
  { 2.878906, 1, 1.113281, 0 } },
{ "NIKON",
  "D3X",
  QObject::tr("10000K"),
  { 3.000000, 1, 1.062500, 0 } },

// NIKON , D4
{ "NIKON",
  "D4",
  QObject::tr("Incandescent"),
  { 1.281250, 1, 2.371094, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("SodiumVaporFluorescent"),
  { 1.195313, 1, 2.589844, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.277344, 1, 2.109375, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("WhiteFluorescent"),
  { 1.488281, 1, 2.476563, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.855469, 1, 2.156250, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("DayWhiteFluorescent"),
  { 1.867188, 1, 1.476563, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("DaylightFluorescent"),
  { 2.132813, 1, 1.156250, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("HighTempMercuryVaporFluorescent"),
  { 2.546875, 1, 1.425781, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("DirectSunlight"),
  { 2.019531, 1, 1.437500, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("Flash"),
  { 2.300781, 1, 1.253906, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("Cloudy"),
  { 2.175781, 1, 1.300781, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("Shade"),
  { 2.511719, 1, 1.164063, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("2500K"),
  { 1.097656, 1, 3.027344, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("2600K"),
  { 1.132813, 1, 2.859375, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("2700K"),
  { 1.167969, 1, 2.710938, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("2800K"),
  { 1.207031, 1, 2.582031, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("2900K"),
  { 1.246094, 1, 2.464844, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("3000K"),
  { 1.281250, 1, 2.363281, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("3200K"),
  { 1.355469, 1, 2.195313, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("3400K"),
  { 1.425781, 1, 2.054688, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("3600K"),
  { 1.496094, 1, 1.929688, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("3800K"),
  { 1.558594, 1, 1.812500, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("4000K"),
  { 1.628906, 1, 1.718750, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("4200K"),
  { 1.703125, 1, 1.660156, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("4400K"),
  { 1.781250, 1, 1.621094, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("4600K"),
  { 1.859375, 1, 1.589844, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("4800K"),
  { 1.921875, 1, 1.539063, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("5000K"),
  { 1.972656, 1, 1.488281, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("5300K"),
  { 2.039063, 1, 1.417969, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("5600K"),
  { 2.097656, 1, 1.359375, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("5900K"),
  { 2.152344, 1, 1.308594, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("6200K"),
  { 2.207031, 1, 1.273438, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("6500K"),
  { 2.261719, 1, 1.242188, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("6800K"),
  { 2.312500, 1, 1.222656, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("7100K"),
  { 2.363281, 1, 1.203125, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("7500K"),
  { 2.433594, 1, 1.183594, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("8000K"),
  { 2.511719, 1, 1.160156, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("8500K"),
  { 2.589844, 1, 1.140625, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("9000K"),
  { 2.660156, 1, 1.117188, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("9500K"),
  { 2.730469, 1, 1.093750, 0 } },
{ "NIKON",
  "D4",
  QObject::tr("10000K"),
  { 2.792969, 1, 1.066406, 0 } },

// NIKON , D100
{ "NIKON",
  "D100",
  QObject::tr("Incandescent"),
  { 1.406250, 1, 2.828125, 0 } },
{ "NIKON",
  "D100",
  QObject::tr("Fluorescent"),
  { 2.058594, 1, 2.617187, 0 } },
{ "NIKON",
  "D100",
  QObject::tr("DirectSunlight"),
  { 2.257812, 1, 1.757812, 0 } },
{ "NIKON",
  "D100",
  QObject::tr("Flash"),
  { 2.539062, 1, 1.539062, 0 } },
{ "NIKON",
  "D100",
  QObject::tr("Cloudy"),
  { 2.507813, 1, 1.628906, 0 } },
{ "NIKON",
  "D100",
  QObject::tr("Shade"),
  { 2.906250, 1, 1.437500, 0 } },

// NIKON , D200
{ "NIKON",
  "D200",
  QObject::tr("Incandescent"),
  { 1.148437, 1, 2.398438, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("Fluorescent"),
  { 1.664062, 1, 2.148437, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("DirectSunlight"),
  { 1.804688, 1, 1.398437, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("Flash"),
  { 2.007813, 1, 1.171875, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("Cloudy"),
  { 1.917969, 1, 1.261719, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("Shade"),
  { 2.234375, 1, 1.125000, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("2500K"),
  { 1.000000, 1, 3.121094, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("2550K"),
  { 1.000000, 1, 3.035156, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("2650K"),
  { 1.011719, 1, 2.878906, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("2700K"),
  { 1.031250, 1, 2.804688, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("2800K"),
  { 1.074219, 1, 2.648438, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("2850K"),
  { 1.089844, 1, 2.589844, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("2950K"),
  { 1.132813, 1, 2.453125, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("3000K"),
  { 1.148438, 1, 2.398438, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("3100K"),
  { 1.183594, 1, 2.289063, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("3200K"),
  { 1.218750, 1, 2.187500, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("3300K"),
  { 1.250000, 1, 2.097656, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("3400K"),
  { 1.281250, 1, 2.015625, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("3600K"),
  { 1.343750, 1, 1.871094, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("3700K"),
  { 1.371094, 1, 1.820313, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("3800K"),
  { 1.402344, 1, 1.761719, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("4000K"),
  { 1.457031, 1, 1.667969, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("4200K"),
  { 1.511719, 1, 1.593750, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("4300K"),
  { 1.535156, 1, 1.558594, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("4500K"),
  { 1.589844, 1, 1.500000, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("4800K"),
  { 1.687500, 1, 1.449219, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("5000K"),
  { 1.746094, 1, 1.425781, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("5300K"),
  { 1.820313, 1, 1.375000, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("5600K"),
  { 1.863281, 1, 1.320313, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("5900K"),
  { 1.902344, 1, 1.273438, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("6300K"),
  { 1.972656, 1, 1.234375, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("6700K"),
  { 2.046875, 1, 1.199219, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("7100K"),
  { 2.105469, 1, 1.175781, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("7700K"),
  { 2.191406, 1, 1.144531, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("8300K"),
  { 2.277344, 1, 1.109375, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("9300K"),
  { 2.367188, 1, 1.070313, 0 } },
{ "NIKON",
  "D200",
  QObject::tr("10000K"),
  { 2.453125, 1, 1.035156, 0 } },

// NIKON , D300
{ "NIKON",
  "D300",
  QObject::tr("Incandescent"),
  { 1.019531, 1, 2.109375, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("SodiumVaporFluorescent"),
  { 0.949219, 1, 2.320313, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.046875, 1, 1.960938, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("WhiteFluorescent"),
  { 1.191406, 1, 2.304688, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.468750, 1, 2.035156, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("DayWhiteFluorescent"),
  { 1.492188, 1, 1.363281, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("DaylightFluorescent"),
  { 1.699219, 1, 1.101563, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("HighTempMercuryVaporFluorescent"),
  { 1.960938, 1, 1.328125, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("DirectSunlight"),
  { 1.562500, 1, 1.308594, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("Flash"),
  { 1.687500, 1, 1.152344, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("Cloudy"),
  { 1.664063, 1, 1.187500, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("Shade"),
  { 1.902344, 1, 1.066406, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("2500K"),
  { 0.894531, 1, 2.632813, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("2560K"),
  { 0.906250, 1, 2.550781, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("2630K"),
  { 0.921875, 1, 2.468750, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("2700K"),
  { 0.941406, 1, 2.390625, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("2780K"),
  { 0.960938, 1, 2.312500, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("2860K"),
  { 0.980469, 1, 2.234375, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("2940K"),
  { 1.003906, 1, 2.160156, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("3030K"),
  { 1.027344, 1, 2.085938, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("3130K"),
  { 1.050781, 1, 2.015625, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("3230K"),
  { 1.078125, 1, 1.945313, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("3330K"),
  { 1.109375, 1, 1.875000, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("3450K"),
  { 1.136719, 1, 1.808594, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("3570K"),
  { 1.167969, 1, 1.742188, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("3700K"),
  { 1.203125, 1, 1.679688, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("3850K"),
  { 1.238281, 1, 1.617188, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("4000K"),
  { 1.277344, 1, 1.554688, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("4170K"),
  { 1.316406, 1, 1.500000, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("4350K"),
  { 1.386719, 1, 1.484375, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("4550K"),
  { 1.441406, 1, 1.449219, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("4760K"),
  { 1.488281, 1, 1.402344, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("5000K"),
  { 1.531250, 1, 1.351563, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("5260K"),
  { 1.570313, 1, 1.296875, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("5560K"),
  { 1.613281, 1, 1.246094, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("5880K"),
  { 1.652344, 1, 1.199219, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("6250K"),
  { 1.695313, 1, 1.160156, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("6670K"),
  { 1.746094, 1, 1.125000, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("7140K"),
  { 1.804688, 1, 1.097656, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("7690K"),
  { 1.867188, 1, 1.074219, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("8330K"),
  { 1.937500, 1, 1.054688, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("9090K"),
  { 2.019531, 1, 1.027344, 0 } },
{ "NIKON",
  "D300",
  QObject::tr("10000K"),
  { 2.109375, 1, 1.000000, 0 } },

// NIKON , D300S
{ "NIKON",
  "D300S",
  QObject::tr("DirectSunlight"),
  { 1.563, 1, 1.309, 0 } },
{ "NIKON",
  "D300S",
  QObject::tr("Flash"),
  { 1.687, 1, 1.152, 0 } },
{ "NIKON",
  "D300S",
  QObject::tr("Cloudy"),
  { 1.664, 1, 1.187, 0 } },
{ "NIKON",
  "D300S",
  QObject::tr("Shade"),
  { 1.902, 1, 1.066, 0 } },
{ "NIKON",
  "D300S",
  QObject::tr("Incandescent"),
  { 1.020, 1, 2.109, 0 } },
{ "NIKON",
  "D300S",
  QObject::tr("SodiumVaporFluorescent"),
  { 1, 1.053, 2.444, 0 } },
{ "NIKON",
  "D300S",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.047, 1, 1.961, 0 } },
{ "NIKON",
  "D300S",
  QObject::tr("WhiteFluorescent"),
  { 1.191, 1, 2.305, 0 } },
{ "NIKON",
  "D300S",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.469, 1, 2.035, 0 } },
{ "NIKON",
  "D300S",
  QObject::tr("DayWhiteFluorescent"),
  { 1.492, 1, 1.363, 0 } },
{ "NIKON",
  "D300S",
  QObject::tr("DaylightFluorescent"),
  { 1.699, 1, 1.102, 0 } },
{ "NIKON",
  "D300S",
  QObject::tr("HighTempMercuryVaporFluorescent"),
  { 1.961, 1, 1.328, 0 } },

// NIKON , D700
{ "NIKON",
  "D700",
  QObject::tr("DirectSunlight"),
  { 1.816406, 1, 1.355469, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("Flash"),
  { 2.039063, 1, 1.171875, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("Cloudy"),
  { 1.949219, 1, 1.222656, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("Shade"),
  { 2.246094, 1, 1.085937, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("Incandescent"),
  { 1.167969, 1, 2.316406, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("SodiumVaporFluorescent"),
  { 1.062500, 1, 2.464844, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.167969, 1, 2.109375, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("WhiteFluorescent"),
  { 1.363281, 1, 2.425781, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.687500, 1, 2.101563, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("DayWhiteFluorescent"),
  { 1.710938, 1, 1.410156, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("DaylightFluorescent"),
  { 1.953125, 1, 1.113281, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("HighTempMercuryVaporFluorescent"),
  { 2.289063, 1, 1.363281, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("2500K"),
  { 1.003906, 1, 3.000000, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("2560K"),
  { 1.023438, 1, 2.890625, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("2630K"),
  { 1.042969, 1, 2.781250, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("2700K"),
  { 1.066406, 1, 2.679687, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("2780K"),
  { 1.093750, 1, 2.578125, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("2860K"),
  { 1.117187, 1, 2.476562, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("2940K"),
  { 1.148437, 1, 2.382812, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("3030K"),
  { 1.175781, 1, 2.289063, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("3130K"),
  { 1.207031, 1, 2.199219, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("3230K"),
  { 1.242188, 1, 2.109375, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("3330K"),
  { 1.277344, 1, 2.023438, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("3450K"),
  { 1.312500, 1, 1.941406, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("3570K"),
  { 1.351562, 1, 1.859375, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("3700K"),
  { 1.390625, 1, 1.781250, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("3850K"),
  { 1.433594, 1, 1.707031, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("4000K"),
  { 1.476563, 1, 1.632813, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("4170K"),
  { 1.523437, 1, 1.566406, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("4350K"),
  { 1.601562, 1, 1.550781, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("4760K"),
  { 1.722656, 1, 1.460938, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("5000K"),
  { 1.777344, 1, 1.402344, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("5260K"),
  { 1.828125, 1, 1.343750, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("5560K"),
  { 1.878906, 1, 1.285156, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("5880K"),
  { 1.933594, 1, 1.234375, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("6250K"),
  { 1.992187, 1, 1.191406, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("6670K"),
  { 2.054688, 1, 1.156250, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("7140K"),
  { 2.125000, 1, 1.125000, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("7690K"),
  { 2.203125, 1, 1.097656, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("8330K"),
  { 2.289063, 1, 1.070313, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("9090K"),
  { 2.382812, 1, 1.035156, 0 } },
{ "NIKON",
  "D700",
  QObject::tr("10000K"),
  { 2.480469, 1, 1.000000, 0 } },

// NIKON , D800
{ "NIKON",
  "D800",
  QObject::tr("Incandescent"),
  { 1.289063, 1, 2.175781, 0 } },
{ "NIKON",
  "D800",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.765625, 1, 2.003906, 0 } },
{ "NIKON",
  "D800",
  QObject::tr("DirectSunlight"),
  { 1.972656, 1, 1.375000, 0 } },
{ "NIKON",
  "D800",
  QObject::tr("Flash"),
  { 2.187500, 1, 1.207031, 0 } },
{ "NIKON",
  "D800",
  QObject::tr("Cloudy"),
  { 2.101563, 1, 1.253906, 0 } },
{ "NIKON",
  "D800",
  QObject::tr("Shade"),
  { 2.378906, 1, 1.128906, 0 } },

// NIKON , D40
{ "NIKON",
  "D40",
  QObject::tr("Incandescent"),
  { 1.375000, 1, 2.511719, 0 } },
{ "NIKON",
  "D40",
  QObject::tr("Fluorescent"),
  { 2.007813, 1, 2.269531, 0 } },
{ "NIKON",
  "D40",
  QObject::tr("DirectSunlight"),
  { 2.195313, 1, 1.421875, 0 } },
{ "NIKON",
  "D40",
  QObject::tr("Flash"),
  { 2.464844, 1, 1.281250, 0 } },
{ "NIKON",
  "D40",
  QObject::tr("Cloudy"),
  { 2.429688, 1, 1.320313, 0 } },
{ "NIKON",
  "D40",
  QObject::tr("Shade"),
  { 2.820313, 1, 1.125000, 0 } },

// NIKON , D40X
{ "NIKON",
  "D40X",
  QObject::tr("Incandescent"),
  { 1.148438, 1, 2.386719, 0 } },
{ "NIKON",
  "D40X",
  QObject::tr("Fluorescent"),
  { 1.683594, 1, 2.117188, 0 } },
{ "NIKON",
  "D40X",
  QObject::tr("DirectSunlight"),
  { 1.792969, 1, 1.371094, 0 } },
{ "NIKON",
  "D40X",
  QObject::tr("Flash"),
  { 1.949219, 1, 1.187500, 0 } },
{ "NIKON",
  "D40X",
  QObject::tr("Cloudy"),
  { 1.960938, 1, 1.253906, 0 } },
{ "NIKON",
  "D40X",
  QObject::tr("Shade"),
  { 2.277344, 1, 1.089844, 0 } },

// NIKON , D50
{ "NIKON",
  "D50",
  QObject::tr("Incandescent"),
  { 1.328125, 1, 2.500000, 0 } },
{ "NIKON",
  "D50",
  QObject::tr("Fluorescent"),
  { 1.945312, 1, 2.191406, 0 } },
{ "NIKON",
  "D50",
  QObject::tr("DirectSunlight"),
  { 2.140625, 1, 1.398438, 0 } },
{ "NIKON",
  "D50",
  QObject::tr("Flash"),
  { 2.398438, 1, 1.339844, 0 } },
{ "NIKON",
  "D50",
  QObject::tr("Cloudy"),
  { 2.360269, 1, 1.282828, 0 } },
{ "NIKON",
  "D50",
  QObject::tr("Shade"),
  { 2.746094, 1, 1.156250, 0 } },

// NIKON , D60
{ "NIKON",
  "D60",
  QObject::tr("DirectSunlight"),
  { 1.792969, 1, 1.371094, 0 } },
{ "NIKON",
  "D60",
  QObject::tr("Flash"),
  { 2.007813, 1, 1.187500, 0 } },
{ "NIKON",
  "D60",
  QObject::tr("Cloudy"),
  { 1.960937, 1, 1.253906, 0 } },
{ "NIKON",
  "D60",
  QObject::tr("Shade"),
  { 2.277344, 1, 1.089844, 0 } },
{ "NIKON",
  "D60",
  QObject::tr("Incandescent"),
  { 1.148437, 1, 2.382812, 0 } },
{ "NIKON",
  "D60",
  QObject::tr("SodiumVaporFluorescent"),
  { 1.035156, 1, 2.468750, 0 } },
{ "NIKON",
  "D60",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.136719, 1, 2.167969, 0 } },
{ "NIKON",
  "D60",
  QObject::tr("WhiteFluorescent"),
  { 1.343750, 1, 2.480469, 0 } },
{ "NIKON",
  "D60",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.683594, 1, 2.117187, 0 } },
{ "NIKON",
  "D60",
  QObject::tr("DayWhiteFluorescent"),
  { 1.679688, 1, 1.414063, 0 } },
{ "NIKON",
  "D60",
  QObject::tr("DaylightFluorescent"),
  { 1.953125, 1, 1.121094, 0 } },
{ "NIKON",
  "D60",
  QObject::tr("HighTempMercuryVaporFluorescent"),
  { 2.296875, 1, 1.398438, 0 } },

// NIKON , D70
{ "NIKON",
  "D70",
  QObject::tr("Incandescent"),
  { 1.343750, 1, 2.816406, 0 } },
{ "NIKON",
  "D70",
  QObject::tr("Fluorescent"),
  { 1.964844, 1, 2.476563, 0 } },
{ "NIKON",
  "D70",
  QObject::tr("DirectSunlight"),
  { 2.062500, 1, 1.597656, 0 } },
{ "NIKON",
  "D70",
  QObject::tr("Flash"),
  { 2.441406, 1, 1.500000, 0 } },
{ "NIKON",
  "D70",
  QObject::tr("Cloudy"),
  { 2.257813, 1, 1.457031, 0 } },
{ "NIKON",
  "D70",
  QObject::tr("Shade"),
  { 2.613281, 1, 1.277344, 0 } },

// NIKON , D70s
{ "NIKON",
  "D70s",
  QObject::tr("Incandescent"),
  { 1.343750, 1, 2.816406, 0 } },
{ "NIKON",
  "D70s",
  QObject::tr("Fluorescent"),
  { 1.964844, 1, 2.476563, 0 } },
{ "NIKON",
  "D70s",
  QObject::tr("DirectSunlight"),
  { 2.062500, 1, 1.597656, 0 } },
{ "NIKON",
  "D70s",
  QObject::tr("Flash"),
  { 2.441406, 1, 1.500000, 0 } },
{ "NIKON",
  "D70s",
  QObject::tr("Cloudy"),
  { 2.257813, 1, 1.457031, 0 } },
{ "NIKON",
  "D70s",
  QObject::tr("Shade"),
  { 2.613281, 1, 1.277344, 0 } },

// NIKON , D80
{ "NIKON",
  "D80",
  QObject::tr("Incandescent"),
  { 1.148438, 1, 2.386719, 0 } },
{ "NIKON",
  "D80",
  QObject::tr("Fluorescent"),
  { 1.683594, 1, 2.117188, 0 } },
{ "NIKON",
  "D80",
  QObject::tr("Daylight"),
  { 1.792969, 1, 1.371094, 0 } },
{ "NIKON",
  "D80",
  QObject::tr("Flash"),
  { 2.007812, 1, 1.242188, 0 } },
{ "NIKON",
  "D80",
  QObject::tr("Cloudy"),
  { 1.960938, 1, 1.253906, 0 } },
{ "NIKON",
  "D80",
  QObject::tr("Shade"),
  { 2.277344, 1, 1.089844, 0 } },
{ "NIKON",
  "D80",
  QObject::tr("4300K"),
  { 1.562500, 1, 1.523438, 0 } },
{ "NIKON",
  "D80",
  QObject::tr("5000K"),
  { 1.746094, 1, 1.410156, 0 } },
{ "NIKON",
  "D80",
  QObject::tr("5900K"),
  { 1.941406, 1, 1.265625, 0 } },

// NIKON , D90
{ "NIKON",
  "D90",
  QObject::tr("Incandescent"),
  { 1.179688, 1, 2.097656, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("SodiumVaporFluorescent"),
  { 1.062500, 1, 2.289063, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.191406, 1, 1.871094, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("WhiteFluorescent"),
  { 1.343750, 1, 2.183594, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.644531, 1, 1.937500, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("DayWhiteFluorescent"),
  { 1.695313, 1, 1.312500, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("DaylightFluorescent"),
  { 1.925781, 1, 1.074219, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("HighTempMercuryVaporFluorescent"),
  { 2.207031, 1, 1.292969, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("DirectSunlight"),
  { 1.800781, 1, 1.308594, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("Flash"),
  { 1.976563, 1, 1.152344, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("Cloudy"),
  { 1.917969, 1, 1.187500, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("Shade"),
  { 2.183594, 1, 1.062500, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("2500K"),
  { 1.023438, 1, 2.644531, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("2560K"),
  { 1.046875, 1, 2.554688, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("2630K"),
  { 1.070313, 1, 2.464844, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("2700K"),
  { 1.093750, 1, 2.378906, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("2780K"),
  { 1.117188, 1, 2.296875, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("2860K"),
  { 1.140625, 1, 2.218750, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("2940K"),
  { 1.164063, 1, 2.144531, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("3030K"),
  { 1.187500, 1, 2.078125, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("3130K"),
  { 1.218750, 1, 2.011719, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("3230K"),
  { 1.250000, 1, 1.949219, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("3330K"),
  { 1.285156, 1, 1.886719, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("3450K"),
  { 1.324219, 1, 1.828125, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("3570K"),
  { 1.359375, 1, 1.769531, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("3700K"),
  { 1.398438, 1, 1.707031, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("3850K"),
  { 1.437500, 1, 1.636719, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("4000K"),
  { 1.480469, 1, 1.562500, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("4170K"),
  { 1.535156, 1, 1.519531, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("4350K"),
  { 1.593750, 1, 1.488281, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("4550K"),
  { 1.652344, 1, 1.445313, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("4760K"),
  { 1.707031, 1, 1.398438, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("5000K"),
  { 1.761719, 1, 1.347656, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("5260K"),
  { 1.808594, 1, 1.296875, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("5560K"),
  { 1.859375, 1, 1.250000, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("5880K"),
  { 1.910156, 1, 1.207031, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("6250K"),
  { 1.960938, 1, 1.164063, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("6670K"),
  { 2.011719, 1, 1.128906, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("7140K"),
  { 2.074219, 1, 1.097656, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("7690K"),
  { 2.140625, 1, 1.074219, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("8330K"),
  { 2.218750, 1, 1.050781, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("9090K"),
  { 2.308594, 1, 1.027344, 0 } },
{ "NIKON",
  "D90",
  QObject::tr("10000K"),
  { 2.414063, 1, 1.007813, 0 } },

// NIKON , D3000
{ "NIKON",
  "D3000",
  QObject::tr("DirectSunlight"),
  { 1.851563, 1, 1.347656, 0 } },
{ "NIKON",
  "D3000",
  QObject::tr("Flash"),
  { 2.113281, 1, 1.164062, 0 } },
{ "NIKON",
  "D3000",
  QObject::tr("Cloudy"),
  { 2.019531, 1, 1.214844, 0 } },
{ "NIKON",
  "D3000",
  QObject::tr("Shade"),
  { 2.355469, 1, 1.082031, 0 } },
{ "NIKON",
  "D3000",
  QObject::tr("Incandescent"),
  { 1.171875, 1, 2.316406, 0 } },
{ "NIKON",
  "D3000",
  QObject::tr("SodiumVaporFluorescent"),
  { 1.023438, 1, 2.371094, 0 } },
{ "NIKON",
  "D3000",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.179688, 1, 2.074219, 0 } },
{ "NIKON",
  "D3000",
  QObject::tr("WhiteFluorescent"),
  { 1.355469, 1, 2.328125, 0 } },
{ "NIKON",
  "D3000",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.703125, 1, 2.019531, 0 } },
{ "NIKON",
  "D3000",
  QObject::tr("DayWhiteFluorescent"),
  { 1.750000, 1, 1.386719, 0 } },
{ "NIKON",
  "D3000",
  QObject::tr("DaylightFluorescent"),
  { 1.960937, 1, 1.105469, 0 } },
{ "NIKON",
  "D3000",
  QObject::tr("HighTempMercuryVaporFluorescent"),
  { 2.351563, 1, 1.328125, 0 } },

// NIKON , D3100
{ "NIKON",
  "D3100",
  QObject::tr("DirectSunlight"),
  { 2.109375, 1, 1.257813, 0 } },
{ "NIKON",
  "D3100",
  QObject::tr("Flash"),
  { 2.386719, 1, 1.097656, 0 } },
{ "NIKON",
  "D3100",
  QObject::tr("Cloudy"),
  { 2.257812, 1, 1.140625, 0 } },
{ "NIKON",
  "D3100",
  QObject::tr("Shade"),
  { 2.609375, 1, 1.015625, 0 } },
{ "NIKON",
  "D3100",
  QObject::tr("Incandescent"),
  { 1.320313, 1, 2.039063, 0 } },
{ "NIKON",
  "D3100",
  QObject::tr("SodiumVaporFluorescent"),
  { 1.222656, 1, 2.238281, 0 } },
{ "NIKON",
  "D3100",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.312500, 1, 1.847656, 0 } },
{ "NIKON",
  "D3100",
  QObject::tr("WhiteFluorescent"),
  { 1.531250, 1, 2.152344, 0 } },
{ "NIKON",
  "D3100",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.894531, 1, 1.875000, 0 } },
{ "NIKON",
  "D3100",
  QObject::tr("DayWhiteFluorescent"),
  { 1.941406, 1, 1.281250, 0 } },
{ "NIKON",
  "D3100",
  QObject::tr("DaylightFluorescent"),
  { 2.214844, 1, 1.015625, 0 } },
{ "NIKON",
  "D3100",
  QObject::tr("HighTempMercuryVaporFluorescent"),
  { 2.589844, 1, 1.250000, 0 } },

// NIKON , D5000
{ "NIKON",
  "D5000",
  QObject::tr("DirectSunlight"),
  { 1.800781, 1, 1.308594, 0 } },
{ "NIKON",
  "D5000",
  QObject::tr("Flash"),
  { 1.976562, 1, 1.152344, 0 } },
{ "NIKON",
  "D5000",
  QObject::tr("Cloudy"),
  { 1.917969, 1, 1.187500, 0 } },
{ "NIKON",
  "D5000",
  QObject::tr("Shade"),
  { 2.183594, 1, 1.062500, 0 } },
{ "NIKON",
  "D5000",
  QObject::tr("Incandescent"),
  { 1.179687, 1, 2.097656, 0 } },
{ "NIKON",
  "D5000",
  QObject::tr("SodiumVaporFluorescent"),
  { 1.062500, 1, 2.289063, 0 } },
{ "NIKON",
  "D5000",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.191406, 1, 1.871094, 0 } },
{ "NIKON",
  "D5000",
  QObject::tr("WhiteFluorescent"),
  { 1.343750, 1, 2.183594, 0 } },
{ "NIKON",
  "D5000",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.644531, 1, 1.937500, 0 } },
{ "NIKON",
  "D5000",
  QObject::tr("DayWhiteFluorescent"),
  { 1.695313, 1, 1.312500, 0 } },
{ "NIKON",
  "D5000",
  QObject::tr("DaylightFluorescent"),
  { 1.925781, 1, 1.074219, 0 } },
{ "NIKON",
  "D5000",
  QObject::tr("HighTempMercuryVaporFluorescent"),
  { 2.207031, 1, 1.292969, 0 } },

// NIKON , D5100
{ "NIKON",
  "D5100",
  QObject::tr("DirectSunlight"),
  { 2.0273, 1, 1.3906, 0 } },
{ "NIKON",
  "D5100",
  QObject::tr("Flash"),
  { 2.2813, 1, 1.2109, 0 } },
{ "NIKON",
  "D5100",
  QObject::tr("Cloudy"),
  { 2.1680, 1, 1.2656, 0 } },
{ "NIKON",
  "D5100",
  QObject::tr("Shade"),
  { 2.4922, 1, 1.1367, 0 } },
{ "NIKON",
  "D5100",
  QObject::tr("Incandescent"),
  { 1.3047, 1, 2.2148, 0 } },
{ "NIKON",
  "D5100",
  QObject::tr("SodiumVaporFluorescent"),
  { 1.2031, 1, 2.4375, 0 } },
{ "NIKON",
  "D5100",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.3008, 1, 1.9648, 0 } },
{ "NIKON",
  "D5100",
  QObject::tr("WhiteFluorescent"),
  { 1.4961, 1, 2.3047, 0 } },
{ "NIKON",
  "D5100",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.8320, 1, 2.0273, 0 } },
{ "NIKON",
  "D5100",
  QObject::tr("DayWhiteFluorescent"),
  { 1.8984, 1, 1.3906, 0 } },
{ "NIKON",
  "D5100",
  QObject::tr("DaylightFluorescent"),
  { 2.1680, 1, 1.1172, 0 } },
{ "NIKON",
  "D5100",
  QObject::tr("HighTempMercuryVaporFluorescent"),
  { 2.4844, 1, 1.3672, 0 } },

// NIKON , D7000
{ "NIKON",
  "D7000",
  QObject::tr("DirectSunlight"),
  { 2.027344, 1, 1.390625, 0 } },
{ "NIKON",
  "D7000",
  QObject::tr("Flash"),
  { 2.281250, 1, 1.210938, 0 } },
{ "NIKON",
  "D7000",
  QObject::tr("Cloudy"),
  { 2.167969, 1, 1.265625, 0 } },
{ "NIKON",
  "D7000",
  QObject::tr("Shade"),
  { 2.492188, 1, 1.136719, 0 } },
{ "NIKON",
  "D7000",
  QObject::tr("Incandescent"),
  { 1.304688, 1, 2.214844, 0 } },
{ "NIKON",
  "D7000",
  QObject::tr("SodiumVaporFluorescent"),
  { 1.203125, 1, 2.437500, 0 } },
{ "NIKON",
  "D7000",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.300781, 1, 1.964844, 0 } },
{ "NIKON",
  "D7000",
  QObject::tr("WhiteFluorescent"),
  { 1.496094, 1, 2.304688, 0 } },
{ "NIKON",
  "D7000",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.832031, 1, 2.027344, 0 } },
{ "NIKON",
  "D7000",
  QObject::tr("DayWhiteFluorescent"),
  { 1.898438, 1, 1.390625, 0 } },
{ "NIKON",
  "D7000",
  QObject::tr("DaylightFluorescent"),
  { 2.167969, 1, 1.117187, 0 } },
{ "NIKON",
  "D7000",
  QObject::tr("HighTempMercuryVaporFluorescent"),
  { 2.484375, 1, 1.367188, 0 } },

// NIKON , E5400
{ "NIKON",
  "E5400",
  QObject::tr("Daylight"),
  { 1.800781, 1, 1.636719, 0 } },
{ "NIKON",
  "E5400",
  QObject::tr("Incandescent"),
  { 1.218750, 1, 2.656250, 0 } },
{ "NIKON",
  "E5400",
  QObject::tr("Fluorescent"),
  { 1.218750, 1, 2.656250, 0 } },
{ "NIKON",
  "E5400",
  QObject::tr("Cloudy"),
  { 1.996094, 1, 1.421875, 0 } },
{ "NIKON",
  "E5400",
  QObject::tr("Flash"),
  { 2.328125, 1, 1.386719, 0 } },
{ "NIKON",
  "E5400",
  QObject::tr("Shade"),
  { 2.269531, 1, 1.218750, 0 } },

// NIKON , E8700
{ "NIKON",
  "E8700",
  QObject::tr("Daylight"),
  { 1.968750, 1, 1.582031, 0 } },
{ "NIKON",
  "E8700",
  QObject::tr("Incandescent"),
  { 1.265625, 1, 2.765625, 0 } },
{ "NIKON",
  "E8700",
  QObject::tr("Fluorescent"),
  { 1.863281, 1, 2.304688, 0 } },
{ "NIKON",
  "E8700",
  QObject::tr("Cloudy"),
  { 2.218750, 1, 1.359375, 0 } },
{ "NIKON",
  "E8700",
  QObject::tr("Flash"),
  { 2.535156, 1, 1.273438, 0 } },
{ "NIKON",
  "E8700",
  QObject::tr("Shade"),
  { 2.527344, 1, 1.175781, 0 } },

// OLYMPUS , C5050Z
{ "OLYMPUS",
  "C5050Z",
  QObject::tr("Shade"),
  { 1.757812, 1, 1.437500, 0 } },
{ "OLYMPUS",
  "C5050Z",
  QObject::tr("Cloudy"),
  { 1.570312, 1, 1.531250, 0 } },
{ "OLYMPUS",
  "C5050Z",
  QObject::tr("Daylight"),
  { 1.480469, 1, 1.628906, 0 } },
{ "OLYMPUS",
  "C5050Z",
  QObject::tr("EveningSun"),
  { 1.527344, 1, 1.578125, 0 } },
{ "OLYMPUS",
  "C5050Z",
  QObject::tr("DaylightFluorescent"),
  { 1.558594, 1, 1.492188, 0 } },
{ "OLYMPUS",
  "C5050Z",
  QObject::tr("NeutralFluorescent"),
  { 1.378906, 1, 1.679688, 0 } },
{ "OLYMPUS",
  "C5050Z",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.371094, 1, 2.210938, 0 } },
{ "OLYMPUS",
  "C5050Z",
  QObject::tr("WhiteFluorescent"),
  { 1, 1.075630, 3.151261, 0 } },
{ "OLYMPUS",
  "C5050Z",
  QObject::tr("Incandescent"),
  { 1, 1.094017, 3.123932, 0 } },

// OLYMPUS , C5060WZ
{ "OLYMPUS",
  "C5060WZ",
  QObject::tr("Shade"),
  { 1.949219, 1, 1.195312, 0 } },
{ "OLYMPUS",
  "C5060WZ",
  QObject::tr("Cloudy"),
  { 1.621094, 1, 1.410156, 0 } },
{ "OLYMPUS",
  "C5060WZ",
  QObject::tr("DirectSunlight"),
  { 1.511719, 1, 1.500000, 0 } },
{ "OLYMPUS",
  "C5060WZ",
  QObject::tr("EveningSun"),
  { 1.636719, 1, 1.496094, 0 } },
{ "OLYMPUS",
  "C5060WZ",
  QObject::tr("DaylightFluorescent"),
  { 1.734375, 1, 1.343750, 0 } },
{ "OLYMPUS",
  "C5060WZ",
  QObject::tr("NeutralFluorescent"),
  { 1.457031, 1, 1.691406, 0 } },
{ "OLYMPUS",
  "C5060WZ",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.417969, 1, 2.230469, 0 } },
{ "OLYMPUS",
  "C5060WZ",
  QObject::tr("WhiteFluorescent"),
  { 1, 1.103448, 3.422414, 0 } },
{ "OLYMPUS",
  "C5060WZ",
  QObject::tr("Incandescent"),
  { 1, 1.153153, 3.662162, 0 } },
{ "OLYMPUS",
  "C5060WZ",
  QObject::tr("FlashAuto"),
  { 1.850000, 1, 1.308044, 0 } },

// OLYMPUS , C8080WZ
{ "OLYMPUS",
  "C8080WZ",
  QObject::tr("Shade"),
  { 2.617188, 1, 1.191406, 0 } },
{ "OLYMPUS",
  "C8080WZ",
  QObject::tr("Cloudy"),
  { 2.203125, 1, 1.453125, 0 } },
{ "OLYMPUS",
  "C8080WZ",
  QObject::tr("Daylight"),
  { 2.128906, 1, 1.574219, 0 } },
{ "OLYMPUS",
  "C8080WZ",
  QObject::tr("EveningSun"),
  { 2.261719, 1, 1.476562, 0 } },
{ "OLYMPUS",
  "C8080WZ",
  QObject::tr("FlashAuto"),
  { 2.570312, 1, 1.468750, 0 } },
{ "OLYMPUS",
  "C8080WZ",
  QObject::tr("DaylightFluorescent"),
  { 2.460938, 1, 1.410156, 0 } },
{ "OLYMPUS",
  "C8080WZ",
  QObject::tr("NeutralFluorescent"),
  { 2.062500, 1, 1.738281, 0 } },
{ "OLYMPUS",
  "C8080WZ",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.917969, 1, 2.187500, 0 } },
{ "OLYMPUS",
  "C8080WZ",
  QObject::tr("WhiteFluorescent"),
  { 1.281250, 1, 2.972656, 0 } },
{ "OLYMPUS",
  "C8080WZ",
  QObject::tr("Tungsten"),
  { 1.164062, 1, 3.152344, 0 } },

// OLYMPUS , E-1
{ "OLYMPUS",
  "E-1",
  QObject::tr("Incandescent"),
  { 1.156250, 1, 1.640625, 0 } },
{ "OLYMPUS",
  "E-1",
  QObject::tr("IncandescentWarm"),
  { 1, 1.024000, 1.992000, 0 } },
{ "OLYMPUS",
  "E-1",
  QObject::tr("WhiteFluorescent"),
  { 2.062500, 1, 1.679688, 0 } },
{ "OLYMPUS",
  "E-1",
  QObject::tr("NeutralFluorescent"),
  { 1.835938, 1, 1.320312, 0 } },
{ "OLYMPUS",
  "E-1",
  QObject::tr("DaylightFluorescent"),
  { 2.101562, 1, 1.085938, 0 } },
{ "OLYMPUS",
  "E-1",
  QObject::tr("Daylight"),
  { 1.664062, 1, 1.140625, 0 } },
{ "OLYMPUS",
  "E-1",
  QObject::tr("Cloudy"),
  { 1.796875, 1, 1.046875, 0 } },
{ "OLYMPUS",
  "E-1",
  QObject::tr("Shade"),
  { 2.196581, 1.094017, 1, 0 } },
{ "OLYMPUS",
  "E-1",
  QObject::tr("3300K"),
  { 1.070312, 1, 1.773438, 0 } },
{ "OLYMPUS",
  "E-1",
  QObject::tr("3900K"),
  { 1.234375, 1, 1.523438, 0 } },
{ "OLYMPUS",
  "E-1",
  QObject::tr("4300K"),
  { 1.414062, 1, 1.343750, 0 } },
{ "OLYMPUS",
  "E-1",
  QObject::tr("4800K"),
  { 1.546875, 1, 1.226562, 0 } },

// OLYMPUS , E-10
{ "OLYMPUS",
  "E-10",
  QObject::tr("Incandescent"),
  { 1, 1.153153, 3.441442, 0 } },
{ "OLYMPUS",
  "E-10",
  QObject::tr("IncandescentWarm"),
  { 1.101562, 1, 2.351562, 0 } },
{ "OLYMPUS",
  "E-10",
  QObject::tr("WhiteFluorescent"),
  { 1.460938, 1, 2.546875, 0 } },
{ "OLYMPUS",
  "E-10",
  QObject::tr("DaylightFluorescent"),
  { 1.460938, 1, 1.843750, 0 } },
{ "OLYMPUS",
  "E-10",
  QObject::tr("Daylight"),
  { 1.523438, 1, 1.617188, 0 } },
{ "OLYMPUS",
  "E-10",
  QObject::tr("Cloudy"),
  { 1.687500, 1, 1.437500, 0 } },
{ "OLYMPUS",
  "E-10",
  QObject::tr("Shade"),
  { 1.812500, 1, 1.312500, 0 } },

// OLYMPUS , E-3
{ "OLYMPUS",
  "E-3",
  QObject::tr("Daylight"),
  { 1.7812, 1, 1.4688, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("Shade"),
  { 2.1406, 1, 1.1484, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("Cloudy"),
  { 1.9531, 1, 1.3359, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("Incandescent"),
  { 1.0312, 1, 2.8125, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("WhiteFluorescent"),
  { 1.6328, 1, 2.1953, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("NeutralFluorescent"),
  { 1.6641, 1, 1.7734, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("DaylightFluorescent"),
  { 1.9531, 1, 1.4844, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("Flash"),
  { 1.9609, 1, 1.3359, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2000K"),
  { 1, 2.5600, 11.3600, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2050K"),
  { 1, 2.2069, 9.4655, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2100K"),
  { 1, 2.0000, 8.3281, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2150K"),
  { 1, 1.8286, 7.3857, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2200K"),
  { 1, 1.6623, 6.4935, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2250K"),
  { 1, 1.5422, 5.8434, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2300K"),
  { 1, 1.4545, 5.3523, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2350K"),
  { 1, 1.3763, 4.9140, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2400K"),
  { 1, 1.2929, 4.4848, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2450K"),
  { 1, 1.2427, 4.1942, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2500K"),
  { 1, 1.1852, 3.8889, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2550K"),
  { 1, 1.1429, 3.6429, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2600K"),
  { 1, 1.0940, 3.3932, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2650K"),
  { 1, 1.0579, 3.1901, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2700K"),
  { 1, 1.0240, 3.0080, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2750K"),
  { 1.0078, 1, 2.8516, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2800K"),
  { 1.0312, 1, 2.7891, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("2900K"),
  { 1.0859, 1, 2.6406, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("3000K"),
  { 1.1328, 1, 2.5156, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("3100K"),
  { 1.1875, 1, 2.3906, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("3200K"),
  { 1.2266, 1, 2.2734, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("3300K"),
  { 1.2734, 1, 2.1641, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("3400K"),
  { 1.3047, 1, 2.1016, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("3500K"),
  { 1.3438, 1, 2.0312, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("3600K"),
  { 1.3750, 1, 1.9687, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("3700K"),
  { 1.4063, 1, 1.9219, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("3800K"),
  { 1.4375, 1, 1.8750, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("3900K"),
  { 1.4687, 1, 1.8281, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("4000K"),
  { 1.4844, 1, 1.8047, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("4200K"),
  { 1.5234, 1, 1.7422, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("4400K"),
  { 1.5703, 1, 1.6875, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("4600K"),
  { 1.6250, 1, 1.6250, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("4800K"),
  { 1.6797, 1, 1.5625, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("5000K"),
  { 1.7266, 1, 1.5156, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("5200K"),
  { 1.7734, 1, 1.4688, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("5400K"),
  { 1.8203, 1, 1.4297, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("5600K"),
  { 1.8750, 1, 1.3906, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("5800K"),
  { 1.9297, 1, 1.3516, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("6000K"),
  { 1.9844, 1, 1.3125, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("6200K"),
  { 2.0078, 1, 1.2891, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("6400K"),
  { 2.0312, 1, 1.2656, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("6600K"),
  { 2.0625, 1, 1.2344, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("6800K"),
  { 2.0859, 1, 1.2109, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("7000K"),
  { 2.1094, 1, 1.1875, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("7400K"),
  { 2.1484, 1, 1.1484, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("7800K"),
  { 2.1875, 1, 1.1094, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("8200K"),
  { 2.2266, 1, 1.0703, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("8600K"),
  { 2.2500, 1, 1.0469, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("9000K"),
  { 2.2813, 1, 1.0156, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("9400K"),
  { 2.3228, 1.0079, 1, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("9800K"),
  { 2.4032, 1.0323, 1, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("10000K"),
  { 2.4590, 1.0492, 1, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("11000K"),
  { 2.6379, 1.1034, 1, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("12000K"),
  { 2.8018, 1.1532, 1, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("13000K"),
  { 2.9811, 1.2075, 1, 0 } },
{ "OLYMPUS",
  "E-3",
  QObject::tr("14000K"),
  { 3.1373, 1.2549, 1, 0 } },

// OLYMPUS , E-5
{ "OLYMPUS",
  "E-5",
  QObject::tr("Daylight"),
  { 1.7344, 1, 1.3203, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("Shade"),
  { 2.1016, 1, 1.0313, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("Cloudy"),
  { 1.9141, 1, 1.1953, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("Incandescent"),
  { 1.0000, 1, 2.3906, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("Fluorescent"),
  { 1.6484, 1, 1.9141, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("Underwater"),
  { 1.7266, 1, 1.3828, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("Flash"),
  { 1.9063, 1, 1.1797, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2000K"),
  { 1, 2.6122, 9.7959, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2050K"),
  { 1, 2.2857, 8.2857, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2100K"),
  { 1, 2.0645, 7.2742, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2150K"),
  { 1, 1.8824, 6.4412, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2200K"),
  { 1, 1.7067, 5.6533, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2250K"),
  { 1, 1.6000, 5.1500, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2300K"),
  { 1, 1.4884, 4.6512, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2350K"),
  { 1, 1.4066, 4.2747, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2400K"),
  { 1, 1.3333, 3.9375, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2450K"),
  { 1, 1.2800, 3.6800, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2500K"),
  { 1, 1.2190, 3.4095, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2550K"),
  { 1, 1.1743, 3.1927, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2600K"),
  { 1, 1.1327, 3.0000, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2650K"),
  { 1, 1.0940, 2.8205, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2700K"),
  { 1, 1.0579, 2.6529, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2750K"),
  { 1, 1.0240, 2.5040, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2800K"),
  { 1.0000, 1, 2.3906, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("2900K"),
  { 1.0547, 1, 2.2734, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("3000K"),
  { 1.1016, 1, 2.1641, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("3100K"),
  { 1.1484, 1, 2.0625, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("3200K"),
  { 1.1953, 1, 1.9609, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("3300K"),
  { 1.2344, 1, 1.8750, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("3400K"),
  { 1.2656, 1, 1.8281, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("3500K"),
  { 1.3047, 1, 1.7734, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("3600K"),
  { 1.3359, 1, 1.7188, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("3700K"),
  { 1.3594, 1, 1.6797, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("3800K"),
  { 1.3906, 1, 1.6406, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("3900K"),
  { 1.4219, 1, 1.6016, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("4000K"),
  { 1.4375, 1, 1.5937, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("4200K"),
  { 1.4766, 1, 1.5703, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("4400K"),
  { 1.5234, 1, 1.5313, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("4600K"),
  { 1.5781, 1, 1.4766, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("4800K"),
  { 1.6250, 1, 1.4219, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("5000K"),
  { 1.6641, 1, 1.3828, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("5200K"),
  { 1.7109, 1, 1.3438, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("5400K"),
  { 1.7578, 1, 1.3047, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("5600K"),
  { 1.8125, 1, 1.2656, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("5800K"),
  { 1.8594, 1, 1.2344, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("6000K"),
  { 1.9141, 1, 1.1953, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("6200K"),
  { 1.9375, 1, 1.1719, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("6400K"),
  { 1.9687, 1, 1.1484, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("6600K"),
  { 1.9922, 1, 1.1250, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("6800K"),
  { 2.0156, 1, 1.1016, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("7000K"),
  { 2.0469, 1, 1.0781, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("7400K"),
  { 2.0859, 1, 1.0469, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("7800K"),
  { 2.1250, 1, 1.0078, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("8200K"),
  { 2.2160, 1.0240, 1, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("8600K"),
  { 2.3033, 1.0492, 1, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("9000K"),
  { 2.4153, 1.0847, 1, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("9400K"),
  { 2.5043, 1.1130, 1, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("9800K"),
  { 2.5752, 1.1327, 1, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("10000K"),
  { 2.6396, 1.1532, 1, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("11000K"),
  { 2.8571, 1.2190, 1, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("12000K"),
  { 3.0198, 1.2673, 1, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("13000K"),
  { 3.2292, 1.3333, 1, 0 } },
{ "OLYMPUS",
  "E-5",
  QObject::tr("14000K"),
  { 3.3763, 1.3763, 1, 0 } },

// OLYMPUS , E-30
{ "OLYMPUS",
  "E-30",
  QObject::tr("Daylight"),
  { 1.812500, 1, 1.335937, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("Shade"),
  { 2.179688, 1, 1.031250, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("Cloudy"),
  { 1.992187, 1, 1.195312, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("Incandescent"),
  { 1.054687, 1, 2.500000, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("WhiteFluorescent"),
  { 1.695313, 1, 1.921875, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("NeutralFluorescent"),
  { 1.679687, 1, 1.695313, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("DaylightFluorescent"),
  { 2.054688, 1, 1.320313, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("Flash"),
  { 1.992187, 1, 1.195312, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2000K"),
  { 1, 2.509804, 10.058823, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2050K"),
  { 1, 2.206897, 8.534483, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2100K"),
  { 1, 1.969231, 7.384615, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2150K"),
  { 1, 1.802817, 6.563380, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2200K"),
  { 1, 1.641026, 5.782051, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2250K"),
  { 1, 1.523809, 5.202381, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2300K"),
  { 1, 1.422222, 4.711111, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2350K"),
  { 1, 1.347368, 4.326316, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2400K"),
  { 1, 1.267327, 3.950495, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2450K"),
  { 1, 1.219048, 3.695238, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2500K"),
  { 1, 1.163636, 3.436364, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2550K"),
  { 1, 1.113043, 3.191304, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2600K"),
  { 1, 1.075630, 2.991597, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2650K"),
  { 1, 1.032258, 2.798387, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2700K"),
  { 1.000000, 1, 2.632813, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2750K"),
  { 1.031250, 1, 2.562500, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2800K"),
  { 1.054687, 1, 2.500000, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("2900K"),
  { 1.109375, 1, 2.367187, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("3000K"),
  { 1.164062, 1, 2.250000, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("3100K"),
  { 1.210938, 1, 2.132812, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("3200K"),
  { 1.257812, 1, 2.031250, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("3300K"),
  { 1.304687, 1, 1.929687, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("3400K"),
  { 1.335937, 1, 1.875000, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("3500K"),
  { 1.375000, 1, 1.812500, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("3600K"),
  { 1.406250, 1, 1.757812, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("3700K"),
  { 1.437500, 1, 1.718750, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("3800K"),
  { 1.468750, 1, 1.679688, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("3900K"),
  { 1.500000, 1, 1.632813, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("4000K"),
  { 1.515625, 1, 1.625000, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("4200K"),
  { 1.546875, 1, 1.601562, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("4400K"),
  { 1.585938, 1, 1.562500, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("4600K"),
  { 1.640625, 1, 1.500000, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("4800K"),
  { 1.695313, 1, 1.445312, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("5000K"),
  { 1.742187, 1, 1.406250, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("5200K"),
  { 1.789062, 1, 1.359375, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("5400K"),
  { 1.835938, 1, 1.320313, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("5600K"),
  { 1.890625, 1, 1.273438, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("5800K"),
  { 1.937500, 1, 1.234375, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("6000K"),
  { 1.992187, 1, 1.195312, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("6200K"),
  { 2.015625, 1, 1.171875, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("6400K"),
  { 2.046875, 1, 1.148438, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("6600K"),
  { 2.070312, 1, 1.125000, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("6800K"),
  { 2.093750, 1, 1.101563, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("7000K"),
  { 2.125000, 1, 1.078125, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("7400K"),
  { 2.164063, 1, 1.046875, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("7800K"),
  { 2.203125, 1, 1.007813, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("8200K"),
  { 2.296000, 1.024000, 1, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("8600K"),
  { 2.385246, 1.049180, 1, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("9000K"),
  { 2.500000, 1.084746, 1, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("9400K"),
  { 2.591304, 1.113043, 1, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("9800K"),
  { 2.663717, 1.132743, 1, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("10000K"),
  { 2.729730, 1.153153, 1, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("11000K"),
  { 2.952381, 1.219048, 1, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("12000K"),
  { 3.118812, 1.267327, 1, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("13000K"),
  { 3.333333, 1.333333, 1, 0 } },
{ "OLYMPUS",
  "E-30",
  QObject::tr("14000K"),
  { 3.483871, 1.376344, 1, 0 } },

// OLYMPUS , E-300
{ "OLYMPUS",
  "E-300",
  QObject::tr("Incandescent"),
  { 1.140625, 1, 2.203125, 0 } },
{ "OLYMPUS",
  "E-300",
  QObject::tr("IncandescentWarm"),
  { 1.312500, 1, 1.906250, 0 } },
{ "OLYMPUS",
  "E-300",
  QObject::tr("WhiteFluorescent"),
  { 1.976562, 1, 1.921875, 0 } },
{ "OLYMPUS",
  "E-300",
  QObject::tr("NeutralFluorescent"),
  { 1.820312, 1, 1.562500, 0 } },
{ "OLYMPUS",
  "E-300",
  QObject::tr("DaylightFluorescent"),
  { 2.031250, 1, 1.328125, 0 } },
{ "OLYMPUS",
  "E-300",
  QObject::tr("Daylight"),
  { 1.789062, 1, 1.351562, 0 } },
{ "OLYMPUS",
  "E-300",
  QObject::tr("Cloudy"),
  { 1.890625, 1, 1.257812, 0 } },
{ "OLYMPUS",
  "E-300",
  QObject::tr("Shade"),
  { 2.070312, 1, 1.109375, 0 } },

// OLYMPUS , E-330
{ "OLYMPUS",
  "E-330",
  QObject::tr("Daylight"),
  { 1.812500, 1, 1.296875, 0 } },
{ "OLYMPUS",
  "E-330",
  QObject::tr("Cloudy"),
  { 1.953125, 1, 1.195312, 0 } },
{ "OLYMPUS",
  "E-330",
  QObject::tr("Shade"),
  { 2.187500, 1, 1.054688, 0 } },
{ "OLYMPUS",
  "E-330",
  QObject::tr("Incandescent"),
  { 1.039062, 1, 2.437500, 0 } },
{ "OLYMPUS",
  "E-330",
  QObject::tr("WhiteFluorescent"),
  { 1.710938, 1, 1.906250, 0 } },
{ "OLYMPUS",
  "E-330",
  QObject::tr("NeutralFluorescent"),
  { 1.750000, 1, 1.531250, 0 } },
{ "OLYMPUS",
  "E-330",
  QObject::tr("DaylightFluorescent"),
  { 2.062500, 1, 1.289062, 0 } },

// OLYMPUS , E-400
{ "OLYMPUS",
  "E-400",
  QObject::tr("Daylight"),
  { 2.312500, 1, 1.179687, 0 } },
{ "OLYMPUS",
  "E-400",
  QObject::tr("Cloudy"),
  { 2.437500, 1, 1.093750, 0 } },
{ "OLYMPUS",
  "E-400",
  QObject::tr("Shade"),
  { 2.754098, 1.049180, 1, 0 } },
{ "OLYMPUS",
  "E-400",
  QObject::tr("Incandescent"),
  { 1.460937, 1, 2.171875, 0 } },
{ "OLYMPUS",
  "E-400",
  QObject::tr("WhiteFluorescent"),
  { 2.390625, 1, 1.796875, 0 } },
{ "OLYMPUS",
  "E-400",
  QObject::tr("NeutralFluorescent"),
  { 2.132812, 1, 1.468750, 0 } },
{ "OLYMPUS",
  "E-400",
  QObject::tr("DaylightFluorescent"),
  { 2.445313, 1, 1.195313, 0 } },

// OLYMPUS , E-410
{ "OLYMPUS",
  "E-410",
  QObject::tr("Daylight"),
  { 1.914063, 1, 1.367188, 0 } },
{ "OLYMPUS",
  "E-410",
  QObject::tr("Cloudy"),
  { 2.054688, 1, 1.250000, 0 } },
{ "OLYMPUS",
  "E-410",
  QObject::tr("Shade"),
  { 2.304688, 1, 1.031250, 0 } },
{ "OLYMPUS",
  "E-410",
  QObject::tr("Incandescent"),
  { 1.062500, 1, 2.781250, 0 } },
{ "OLYMPUS",
  "E-410",
  QObject::tr("WhiteFluorescent"),
  { 1.726562, 1, 2.226562, 0 } },
{ "OLYMPUS",
  "E-410",
  QObject::tr("NeutralFluorescent"),
  { 1.703125, 1, 1.796875, 0 } },
{ "OLYMPUS",
  "E-410",
  QObject::tr("DaylightFluorescent"),
  { 2.039063, 1, 1.476562, 0 } },

// OLYMPUS , E-420
{ "OLYMPUS",
  "E-420",
  QObject::tr("Daylight"),
  { 1.820313, 1, 1.437500, 0 } },
{ "OLYMPUS",
  "E-420",
  QObject::tr("Shade"),
  { 2.179688, 1, 1.140625, 0 } },
{ "OLYMPUS",
  "E-420",
  QObject::tr("Cloudy"),
  { 2.000000, 1, 1.289062, 0 } },
{ "OLYMPUS",
  "E-420",
  QObject::tr("Incandescent"),
  { 1.039062, 1, 2.726562, 0 } },
{ "OLYMPUS",
  "E-420",
  QObject::tr("WhiteFluorescent"),
  { 1.703125, 1, 2.109375, 0 } },
{ "OLYMPUS",
  "E-420",
  QObject::tr("NeutralFluorescent"),
  { 1.703125, 1, 1.757812, 0 } },
{ "OLYMPUS",
  "E-420",
  QObject::tr("Flash"),
  { 2.078125, 1, 1.375000, 0 } },
{ "OLYMPUS",
  "E-420",
  QObject::tr("2000K"),
  { 1.992187, 1, 1.289062, 0 } },
{ "OLYMPUS",
  "E-420",
  QObject::tr("7000K"),
  { 2.125000, 1, 1.187500, 0 } },
{ "OLYMPUS",
  "E-420",
  QObject::tr("14000K"),
  { 2.900901, 1.153153, 1, 0 } },

// OLYMPUS , E-500
{ "OLYMPUS",
  "E-500",
  QObject::tr("Daylight"),
  { 1.898438, 1, 1.359375, 0 } },
{ "OLYMPUS",
  "E-500",
  QObject::tr("Cloudy"),
  { 1.992188, 1, 1.265625, 0 } },
{ "OLYMPUS",
  "E-500",
  QObject::tr("Shade"),
  { 2.148438, 1, 1.125000, 0 } },
{ "OLYMPUS",
  "E-500",
  QObject::tr("Incandescent"),
  { 1.265625, 1, 2.195312, 0 } },
{ "OLYMPUS",
  "E-500",
  QObject::tr("WhiteFluorescent"),
  { 1.976562, 1, 1.914062, 0 } },
{ "OLYMPUS",
  "E-500",
  QObject::tr("NeutralFluorescent"),
  { 1.828125, 1, 1.562500, 0 } },
{ "OLYMPUS",
  "E-500",
  QObject::tr("DaylightFluorescent"),
  { 2.046875, 1, 1.359375, 0 } },

// OLYMPUS , E-510
{ "OLYMPUS",
  "E-510",
  QObject::tr("Daylight"),
  { 1.968750, 1, 1.296875, 0 } },
{ "OLYMPUS",
  "E-510",
  QObject::tr("Shade"),
  { 2.439024, 1.040650, 1, 0 } },
{ "OLYMPUS",
  "E-510",
  QObject::tr("Cloudy"),
  { 2.109375, 1, 1.187500, 0 } },
{ "OLYMPUS",
  "E-510",
  QObject::tr("Incandescent"),
  { 1.093750, 1, 2.671875, 0 } },
{ "OLYMPUS",
  "E-510",
  QObject::tr("WhiteFluorescent"),
  { 1.718750, 1, 2.109375, 0 } },
{ "OLYMPUS",
  "E-510",
  QObject::tr("NeutralFluorescent"),
  { 1.687500, 1, 1.710938, 0 } },
{ "OLYMPUS",
  "E-510",
  QObject::tr("DaylightFluorescent"),
  { 2.023438, 1, 1.398437, 0 } },

// OLYMPUS , E-520
{ "OLYMPUS",
  "E-520",
  QObject::tr("Daylight"),
  { 1.859375, 1, 1.445312, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("Shade"),
  { 2.234375, 1, 1.140625, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("Cloudy"),
  { 2.046875, 1, 1.296875, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("Tungsten"),
  { 1.062500, 1, 2.687500, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("WhiteFluorescent"),
  { 1.703125, 1, 2.109375, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("NeutralFluorescent"),
  { 1.718750, 1, 1.765625, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("DaylightFluorescent"),
  { 2.101563, 1, 1.375000, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("Flash"),
  { 2.039063, 1, 1.296875, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2000K"),
  { 1, 2.461538, 10.576923, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2050K"),
  { 1, 2.169491, 9.000000, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2100K"),
  { 1, 1.939394, 7.803031, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2150K"),
  { 1, 1.777778, 6.944445, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2200K"),
  { 1, 1.620253, 6.126582, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2250K"),
  { 1, 1.505882, 5.517647, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2300K"),
  { 1, 1.406593, 5.000000, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2350K"),
  { 1, 1.333333, 4.604167, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2400K"),
  { 1, 1.254902, 4.205882, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2450K"),
  { 1, 1.207547, 3.933962, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2500K"),
  { 1, 1.153153, 3.657658, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2550K"),
  { 1, 1.103448, 3.396552, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2600K"),
  { 1, 1.066667, 3.191667, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2650K"),
  { 1, 1.024000, 2.976000, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2700K"),
  { 1.007812, 1, 2.828125, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2750K"),
  { 1.039062, 1, 2.750000, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2800K"),
  { 1.062500, 1, 2.687500, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("2900K"),
  { 1.117188, 1, 2.546875, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("3000K"),
  { 1.171875, 1, 2.421875, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("3100K"),
  { 1.218750, 1, 2.296875, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("3200K"),
  { 1.265625, 1, 2.179688, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("3300K"),
  { 1.312500, 1, 2.078125, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("3400K"),
  { 1.343750, 1, 2.023438, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("3500K"),
  { 1.382812, 1, 1.960937, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("3600K"),
  { 1.414063, 1, 1.906250, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("3700K"),
  { 1.445312, 1, 1.859375, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("3800K"),
  { 1.476563, 1, 1.812500, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("3900K"),
  { 1.507813, 1, 1.765625, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("4000K"),
  { 1.531250, 1, 1.757812, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("4200K"),
  { 1.578125, 1, 1.726562, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("4400K"),
  { 1.625000, 1, 1.679688, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("4600K"),
  { 1.687500, 1, 1.617187, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("4800K"),
  { 1.742187, 1, 1.554687, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("5000K"),
  { 1.789062, 1, 1.515625, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("5200K"),
  { 1.835938, 1, 1.468750, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("5400K"),
  { 1.882812, 1, 1.429687, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("5600K"),
  { 1.937500, 1, 1.382812, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("5800K"),
  { 1.992187, 1, 1.343750, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("6000K"),
  { 2.046875, 1, 1.296875, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("6200K"),
  { 2.070312, 1, 1.273438, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("6400K"),
  { 2.101563, 1, 1.250000, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("6600K"),
  { 2.125000, 1, 1.226563, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("6800K"),
  { 2.148437, 1, 1.210937, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("7000K"),
  { 2.179688, 1, 1.187500, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("7400K"),
  { 2.218750, 1, 1.156250, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("7800K"),
  { 2.257812, 1, 1.117187, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("8200K"),
  { 2.296875, 1, 1.085938, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("8600K"),
  { 2.328125, 1, 1.062500, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("9000K"),
  { 2.359375, 1, 1.039063, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("9400K"),
  { 2.382812, 1, 1.015625, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("9800K"),
  { 2.406250, 1, 1.000000, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("10000K"),
  { 2.460317, 1.015873, 1, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("11000K"),
  { 2.641667, 1.066667, 1, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("12000K"),
  { 2.775862, 1.103448, 1, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("13000K"),
  { 2.919643, 1.142857, 1, 0 } },
{ "OLYMPUS",
  "E-520",
  QObject::tr("14000K"),
  { 3.036697, 1.174312, 1, 0 } },

// OLYMPUS , E-600
{ "OLYMPUS",
  "E-600",
  QObject::tr("Daylight"),
  { 1.851563, 1, 1.289063, 0 } },
{ "OLYMPUS",
  "E-600",
  QObject::tr("Shade"),
  { 2.244094, 1.007874, 1, 0 } },
{ "OLYMPUS",
  "E-600",
  QObject::tr("Cloudy"),
  { 2.046875, 1, 1.164062, 0 } },
{ "OLYMPUS",
  "E-600",
  QObject::tr("Incandescent"),
  { 1.093750, 1, 2.437500, 0 } },
{ "OLYMPUS",
  "E-600",
  QObject::tr("WhiteFluorescent"),
  { 1.750000, 1, 1.851563, 0 } },
{ "OLYMPUS",
  "E-600",
  QObject::tr("NeutralFluorescent"),
  { 1.710938, 1, 1.625000, 0 } },
{ "OLYMPUS",
  "E-600",
  QObject::tr("DaylightFluorescent"),
  { 2.085937, 1, 1.265625, 0 } },
{ "OLYMPUS",
  "E-600",
  QObject::tr("Flash"),
  { 2.039063, 1, 1.156250, 0 } },

// OLYMPUS , E-620
{ "OLYMPUS",
  "E-620",
  QObject::tr("Daylight"),
  { 1.851563, 1, 1.335938, 0 } },
{ "OLYMPUS",
  "E-620",
  QObject::tr("Shade"),
  { 2.218750, 1, 1.023438, 0 } },
{ "OLYMPUS",
  "E-620",
  QObject::tr("Cloudy"),
  { 2.039063, 1, 1.187500, 0 } },
{ "OLYMPUS",
  "E-620",
  QObject::tr("Incandescent"),
  { 1.101563, 1, 2.531250, 0 } },
{ "OLYMPUS",
  "E-620",
  QObject::tr("WhiteFluorescent"),
  { 1.718750, 1, 1.929687, 0 } },
{ "OLYMPUS",
  "E-620",
  QObject::tr("NeutralFluorescent"),
  { 1.671875, 1, 1.679688, 0 } },
{ "OLYMPUS",
  "E-620",
  QObject::tr("DaylightFluorescent"),
  { 2.046875, 1, 1.304687, 0 } },
{ "OLYMPUS",
  "E-620",
  QObject::tr("Flash"),
  { 2.039063, 1, 1.195313, 0 } },

// OLYMPUS , E-M5
{ "OLYMPUS",
  "E-M5",
  QObject::tr("Daylight"),
  { 2.1016, 1, 1.8047, 0 } },
{ "OLYMPUS",
  "E-M5",
  QObject::tr("Shade"),
  { 2.5000, 1, 1.4219, 0 } },
{ "OLYMPUS",
  "E-M5",
  QObject::tr("Cloudy"),
  { 2.3047, 1, 1.6250, 0 } },
{ "OLYMPUS",
  "E-M5",
  QObject::tr("Incandescent"),
  { 1.3125, 1, 3.2422, 0 } },
{ "OLYMPUS",
  "E-M5",
  QObject::tr("Fluorescent"),
  { 2.0234, 1, 2.5781, 0 } },
{ "OLYMPUS",
  "E-M5",
  QObject::tr("Underwater"),
  { 1.6250, 1, 2.4844, 0 } },
{ "OLYMPUS",
  "E-M5",
  QObject::tr("Flash"),
  { 2.3750, 1, 1.6016, 0 } },

// OLYMPUS , E-P1
{ "OLYMPUS",
  "E-P1",
  QObject::tr("Daylight"),
  { 1.835938, 1, 1.351563, 0 } },
{ "OLYMPUS",
  "E-P1",
  QObject::tr("Shade"),
  { 2.195313, 1, 1.046875, 0 } },
{ "OLYMPUS",
  "E-P1",
  QObject::tr("Cloudy"),
  { 2.031250, 1, 1.203125, 0 } },
{ "OLYMPUS",
  "E-P1",
  QObject::tr("Incandescent"),
  { 1.078125, 1, 2.570312, 0 } },
{ "OLYMPUS",
  "E-P1",
  QObject::tr("WhiteFluorescent"),
  { 1.695313, 1, 1.937500, 0 } },
{ "OLYMPUS",
  "E-P1",
  QObject::tr("NeutralFluorescent"),
  { 1.687500, 1, 1.703125, 0 } },
{ "OLYMPUS",
  "E-P1",
  QObject::tr("DaylightFluorescent"),
  { 2.070312, 1, 1.312500, 0 } },

// OLYMPUS , E-P2
{ "OLYMPUS",
  "E-P2",
  QObject::tr("Daylight"),
  { 1.828125, 1, 1.382813, 0 } },
{ "OLYMPUS",
  "E-P2",
  QObject::tr("Shade"),
  { 2.171875, 1, 1.070313, 0 } },
{ "OLYMPUS",
  "E-P2",
  QObject::tr("Cloudy"),
  { 2.000000, 1, 1.250000, 0 } },
{ "OLYMPUS",
  "E-P2",
  QObject::tr("Incandescent"),
  { 1.070313, 1, 2.656250, 0 } },
{ "OLYMPUS",
  "E-P2",
  QObject::tr("WhiteFluorescent"),
  { 1.781250, 1, 2.015625, 0 } },
{ "OLYMPUS",
  "E-P2",
  QObject::tr("NeutralFluorescent"),
  { 1.742188, 1, 1.796875, 0 } },
{ "OLYMPUS",
  "E-P2",
  QObject::tr("DaylightFluorescent"),
  { 2.132813, 1, 1.367188, 0 } },
{ "OLYMPUS",
  "E-P2",
  QObject::tr("Flash"),
  { 2.007813, 1, 1.242188, 0 } },

// OLYMPUS , E-P3
{ "OLYMPUS",
  "E-P3",
  QObject::tr("Daylight"),
  { 2.0469, 1, 1.4922, 0 } },
{ "OLYMPUS",
  "E-P3",
  QObject::tr("Shade"),
  { 2.4375, 1, 1.1875, 0 } },
{ "OLYMPUS",
  "E-P3",
  QObject::tr("Cloudy"),
  { 2.2188, 1, 1.3750, 0 } },
{ "OLYMPUS",
  "E-P3",
  QObject::tr("Incandescent"),
  { 1.2266, 1, 2.5312, 0 } },
{ "OLYMPUS",
  "E-P3",
  QObject::tr("Fluorescent"),
  { 1.9766, 1, 1.9766, 0 } },
{ "OLYMPUS",
  "E-P3",
  QObject::tr("Flash"),
  { 2.2109, 1, 1.3672, 0 } },

// OLYMPUS , E-PL1
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("Daylight"),
  { 1.726562, 1, 1.343750, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("Shade"),
  { 2.085937, 1, 1.039062, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("Cloudy"),
  { 1.906250, 1, 1.210938, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("Incandescent"),
  { 1, 1.015873, 2.619048, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("WhiteFluorescent"),
  { 1.648437, 1, 1.960937, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("NeutralFluorescent"),
  { 1.609375, 1, 1.710938, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("DaylightFluorescent"),
  { 2.015625, 1, 1.328125, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("Flash"),
  { 1.898438, 1, 1.203125, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2000K"),
  { 1, 2.723404, 11.340426, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2050K"),
  { 1, 2.370370, 9.537037, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2100K"),
  { 1, 2.133333, 8.316667, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2150K"),
  { 1, 1.939394, 7.333333, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2200K"),
  { 1, 1.753425, 6.410959, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2250K"),
  { 1, 1.641026, 5.820513, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2300K"),
  { 1, 1.523810, 5.226190, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2350K"),
  { 1, 1.438202, 4.786517, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2400K"),
  { 1, 1.361702, 4.404255, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2450K"),
  { 1, 1.306122, 4.102041, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2500K"),
  { 1, 1.242718, 3.796117, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2550K"),
  { 1, 1.196262, 3.542056, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2600K"),
  { 1, 1.153153, 3.315315, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2650K"),
  { 1, 1.113043, 3.113044, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2700K"),
  { 1, 1.075630, 2.924370, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2750K"),
  { 1, 1.040650, 2.747968, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2800K"),
  { 1, 1.015873, 2.619048, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("2900K"),
  { 1.039062, 1, 2.437500, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("3000K"),
  { 1.085937, 1, 2.312500, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("3100K"),
  { 1.132812, 1, 2.195313, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("3200K"),
  { 1.179687, 1, 2.078125, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("3300K"),
  { 1.218750, 1, 1.976562, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("3400K"),
  { 1.250000, 1, 1.921875, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("3500K"),
  { 1.281250, 1, 1.859375, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("3600K"),
  { 1.312500, 1, 1.796875, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("3700K"),
  { 1.343750, 1, 1.757812, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("3800K"),
  { 1.375000, 1, 1.710938, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("3900K"),
  { 1.406250, 1, 1.664062, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("4000K"),
  { 1.421875, 1, 1.648437, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("4200K"),
  { 1.468750, 1, 1.617188, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("4400K"),
  { 1.515625, 1, 1.578125, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("4600K"),
  { 1.570313, 1, 1.515625, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("4800K"),
  { 1.617188, 1, 1.453125, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("5000K"),
  { 1.656250, 1, 1.414062, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("5200K"),
  { 1.703125, 1, 1.367188, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("5400K"),
  { 1.750000, 1, 1.328125, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("5600K"),
  { 1.804688, 1, 1.289063, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("5800K"),
  { 1.851563, 1, 1.250000, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("6000K"),
  { 1.906250, 1, 1.210938, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("6200K"),
  { 1.929687, 1, 1.187500, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("6400K"),
  { 1.953125, 1, 1.164062, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("6600K"),
  { 1.984375, 1, 1.132813, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("6800K"),
  { 2.007813, 1, 1.117187, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("7000K"),
  { 2.031250, 1, 1.085938, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("7400K"),
  { 2.070312, 1, 1.054688, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("7800K"),
  { 2.109375, 1, 1.015625, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("8200K"),
  { 2.200000, 1.024000, 1, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("8600K"),
  { 2.278689, 1.049180, 1, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("9000K"),
  { 2.369748, 1.075630, 1, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("9400K"),
  { 2.478261, 1.113043, 1, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("9800K"),
  { 2.548673, 1.132743, 1, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("10000K"),
  { 2.612613, 1.153153, 1, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("11000K"),
  { 2.819048, 1.219048, 1, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("12000K"),
  { 3.010000, 1.280000, 1, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("13000K"),
  { 3.221053, 1.347368, 1, 0 } },
{ "OLYMPUS",
  "E-PL1",
  QObject::tr("14000K"),
  { 3.369565, 1.391304, 1, 0 } },

// OLYMPUS , E-PL2
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("Daylight"),
  { 1.4609, 1, 1.4219, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("Shade"),
  { 1.7422, 1, 1.1094, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("Cloudy"),
  { 1.6172, 1, 1.2891, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("Tungsten"),
  { 1, 1.1327, 2.9115, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("Fluorescent"),
  { 1.3828, 1, 2.0859, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("Flash"),
  { 1.6016, 1, 1.2891, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2000K"),
  { 1, 2.3273, 9.1091, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2050K"),
  { 1, 2.0984, 7.9508, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2100K"),
  { 1, 1.9692, 7.2615, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2150K"),
  { 1, 1.8551, 6.6522, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2200K"),
  { 1, 1.7297, 6.0135, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2250K"),
  { 1, 1.6410, 5.5513, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2300K"),
  { 1, 1.5610, 5.1341, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2350K"),
  { 1, 1.4884, 4.7674, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2400K"),
  { 1, 1.4382, 4.4944, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2450K"),
  { 1, 1.3763, 4.1935, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2500K"),
  { 1, 1.3333, 3.9583, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2550K"),
  { 1, 1.2929, 3.7475, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2600K"),
  { 1, 1.2549, 3.5392, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2650K"),
  { 1, 1.2190, 3.3619, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2700K"),
  { 1, 1.1852, 3.1852, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2750K"),
  { 1, 1.1532, 3.0270, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2800K"),
  { 1, 1.1327, 2.9115, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("2900K"),
  { 1, 1.0847, 2.6610, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("3000K"),
  { 1, 1.0492, 2.4672, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("3100K"),
  { 1, 1.0079, 2.2598, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("3200K"),
  { 1.0234, 1, 2.1484, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("3300K"),
  { 1.0547, 1, 2.0625, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("3400K"),
  { 1.0781, 1, 2.0156, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("3500K"),
  { 1.1016, 1, 1.9609, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("3600K"),
  { 1.1250, 1, 1.9063, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("3700K"),
  { 1.1484, 1, 1.8672, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("3800K"),
  { 1.1719, 1, 1.8281, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("3900K"),
  { 1.2031, 1, 1.7812, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("4000K"),
  { 1.2187, 1, 1.7578, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("4200K"),
  { 1.2578, 1, 1.6953, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("4400K"),
  { 1.2969, 1, 1.6406, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("4600K"),
  { 1.3359, 1, 1.5859, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("4800K"),
  { 1.3750, 1, 1.5313, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("5000K"),
  { 1.4062, 1, 1.4922, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("5200K"),
  { 1.4375, 1, 1.4453, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("5400K"),
  { 1.4766, 1, 1.4063, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("5600K"),
  { 1.5313, 1, 1.3672, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("5800K"),
  { 1.5703, 1, 1.3281, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("6000K"),
  { 1.6172, 1, 1.2891, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("6200K"),
  { 1.6328, 1, 1.2656, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("6400K"),
  { 1.6484, 1, 1.2422, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("6600K"),
  { 1.6719, 1, 1.2109, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("6800K"),
  { 1.6875, 1, 1.1875, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("7000K"),
  { 1.7031, 1, 1.1641, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("7400K"),
  { 1.7344, 1, 1.1250, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("7800K"),
  { 1.7578, 1, 1.0859, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("8200K"),
  { 1.7812, 1, 1.0469, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("8600K"),
  { 1.8047, 1, 1.0234, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("9000K"),
  { 1.8346, 1.0079, 1, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("9400K"),
  { 1.9032, 1.0323, 1, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("9800K"),
  { 1.9669, 1.0579, 1, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("10000K"),
  { 2.0084, 1.0756, 1, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("11000K"),
  { 2.1504, 1.1327, 1, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("12000K"),
  { 2.2870, 1.1852, 1, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("13000K"),
  { 2.4272, 1.2427, 1, 0 } },
{ "OLYMPUS",
  "E-PL2",
  QObject::tr("14000K"),
  { 2.5556, 1.2929, 1, 0 } },

// OLYMPUS , E-PL3
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("Daylight"),
  { 1.976562, 1, 1.453125, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("Shade"),
  { 2.382812, 1, 1.156250, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("Cloudy"),
  { 2.164063, 1, 1.335938, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("Tungsten"),
  { 1.171875, 1, 2.468750, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("Fluorescent"),
  { 1.953125, 1, 1.937500, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("Underwater"),
  { 2.390625, 1, 1.484375, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("Flash"),
  { 2.585938, 1, 1.250000, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2000K"),
  { 1, 1.882353, 6.897059, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2050K"),
  { 1, 1.684211, 5.986842, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2100K"),
  { 1, 1.560976, 5.402439, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2150K"),
  { 1, 1.454545, 4.909091, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2200K"),
  { 1, 1.347368, 4.421053, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2250K"),
  { 1, 1.267327, 4.049505, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2300K"),
  { 1, 1.207547, 3.754717, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2350K"),
  { 1, 1.153153, 3.504504, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2400K"),
  { 1, 1.094017, 3.239316, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2450K"),
  { 1, 1.057851, 3.057851, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2500K"),
  { 1, 1.015873, 2.873016, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2550K"),
  { 1.015625, 1, 2.757813, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2600K"),
  { 1.054687, 1, 2.695312, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2650K"),
  { 1.085938, 1, 2.632813, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2700K"),
  { 1.117187, 1, 2.570312, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2750K"),
  { 1.148438, 1, 2.515625, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2800K"),
  { 1.171875, 1, 2.468750, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("2900K"),
  { 1.226562, 1, 2.367187, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("3000K"),
  { 1.273437, 1, 2.273437, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("3100K"),
  { 1.328125, 1, 2.179688, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("3200K"),
  { 1.367188, 1, 2.093750, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("3300K"),
  { 1.414062, 1, 2.015625, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("3400K"),
  { 1.445312, 1, 1.968750, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("3500K"),
  { 1.476563, 1, 1.914063, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("3600K"),
  { 1.507812, 1, 1.867188, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("3700K"),
  { 1.539062, 1, 1.828125, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("3800K"),
  { 1.570312, 1, 1.789062, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("3900K"),
  { 1.609375, 1, 1.750000, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("4000K"),
  { 1.640625, 1, 1.734375, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("4200K"),
  { 1.703125, 1, 1.695313, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("4400K"),
  { 1.757812, 1, 1.648437, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("4600K"),
  { 1.812500, 1, 1.601562, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("4800K"),
  { 1.867188, 1, 1.554688, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("5000K"),
  { 1.906250, 1, 1.515625, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("5200K"),
  { 1.953125, 1, 1.476562, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("5400K"),
  { 2.000000, 1, 1.437500, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("5600K"),
  { 2.054688, 1, 1.406250, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("5800K"),
  { 2.109375, 1, 1.375000, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("6000K"),
  { 2.164063, 1, 1.335938, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("6200K"),
  { 2.195313, 1, 1.312500, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("6400K"),
  { 2.226562, 1, 1.289062, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("6600K"),
  { 2.257812, 1, 1.257812, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("6800K"),
  { 2.289063, 1, 1.234375, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("7000K"),
  { 2.320313, 1, 1.210938, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("7400K"),
  { 2.367187, 1, 1.171875, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("7800K"),
  { 2.414062, 1, 1.132812, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("8200K"),
  { 2.460937, 1, 1.093750, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("8600K"),
  { 2.492188, 1, 1.070313, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("9000K"),
  { 2.523438, 1, 1.039063, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("9400K"),
  { 2.554687, 1, 1.015625, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("9800K"),
  { 2.606299, 1.007874, 1, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("10000K"),
  { 2.664000, 1.024000, 1, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("11000K"),
  { 2.865546, 1.075630, 1, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("12000K"),
  { 3.043860, 1.122807, 1, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("13000K"),
  { 3.238532, 1.174312, 1, 0 } },
{ "OLYMPUS",
  "E-PL3",
  QObject::tr("14000K"),
  { 3.400000, 1.219048, 1, 0 } },

// OLYMPUS , SP500UZ
{ "OLYMPUS",
  "SP500UZ",
  QObject::tr("Daylight"),
  { 1.960937, 1, 1.585937, 0 } },
{ "OLYMPUS",
  "SP500UZ",
  QObject::tr("Cloudy"),
  { 2.058594, 1, 1.484375, 0 } },
{ "OLYMPUS",
  "SP500UZ",
  QObject::tr("EveningSun"),
  { 2.074219, 1, 1.488281, 0 } },
{ "OLYMPUS",
  "SP500UZ",
  QObject::tr("Tungsten"),
  { 1.085937, 1, 2.742188, 0 } },
{ "OLYMPUS",
  "SP500UZ",
  QObject::tr("Fluorescent"),
  { 2.285156, 1, 1.488281, 0 } },

// OLYMPUS , SP510UZ
{ "OLYMPUS",
  "SP510UZ",
  QObject::tr("Daylight"),
  { 1.656250, 1, 1.621094, 0 } },
{ "OLYMPUS",
  "SP510UZ",
  QObject::tr("Cloudy"),
  { 1.789063, 1, 1.546875, 0 } },
{ "OLYMPUS",
  "SP510UZ",
  QObject::tr("Incandescent"),
  { 1, 1.066667, 2.891667, 0 } },
{ "OLYMPUS",
  "SP510UZ",
  QObject::tr("WhiteFluorescent"),
  { 1.929688, 1, 1.562500, 0 } },
{ "OLYMPUS",
  "SP510UZ",
  QObject::tr("NeutralFluorescent"),
  { 1.644531, 1, 1.843750, 0 } },
{ "OLYMPUS",
  "SP510UZ",
  QObject::tr("DaylightFluorescent"),
  { 1.628906, 1, 2.210938, 0 } },

// OLYMPUS , XZ-1
{ "OLYMPUS",
  "XZ-1",
  QObject::tr("Daylight"),
  { 1.8437, 1, 1.8750, 0 } },
{ "OLYMPUS",
  "XZ-1",
  QObject::tr("Shade"),
  { 2.1875, 1, 1.5703, 0 } },
{ "OLYMPUS",
  "XZ-1",
  QObject::tr("Cloudy"),
  { 1.9766, 1, 1.7422, 0 } },
{ "OLYMPUS",
  "XZ-1",
  QObject::tr("Incandescent"),
  { 1.1172, 1, 3.1875, 0 } },
{ "OLYMPUS",
  "XZ-1",
  QObject::tr("Fluorescent"),
  { 1.8125, 1, 2.5938, 0 } },
{ "OLYMPUS",
  "XZ-1",
  QObject::tr("Underwater"),
  { 1.4844, 1, 2.2188, 0 } },

// Panasonic , DMC-FZ8
{ "Panasonic",
  "DMC-FZ8",
  QObject::tr("Daylight"),
  { 1.904943, 1, 1.596958, 0 } },
{ "Panasonic",
  "DMC-FZ8",
  QObject::tr("Cloudy"),
  { 2.060836, 1, 1.498099, 0 } },
{ "Panasonic",
  "DMC-FZ8",
  QObject::tr("Shade"),
  { 2.258555, 1, 1.391635, 0 } },
{ "Panasonic",
  "DMC-FZ8",
  QObject::tr("Incandescent"),
  { 1.247148, 1, 2.288973, 0 } },
{ "Panasonic",
  "DMC-FZ8",
  QObject::tr("Flash"),
  { 2.072243, 1, 1.456274, 0 } },

// Panasonic , DMC-FZ18
{ "Panasonic",
  "DMC-FZ18",
  QObject::tr("Daylight"),
  { 1.783270, 1, 1.889734, 0 } },
{ "Panasonic",
  "DMC-FZ18",
  QObject::tr("Cloudy"),
  { 1.946768, 1, 1.680608, 0 } },
{ "Panasonic",
  "DMC-FZ18",
  QObject::tr("Shade"),
  { 2.117871, 1, 1.558935, 0 } },
{ "Panasonic",
  "DMC-FZ18",
  QObject::tr("Incandescent"),
  { 1.140684, 1, 2.627376, 0 } },
{ "Panasonic",
  "DMC-FZ18",
  QObject::tr("Flash"),
  { 1.882129, 1, 1.703422, 0 } },

// Panasonic , DMC-FZ28
{ "Panasonic",
  "DMC-FZ28",
  QObject::tr("Daylight"),
  { 1.684411, 1, 1.802281, 0 } },
{ "Panasonic",
  "DMC-FZ28",
  QObject::tr("Cloudy"),
  { 1.825095, 1, 1.676806, 0 } },
{ "Panasonic",
  "DMC-FZ28",
  QObject::tr("Shade"),
  { 1.996198, 1, 1.566540, 0 } },
{ "Panasonic",
  "DMC-FZ28",
  QObject::tr("Incandescent"),
  { 1.117871, 1, 2.558935, 0 } },
{ "Panasonic",
  "DMC-FZ28",
  QObject::tr("Flash"),
  { 1.939164, 1, 1.596958, 0 } },
{ "Panasonic",
  "DMC-FZ28",
  QObject::tr("3000K"),
  { 1.015209, 1, 2.771863, 0 } },
{ "Panasonic",
  "DMC-FZ28",
  QObject::tr("4000K"),
  { 1.277566, 1, 2.171103, 0 } },
{ "Panasonic",
  "DMC-FZ28",
  QObject::tr("5000K"),
  { 1.585551, 1, 1.889734, 0 } },
{ "Panasonic",
  "DMC-FZ28",
  QObject::tr("6000K"),
  { 1.764258, 1, 1.737642, 0 } },
{ "Panasonic",
  "DMC-FZ28",
  QObject::tr("7000K"),
  { 1.939164, 1, 1.596958, 0 } },
{ "Panasonic",
  "DMC-FZ28",
  QObject::tr("8000K"),
  { 2.049430, 1, 1.528517, 0 } },

// Panasonic , DMC-FZ30
{ "Panasonic",
  "DMC-FZ30",
  QObject::tr("Daylight"),
  { 1.757576, 1, 1.446970, 0 } },
{ "Panasonic",
  "DMC-FZ30",
  QObject::tr("Cloudy"),
  { 1.943182, 1, 1.276515, 0 } },
{ "Panasonic",
  "DMC-FZ30",
  QObject::tr("Incandescent"),
  { 1.098485, 1, 2.106061, 0 } },
{ "Panasonic",
  "DMC-FZ30",
  QObject::tr("Flash"),
  { 1.965909, 1, 1.303030, 0 } },

// Panasonic , DMC-FZ50
{ "Panasonic",
  "DMC-FZ50",
  QObject::tr("Daylight"),
  { 2.095057, 1, 1.642586, 0 } },
{ "Panasonic",
  "DMC-FZ50",
  QObject::tr("Cloudy"),
  { 2.319392, 1, 1.482890, 0 } },
{ "Panasonic",
  "DMC-FZ50",
  QObject::tr("Shade"),
  { 2.463878, 1, 1.414449, 0 } },
{ "Panasonic",
  "DMC-FZ50",
  QObject::tr("Incandescent"),
  { 1.365019, 1, 2.311787, 0 } },
{ "Panasonic",
  "DMC-FZ50",
  QObject::tr("Flash"),
  { 2.338403, 1, 1.338403, 0 } },

// Panasonic , DMC-G1
{ "Panasonic",
  "DMC-G1",
  QObject::tr("Daylight"),
  { 1.942966, 1, 1.448669, 0 } },
{ "Panasonic",
  "DMC-G1",
  QObject::tr("Cloudy"),
  { 2.106464, 1, 1.326996, 0 } },
{ "Panasonic",
  "DMC-G1",
  QObject::tr("Shade"),
  { 2.323194, 1, 1.224335, 0 } },
{ "Panasonic",
  "DMC-G1",
  QObject::tr("Incandescent"),
  { 1.319392, 1, 2.148289, 0 } },
{ "Panasonic",
  "DMC-G1",
  QObject::tr("Flash"),
  { 1.528517, 1, 1.277567, 0 } },

// Panasonic , DMC-G2
{ "Panasonic",
  "DMC-G2",
  QObject::tr("Daylight"),
  { 1.931559, 1, 1.414449, 0 } },
{ "Panasonic",
  "DMC-G2",
  QObject::tr("Cloudy"),
  { 2.292776, 1, 1.231939, 0 } },
{ "Panasonic",
  "DMC-G2",
  QObject::tr("Shade"),
  { 2.243346, 1, 1.231939, 0 } },
{ "Panasonic",
  "DMC-G2",
  QObject::tr("Incandescent"),
  { 2.190114, 1, 1.250951, 0 } },
{ "Panasonic",
  "DMC-G2",
  QObject::tr("Flash"),
  { 2.296578, 1, 1.190114, 0 } },

// Panasonic , DMC-G3
{ "Panasonic",
  "DMC-G3",
  QObject::tr("Daylight"),
  { 2.7925, 1, 1.5472, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("Cloudy"),
  { 2.9660, 1, 1.4528, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("Shade"),
  { 3.2642, 1, 1.3698, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("Incandescent"),
  { 1.8491, 1, 2.2566, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("Flash"),
  { 3.2868, 1, 1.3547, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("2500K"),
  { 1.4226, 1, 2.8302, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("2600K"),
  { 1.4755, 1, 2.7547, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("2700K"),
  { 1.5283, 1, 2.6717, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("2800K"),
  { 1.5925, 1, 2.5849, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("2900K"),
  { 1.6491, 1, 2.5019, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("3000K"),
  { 1.7094, 1, 2.4189, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("3100K"),
  { 1.7660, 1, 2.3434, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("3200K"),
  { 1.8189, 1, 2.2604, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("3300K"),
  { 1.8792, 1, 2.1849, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("3400K"),
  { 1.9358, 1, 2.1019, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("3500K"),
  { 1.9962, 1, 2.0189, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("3700K"),
  { 2.1245, 1, 1.9358, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("4000K"),
  { 2.3245, 1, 1.8113, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("4200K"),
  { 2.4189, 1, 1.7623, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("4500K"),
  { 2.5585, 1, 1.6868, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("4700K"),
  { 2.6000, 1, 1.6679, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("5000K"),
  { 2.6679, 1, 1.6264, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("5200K"),
  { 2.7170, 1, 1.5962, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("5500K"),
  { 2.7925, 1, 1.5472, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("5700K"),
  { 2.8415, 1, 1.5208, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("6000K"),
  { 2.9283, 1, 1.4717, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("6500K"),
  { 3.0679, 1, 1.4189, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("7000K"),
  { 3.1925, 1, 1.3811, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("7500K"),
  { 3.3170, 1, 1.3472, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("8000K"),
  { 3.3962, 1, 1.3283, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("8500K"),
  { 3.4792, 1, 1.3057, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("9000K"),
  { 3.5585, 1, 1.2868, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("9500K"),
  { 3.6302, 1, 1.2642, 0 } },
{ "Panasonic",
  "DMC-G3",
  QObject::tr("10000K"),
  { 3.7094, 1, 1.2528, 0 } },

// Panasonic , DMC-GF1
{ "Panasonic",
  "DMC-GF1",
  QObject::tr("Daylight"),
  { 2.007605, 1, 1.418251, 0 } },
{ "Panasonic",
  "DMC-GF1",
  QObject::tr("Cloudy"),
  { 2.174905, 1, 1.296578, 0 } },
{ "Panasonic",
  "DMC-GF1",
  QObject::tr("Shade"),
  { 2.395437, 1, 1.193916, 0 } },
{ "Panasonic",
  "DMC-GF1",
  QObject::tr("Incandescent"),
  { 1.365019, 1, 2.117871, 0 } },
{ "Panasonic",
  "DMC-GF1",
  QObject::tr("Flash"),
  { 2.353612, 1, 1.231939, 0 } },

// Panasonic , DMC-GF3
{ "Panasonic",
  "DMC-GF3",
  QObject::tr("Daylight"),
  { 1.9396, 1, 1.4377, 0 } },
{ "Panasonic",
  "DMC-GF3",
  QObject::tr("Cloudy"),
  { 2.0679, 1, 1.3396, 0 } },
{ "Panasonic",
  "DMC-GF3",
  QObject::tr("Shade"),
  { 2.2566, 1, 1.2717, 0 } },
{ "Panasonic",
  "DMC-GF3",
  QObject::tr("Incandescent"),
  { 1.3019, 1, 2.0868, 0 } },
{ "Panasonic",
  "DMC-GF3",
  QObject::tr("Flash"),
  { 2.1962, 1, 1.2717, 0 } },

// Panasonic , DMC-GX1
{ "Panasonic",
  "DMC-GX1",
  QObject::tr("Daylight"),
  { 2.7925, 1, 1.5472, 0 } },
{ "Panasonic",
  "DMC-GX1",
  QObject::tr("Cloudy"),
  { 2.9660, 1, 1.4528, 0 } },
{ "Panasonic",
  "DMC-GX1",
  QObject::tr("Shade"),
  { 3.2642, 1, 1.3698, 0 } },
{ "Panasonic",
  "DMC-GX1",
  QObject::tr("Incandescent"),
  { 1.8491, 1, 2.2566, 0 } },
{ "Panasonic",
  "DMC-GX1",
  QObject::tr("Flash"),
  { 3.2868, 1, 1.3547, 0 } },
{ "Panasonic",
  "DMC-GX1",
  QObject::tr("2700K"),
  { 1.5283, 1, 2.6717, 0 } },
{ "Panasonic",
  "DMC-GX1",
  QObject::tr("3000K"),
  { 1.7094, 1, 2.4189, 0 } },
{ "Panasonic",
  "DMC-GX1",
  QObject::tr("3300K"),
  { 1.8792, 1, 2.1849, 0 } },
{ "Panasonic",
  "DMC-GX1",
  QObject::tr("5000K"),
  { 2.6679, 1, 1.6264, 0 } },
{ "Panasonic",
  "DMC-GX1",
  QObject::tr("5500K"),
  { 2.7925, 1, 1.5472, 0 } },
{ "Panasonic",
  "DMC-GX1",
  QObject::tr("6500K"),
  { 3.0679, 1, 1.4189, 0 } },

// Panasonic , DMC-L1
{ "Panasonic",
  "DMC-L1",
  QObject::tr("Daylight"),
  { 1.980989, 1, 1.444867, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("Cloudy"),
  { 2.129278, 1, 1.300380, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("Shade"),
  { 2.361217, 1, 1.167300, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("Incandescent"),
  { 1.368821, 1, 2.091255, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("Flash"),
  { 2.319392, 1, 1.053232, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("2500K"),
  { 1.209126, 1, 2.722434, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("2600K"),
  { 1.243346, 1, 2.623574, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("2700K"),
  { 1.285171, 1, 2.520913, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("2800K"),
  { 1.323194, 1, 2.418251, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("2900K"),
  { 1.365019, 1, 2.319392, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("3000K"),
  { 1.406844, 1, 2.209126, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("3100K"),
  { 1.441065, 1, 2.193916, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("3200K"),
  { 1.482890, 1, 2.178707, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("3300K"),
  { 1.524715, 1, 2.163498, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("3400K"),
  { 1.566540, 1, 2.148289, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("3500K"),
  { 1.608365, 1, 2.136882, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("3600K"),
  { 1.638783, 1, 2.091255, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("3800K"),
  { 1.699620, 1, 2.000000, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("4000K"),
  { 1.760456, 1, 1.897338, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("4200K"),
  { 1.813688, 1, 1.809886, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("4400K"),
  { 1.874525, 1, 1.722433, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("4600K"),
  { 1.912547, 1, 1.642585, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("4800K"),
  { 1.923954, 1, 1.585551, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("5000K"),
  { 1.942966, 1, 1.528517, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("5300K"),
  { 1.984791, 1, 1.456274, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("5500K"),
  { 2.019011, 1, 1.403042, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("5800K"),
  { 2.057034, 1, 1.361217, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("6000K"),
  { 2.079848, 1, 1.323194, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("6300K"),
  { 2.159696, 1, 1.281369, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("6500K"),
  { 2.216730, 1, 1.243346, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("6800K"),
  { 2.273764, 1, 1.228137, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("7300K"),
  { 2.380228, 1, 1.186312, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("7800K"),
  { 2.452471, 1, 1.144487, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("8300K"),
  { 2.501901, 1, 1.106464, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("9000K"),
  { 2.574145, 1, 1.068441, 0 } },
{ "Panasonic",
  "DMC-L1",
  QObject::tr("10000K"),
  { 2.692015, 1.011407, 1, 0 } },

// Panasonic , DMC-LX1
{ "Panasonic",
  "DMC-LX1",
  QObject::tr("Daylight"),
  { 1.837121, 1, 1.484848, 0 } },
{ "Panasonic",
  "DMC-LX1",
  QObject::tr("Cloudy"),
  { 2.003788, 1, 1.310606, 0 } },
{ "Panasonic",
  "DMC-LX1",
  QObject::tr("Incandescent"),
  { 1.098485, 1, 2.272727, 0 } },

// Panasonic , DMC-LX2
{ "Panasonic",
  "DMC-LX2",
  QObject::tr("Daylight"),
  { 2.114068, 1, 1.726236, 0 } },
{ "Panasonic",
  "DMC-LX2",
  QObject::tr("Cloudy"),
  { 2.338403, 1, 1.577947, 0 } },
{ "Panasonic",
  "DMC-LX2",
  QObject::tr("Shade"),
  { 2.532319, 1, 1.509506, 0 } },
{ "Panasonic",
  "DMC-LX2",
  QObject::tr("Incandescent"),
  { 1.365019, 1, 2.410646, 0 } },

// Panasonic , DMC-LX3
{ "Panasonic",
  "DMC-LX3",
  QObject::tr("Daylight"),
  { 2.022814, 1, 1.623574, 0 } },
{ "Panasonic",
  "DMC-LX3",
  QObject::tr("Cloudy"),
  { 2.224335, 1, 1.520913, 0 } },
{ "Panasonic",
  "DMC-LX3",
  QObject::tr("Shade"),
  { 2.475285, 1, 1.399240, 0 } },
{ "Panasonic",
  "DMC-LX3",
  QObject::tr("Flash"),
  { 2.296578, 1, 1.482890, 0 } },
{ "Panasonic",
  "DMC-LX3",
  QObject::tr("Incandescent"),
  { 1.346008, 1, 2.269962, 0 } },

// Panasonic , DMC-LX5
{ "Panasonic",
  "DMC-LX5",
  QObject::tr("Daylight"),
  { 1.9202, 1, 1.7567, 0 } },
{ "Panasonic",
  "DMC-LX5",
  QObject::tr("Cloudy"),
  { 2.0760, 1, 1.6388, 0 } },
{ "Panasonic",
  "DMC-LX5",
  QObject::tr("Shade"),
  { 2.1635, 1, 1.5817, 0 } },
{ "Panasonic",
  "DMC-LX5",
  QObject::tr("Flash"),
  { 2.0760, 1, 1.6388, 0 } },
{ "Panasonic",
  "DMC-LX5",
  QObject::tr("Incandescent"),
  { 1.2243, 1, 2.5741, 0 } },

// PENTAX , *ist D
{ "PENTAX",
  "*ist D",
  QObject::tr("Daylight"),
  { 1.460938, 1, 1.019531, 0 } },
{ "PENTAX",
  "*ist D",
  QObject::tr("Shade"),
  { 1.734375, 1, 1.000000, 0 } },
{ "PENTAX",
  "*ist D",
  QObject::tr("Cloudy"),
  { 1.634921, 1.015873, 1, 0 } },
{ "PENTAX",
  "*ist D",
  QObject::tr("DaylightFluorescent"),
  { 1.657025, 1.057851, 1, 0 } },
{ "PENTAX",
  "*ist D",
  QObject::tr("NeutralFluorescent"),
  { 1.425781, 1, 1.117188, 0 } },
{ "PENTAX",
  "*ist D",
  QObject::tr("WhiteFluorescent"),
  { 1.328125, 1, 1.210938, 0 } },
{ "PENTAX",
  "*ist D",
  QObject::tr("Tungsten"),
  { 1.000000, 1, 2.226563, 0 } },
{ "PENTAX",
  "*ist D",
  QObject::tr("Flash"),
  { 1.750000, 1, 1.000000, 0 } },

// PENTAX , *ist DL
{ "PENTAX",
  "*ist DL",
  QObject::tr("Daylight"),
  { 1.546875, 1, 1.007812, 0 } },
{ "PENTAX",
  "*ist DL",
  QObject::tr("Shade"),
  { 1.933594, 1, 1.027344, 0 } },
{ "PENTAX",
  "*ist DL",
  QObject::tr("Cloudy"),
  { 1.703125, 1, 1.003906, 0 } },
{ "PENTAX",
  "*ist DL",
  QObject::tr("DaylightFluorescent"),
  { 2.593909, 1.299492, 1, 0 } },
{ "PENTAX",
  "*ist DL",
  QObject::tr("NeutralFluorescent"),
  { 1.539062, 1, 1.003906, 0 } },
{ "PENTAX",
  "*ist DL",
  QObject::tr("WhiteFluorescent"),
  { 1.390625, 1, 1.117188, 0 } },
{ "PENTAX",
  "*ist DL",
  QObject::tr("Tungsten"),
  { 1.000000, 1, 2.074219, 0 } },
{ "PENTAX",
  "*ist DL",
  QObject::tr("Flash"),
  { 1.621094, 1, 1.027344, 0 } },

// PENTAX , *ist DS
{ "PENTAX",
  "*ist DS",
  QObject::tr("Daylight"),
  { 1.632812, 1, 1.000000, 0 } },
{ "PENTAX",
  "*ist DS",
  QObject::tr("Shade"),
  { 1.964844, 1, 1.000000, 0 } },
{ "PENTAX",
  "*ist DS",
  QObject::tr("Cloudy"),
  { 1.761719, 1, 1.000000, 0 } },
{ "PENTAX",
  "*ist DS",
  QObject::tr("DaylightFluorescent"),
  { 1.910156, 1, 1.000000, 0 } },
{ "PENTAX",
  "*ist DS",
  QObject::tr("NeutralFluorescent"),
  { 1.521569, 1.003922, 1, 0 } },
{ "PENTAX",
  "*ist DS",
  QObject::tr("WhiteFluorescent"),
  { 1.496094, 1, 1.023438, 0 } },
{ "PENTAX",
  "*ist DS",
  QObject::tr("Tungsten"),
  { 1.000000, 1, 2.027344, 0 } },
{ "PENTAX",
  "*ist DS",
  QObject::tr("Flash"),
  { 1.695312, 1, 1.000000, 0 } },

// PENTAX , K10D
{ "PENTAX",
  "K10D",
  QObject::tr("Daylight"),
  { 1.660156, 1, 1.066406, 0 } },
{ "PENTAX",
  "K10D",
  QObject::tr("Shade"),
  { 2.434783, 1.236715, 1, 0 } },
{ "PENTAX",
  "K10D",
  QObject::tr("Cloudy"),
  { 1.872428, 1.053498, 1, 0 } },
{ "PENTAX",
  "K10D",
  QObject::tr("DaylightFluorescent"),
  { 2.121094, 1, 1.078125, 0 } },
{ "PENTAX",
  "K10D",
  QObject::tr("NeutralFluorescent"),
  { 1.773438, 1, 1.226562, 0 } },
{ "PENTAX",
  "K10D",
  QObject::tr("WhiteFluorescent"),
  { 1.597656, 1, 1.488281, 0 } },
{ "PENTAX",
  "K10D",
  QObject::tr("Tungsten"),
  { 1.000000, 1, 2.558594, 0 } },
{ "PENTAX",
  "K10D",
  QObject::tr("Flash"),
  { 1.664062, 1, 1.046875, 0 } },

// PENTAX , K20D
{ "PENTAX",
  "K20D",
  QObject::tr("Daylight"),
  { 1.691406, 1, 1.257812, 0 } },
{ "PENTAX",
  "K20D",
  QObject::tr("Shade"),
  { 2.012245, 1.299492, 1, 0 } },
{ "PENTAX",
  "K20D",
  QObject::tr("Cloudy"),
  { 1.792969, 1, 1.109375, 0 } },
{ "PENTAX",
  "K20D",
  QObject::tr("DaylightFluorescent"),
  { 2.234375, 1, 1.183594, 0 } },
{ "PENTAX",
  "K20D",
  QObject::tr("NeutralFluorescent"),
  { 1.898438, 1, 1.347656, 0 } },
{ "PENTAX",
  "K20D",
  QObject::tr("WhiteFluorescent"),
  { 1.769531, 1, 1.675781, 0 } },
{ "PENTAX",
  "K20D",
  QObject::tr("Tungsten"),
  { 1, 1.089362, 2.961702, 0 } },
{ "PENTAX",
  "K20D",
  QObject::tr("Flash"),
  { 1.792969, 1, 1.183594, 0 } },

// PENTAX , K100D
{ "PENTAX",
  "K100D",
  QObject::tr("Daylight"),
  { 1.468750, 1, 1.023438, 0 } },
{ "PENTAX",
  "K100D",
  QObject::tr("Shade"),
  { 1.769531, 1, 1.000000, 0 } },
{ "PENTAX",
  "K100D",
  QObject::tr("Cloudy"),
  { 1.589844, 1, 1.000000, 0 } },
{ "PENTAX",
  "K100D",
  QObject::tr("DaylightFluorescent"),
  { 1.722656, 1, 1.039063, 0 } },
{ "PENTAX",
  "K100D",
  QObject::tr("NeutralFluorescent"),
  { 1.425781, 1, 1.160156, 0 } },
{ "PENTAX",
  "K100D",
  QObject::tr("WhiteFluorescent"),
  { 1.265625, 1, 1.414063, 0 } },
{ "PENTAX",
  "K100D",
  QObject::tr("Tungsten"),
  { 1, 1.015873, 2.055556, 0 } },
{ "PENTAX",
  "K100D",
  QObject::tr("Flash"),
  { 1.527344, 1, 1.000000, 0 } },

// PENTAX , K100D Super
{ "PENTAX",
  "K100D Super",
  QObject::tr("Daylight"),
  { 1.593750, 1, 1.011719, 0 } },
{ "PENTAX",
  "K100D Super",
  QObject::tr("Shade"),
  { 1.917969, 1, 1.000000, 0 } },
{ "PENTAX",
  "K100D Super",
  QObject::tr("Cloudy"),
  { 1.703125, 1, 1.015625, 0 } },
{ "PENTAX",
  "K100D Super",
  QObject::tr("DaylightFluorescent"),
  { 1.708502, 1.036437, 1, 0 } },
{ "PENTAX",
  "K100D Super",
  QObject::tr("NeutralFluorescent"),
  { 1.634538, 1.028112, 1, 0 } },
{ "PENTAX",
  "K100D Super",
  QObject::tr("WhiteFluorescent"),
  { 1.425781, 1, 1.136719, 0 } },
{ "PENTAX",
  "K100D Super",
  QObject::tr("Tungsten"),
  { 1.015625, 1, 2.046875, 0 } },
{ "PENTAX",
  "K100D Super",
  QObject::tr("Flash"),
  { 1.670588, 1.003922, 1, 0 } },

// PENTAX , K110D
{ "PENTAX",
  "K110D",
  QObject::tr("Daylight"),
  { 1.468750, 1, 1.023438, 0 } },
{ "PENTAX",
  "K110D",
  QObject::tr("Shade"),
  { 1.769531, 1, 1.000000, 0 } },
{ "PENTAX",
  "K110D",
  QObject::tr("Cloudy"),
  { 1.589844, 1, 1.000000, 0 } },
{ "PENTAX",
  "K110D",
  QObject::tr("DaylightFluorescent"),
  { 1.722656, 1, 1.039063, 0 } },
{ "PENTAX",
  "K110D",
  QObject::tr("NeutralFluorescent"),
  { 1.425781, 1, 1.160156, 0 } },
{ "PENTAX",
  "K110D",
  QObject::tr("WhiteFluorescent"),
  { 1.265625, 1, 1.414063, 0 } },
{ "PENTAX",
  "K110D",
  QObject::tr("Tungsten"),
  { 1, 1.015873, 2.055556, 0 } },
{ "PENTAX",
  "K110D",
  QObject::tr("Flash"),
  { 1.527344, 1, 1.000000, 0 } },

// PENTAX , K200D
{ "PENTAX",
  "K200D",
  QObject::tr("Daylight"),
  { 1.804688, 1, 1.304688, 0 } },
{ "PENTAX",
  "K200D",
  QObject::tr("Shade"),
  { 2.140625, 1, 1.085937, 0 } },
{ "PENTAX",
  "K200D",
  QObject::tr("Cloudy"),
  { 1.957031, 1, 1.179687, 0 } },
{ "PENTAX",
  "K200D",
  QObject::tr("DaylightFluorescent"),
  { 2.121094, 1, 1.195313, 0 } },
{ "PENTAX",
  "K200D",
  QObject::tr("NeutralFluorescent"),
  { 1.773438, 1, 1.359375, 0 } },
{ "PENTAX",
  "K200D",
  QObject::tr("WhiteFluorescent"),
  { 1.597656, 1, 1.648437, 0 } },
{ "PENTAX",
  "K200D",
  QObject::tr("Tungsten"),
  { 1.000000, 1, 2.835937, 0 } },
{ "PENTAX",
  "K200D",
  QObject::tr("Flash"),
  { 1.917969, 1, 1.214844, 0 } },

// PENTAX , K-5
{ "PENTAX",
  "K-5",
  QObject::tr("Daylight"),
  { 2.1211, 1, 1.5781, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("Shade"),
  { 2.5312, 1, 1.1758, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("Cloudy"),
  { 2.2852, 1, 1.3477, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("DaylightFluorescent"),
  { 2.6172, 1, 1.5195, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("WhiteFluorescent"),
  { 2.2422, 1, 1.6914, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("CoolWhiteFluorescent"),
  { 2.0391, 1, 2.0898, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.7070, 1, 2.6172, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("Tungsten"),
  { 1.2734, 1, 2.9258, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("Flash"),
  { 2.4023, 1, 1.4492, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("2500K"),
  { 1.1445, 1, 2.7891, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("3000K"),
  { 1.3867, 1, 2.2578, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("4000K"),
  { 1.7695, 1, 1.7344, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("5000K"),
  { 2.0508, 1, 1.4883, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("6000K"),
  { 2.2578, 1, 1.3477, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("7000K"),
  { 2.4141, 1, 1.2578, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("8000K"),
  { 2.5312, 1, 1.1992, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("9000K"),
  { 2.6250, 1, 1.1523, 0 } },
{ "PENTAX",
  "K-5",
  QObject::tr("10000K"),
  { 2.6992, 1, 1.1211, 0 } },

// PENTAX , K-7
{ "PENTAX",
  "K-7",
  QObject::tr("Daylight"),
  { 1.808594, 1, 1.285156, 0 } },
{ "PENTAX",
  "K-7",
  QObject::tr("Shade"),
  { 2.207171, 1.019920, 1, 0 } },
{ "PENTAX",
  "K-7",
  QObject::tr("Cloudy"),
  { 1.960937, 1, 1.136719, 0 } },
{ "PENTAX",
  "K-7",
  QObject::tr("DaylightFluorescent"),
  { 2.281250, 1, 1.191406, 0 } },
{ "PENTAX",
  "K-7",
  QObject::tr("NeutralFluorescent"),
  { 1.937500, 1, 1.355469, 0 } },
{ "PENTAX",
  "K-7",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.808594, 1, 1.687500, 0 } },
{ "PENTAX",
  "K-7",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.589844, 1, 2.164063, 0 } },
{ "PENTAX",
  "K-7",
  QObject::tr("Tungsten"),
  { 1.105469, 1, 2.347656, 0 } },
{ "PENTAX",
  "K-7",
  QObject::tr("Flash"),
  { 2.093750, 1, 1.082031, 0 } },

// PENTAX , K-m
{ "PENTAX",
  "K-m",
  QObject::tr("Daylight"),
  { 1.738281, 1, 1.363281, 0 } },
{ "PENTAX",
  "K-m",
  QObject::tr("Shade"),
  { 2.027344, 1, 1.027344, 0 } },
{ "PENTAX",
  "K-m",
  QObject::tr("Cloudy"),
  { 1.832031, 1, 1.183594, 0 } },
{ "PENTAX",
  "K-m",
  QObject::tr("DaylightFluorescent"),
  { 2.183594, 1, 1.250000, 0 } },
{ "PENTAX",
  "K-m",
  QObject::tr("NeutralFluorescent"),
  { 1.824219, 1, 1.417969, 0 } },
{ "PENTAX",
  "K-m",
  QObject::tr("WhiteFluorescent"),
  { 1.644531, 1, 1.714844, 0 } },
{ "PENTAX",
  "K-m",
  QObject::tr("Tungsten"),
  { 1.429687, 1, 1.980469, 0 } },
{ "PENTAX",
  "K-m",
  QObject::tr("Flash"),
  { 1.738281, 1, 1.363281, 0 } },

// PENTAX , K-r
{ "PENTAX",
  "K-r",
  QObject::tr("Daylight"),
  { 1.8477, 1, 1.3906, 0 } },
{ "PENTAX",
  "K-r",
  QObject::tr("Shade"),
  { 2.1133, 1, 1.0586, 0 } },
{ "PENTAX",
  "K-r",
  QObject::tr("Cloudy"),
  { 1.9766, 1, 1.1445, 0 } },
{ "PENTAX",
  "K-r",
  QObject::tr("DaylightFluorescent"),
  { 2.2617, 1, 1.3203, 0 } },
{ "PENTAX",
  "K-r",
  QObject::tr("WhiteFluorescent"),
  { 1.9414, 1, 1.4688, 0 } },
{ "PENTAX",
  "K-r",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.7656, 1, 1.8164, 0 } },
{ "PENTAX",
  "K-r",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.4766, 1, 2.2734, 0 } },
{ "PENTAX",
  "K-r",
  QObject::tr("Tungsten"),
  { 1.1016, 1, 2.5391, 0 } },
{ "PENTAX",
  "K-r",
  QObject::tr("Flash"),
  { 2.0117, 1, 1.1172, 0 } },

// PENTAX , K-x
{ "PENTAX",
  "K-x",
  QObject::tr("Daylight"),
  { 1.8803, 1, 1.4054, 0 } },
{ "PENTAX",
  "K-x",
  QObject::tr("Shade"),
  { 2.2278, 1, 1.0309, 0 } },
{ "PENTAX",
  "K-x",
  QObject::tr("Cloudy"),
  { 2.0077, 1, 1.1853, 0 } },
{ "PENTAX",
  "K-x",
  QObject::tr("DaylightFluorescent"),
  { 2.3012, 1, 1.3359, 0 } },
{ "PENTAX",
  "K-x",
  QObject::tr("WhiteFluorescent"),
  { 1.9730, 1, 1.4826, 0 } },
{ "PENTAX",
  "K-x",
  QObject::tr("CoolWhiteFluorescent"),
  { 1.7915, 1, 1.8378, 0 } },
{ "PENTAX",
  "K-x",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.5019, 1, 2.2973, 0 } },
{ "PENTAX",
  "K-x",
  QObject::tr("Tungsten"),
  { 1.0463, 1, 3.4015, 0 } },
{ "PENTAX",
  "K-x",
  QObject::tr("Flash"),
  { 2.1120, 1, 1.2741, 0 } },

// RICOH , Caplio GX100
{ "RICOH",
  "Caplio GX100",
  QObject::tr("Daylight"),
  { 1.910001, 1, 1.820002, 0 } },
{ "RICOH",
  "Caplio GX100",
  QObject::tr("Cloudy"),
  { 2.240003, 1, 1.530002, 0 } },
{ "RICOH",
  "Caplio GX100",
  QObject::tr("Incandescent"),
  { 1.520002, 1, 2.520003, 0 } },
{ "RICOH",
  "Caplio GX100",
  QObject::tr("Fluorescent"),
  { 1.840001, 1, 1.970001, 0 } },

// SAMSUNG , GX-1S
{ "SAMSUNG",
  "GX-1S",
  QObject::tr("Daylight"),
  { 1.574219, 1, 1.109375, 0 } },
{ "SAMSUNG",
  "GX-1S",
  QObject::tr("Shade"),
  { 1.855469, 1, 1.000000, 0 } },
{ "SAMSUNG",
  "GX-1S",
  QObject::tr("Cloudy"),
  { 1.664062, 1, 1.000000, 0 } },
{ "SAMSUNG",
  "GX-1S",
  QObject::tr("DaylightFluorescent"),
  { 1.854251, 1.036437, 1, 0 } },
{ "SAMSUNG",
  "GX-1S",
  QObject::tr("NeutralFluorescent"),
  { 1.574219, 1, 1.171875, 0 } },
{ "SAMSUNG",
  "GX-1S",
  QObject::tr("WhiteFluorescent"),
  { 1.363281, 1, 1.335938, 0 } },
{ "SAMSUNG",
  "GX-1S",
  QObject::tr("Tungsten"),
  { 1.000000, 1, 2.226562, 0 } },
{ "SAMSUNG",
  "GX-1S",
  QObject::tr("Flash"),
  { 1.609375, 1, 1.031250, 0 } },

// SAMSUNG , GX10
{ "SAMSUNG",
  "GX10",
  QObject::tr("Daylight"),
  { 1.660156, 1, 1.066406, 0 } },
{ "SAMSUNG",
  "GX10",
  QObject::tr("Shade"),
  { 2.434783, 1.236715, 1, 0 } },
{ "SAMSUNG",
  "GX10",
  QObject::tr("Cloudy"),
  { 1.872428, 1.053498, 1, 0 } },
{ "SAMSUNG",
  "GX10",
  QObject::tr("DaylightFluorescent"),
  { 2.121094, 1, 1.078125, 0 } },
{ "SAMSUNG",
  "GX10",
  QObject::tr("NeutralFluorescent"),
  { 1.773438, 1, 1.226562, 0 } },
{ "SAMSUNG",
  "GX10",
  QObject::tr("WhiteFluorescent"),
  { 1.597656, 1, 1.488281, 0 } },
{ "SAMSUNG",
  "GX10",
  QObject::tr("Tungsten"),
  { 1.000000, 1, 2.558594, 0 } },
{ "SAMSUNG",
  "GX10",
  QObject::tr("Flash"),
  { 1.664062, 1, 1.046875, 0 } },

// SAMSUNG , NX100
{ "SAMSUNG",
  "NX100",
  QObject::tr("Daylight"),
  { 1.760078, 1, 1.165404, 0 } },
{ "SAMSUNG",
  "NX100",
  QObject::tr("Cloudy"),
  { 1.841437, 1, 1.103836, 0 } },
{ "SAMSUNG",
  "NX100",
  QObject::tr("WhiteFluorescent"),
  { 1.714879, 1, 1.768141, 0 } },
{ "SAMSUNG",
  "NX100",
  QObject::tr("NeutralFluorescent"),
  { 1.900318, 1, 1.438309, 0 } },
{ "SAMSUNG",
  "NX100",
  QObject::tr("DaylightFluorescent"),
  { 2.242365, 1, 1.231859, 0 } },
{ "SAMSUNG",
  "NX100",
  QObject::tr("Tungsten"),
  { 1.069142, 1, 2.354996, 0 } },
{ "SAMSUNG",
  "NX100",
  QObject::tr("Flash"),
  { 2.071719, 1, 0.978723, 0 } },
{ "SAMSUNG",
  "NX100",
  QObject::tr("5000K"),
  { 1.684339, 1, 1.094063, 0 } },
{ "SAMSUNG",
  "NX100",
  QObject::tr("5500K"),
  { 1.692022, 1, 0.843950, 0 } },
{ "SAMSUNG",
  "NX100",
  QObject::tr("6500K"),
  { 2.110371, 1, 0.879107, 0 } },

// SAMSUNG , NX200
{ "SAMSUNG",
  "NX200",
  QObject::tr("Daylight"),
  { 2.773438, 1, 1.625000, 0 } },
{ "SAMSUNG",
  "NX200",
  QObject::tr("Cloudy"),
  { 2.902344, 1, 1.546875, 0 } },
{ "SAMSUNG",
  "NX200",
  QObject::tr("WhiteFluorescent"),
  { 2.445313, 1, 2.316406, 0 } },
{ "SAMSUNG",
  "NX200",
  QObject::tr("NeutralFluorescent"),
  { 2.746094, 1, 1.937500, 0 } },
{ "SAMSUNG",
  "NX200",
  QObject::tr("DaylightFluorescent"),
  { 3.214844, 1, 1.679688, 0 } },
{ "SAMSUNG",
  "NX200",
  QObject::tr("Tungsten"),
  { 1.511719, 1, 2.941406, 0 } },
{ "SAMSUNG",
  "NX200",
  QObject::tr("Flash"),
  { 2.914063, 1, 1.191406, 0 } },

// SONY , DSLR-A100
{ "SONY",
  "DSLR-A100",
  QObject::tr("Daylight"),
  { 1.746094, 1, 1.843750, 0 } },
{ "SONY",
  "DSLR-A100",
  QObject::tr("Shade"),
  { 2.070312, 1, 1.609375, 0 } },
{ "SONY",
  "DSLR-A100",
  QObject::tr("Cloudy"),
  { 1.855469, 1, 1.628906, 0 } },
{ "SONY",
  "DSLR-A100",
  QObject::tr("Tungsten"),
  { 1.054688, 1, 3.917969, 0 } },
{ "SONY",
  "DSLR-A100",
  QObject::tr("Fluorescent"),
  { 1.718750, 1, 3.058594, 0 } },
{ "SONY",
  "DSLR-A100",
  QObject::tr("Flash"),
  { 1.859375, 1, 1.746094, 0 } },

// SONY , DSLR-A200
{ "SONY",
  "DSLR-A200",
  QObject::tr("Daylight"),
  { 1.664062, 1, 1.757812, 0 } },
{ "SONY",
  "DSLR-A200",
  QObject::tr("Shade"),
  { 1.972656, 1, 1.390625, 0 } },
{ "SONY",
  "DSLR-A200",
  QObject::tr("Cloudy"),
  { 1.800781, 1, 1.585937, 0 } },
{ "SONY",
  "DSLR-A200",
  QObject::tr("Tungsten"),
  { 1, 1.027344, 3.492187, 0 } },
{ "SONY",
  "DSLR-A200",
  QObject::tr("Fluorescent"),
  { 1.554687, 1, 2.601562, 0 } },
{ "SONY",
  "DSLR-A200",
  QObject::tr("Flash"),
  { 1.917969, 1, 1.460937, 0 } },
{ "SONY",
  "DSLR-A200",
  QObject::tr("5600K"),
  { 1.710938, 1, 1.683594, 0 } },

// SONY , DSLR-A300
{ "SONY",
  "DSLR-A300",
  QObject::tr("Daylight"),
  { 1.632813, 1, 1.730469, 0 } },
{ "SONY",
  "DSLR-A300",
  QObject::tr("Shade"),
  { 1.937500, 1, 1.371094, 0 } },
{ "SONY",
  "DSLR-A300",
  QObject::tr("Cloudy"),
  { 1.769531, 1, 1.566406, 0 } },
{ "SONY",
  "DSLR-A300",
  QObject::tr("Tungsten"),
  { 1, 1.039063, 3.449219, 0 } },
{ "SONY",
  "DSLR-A300",
  QObject::tr("Fluorescent"),
  { 1.527344, 1, 2.546875, 0 } },
{ "SONY",
  "DSLR-A300",
  QObject::tr("Flash"),
  { 1.882812, 1, 1.441406, 0 } },

// SONY , DSLR-A330
{ "SONY",
  "DSLR-A330",
  QObject::tr("Daylight"),
  { 1.7500, 1, 1.7227, 0 } },
{ "SONY",
  "DSLR-A330",
  QObject::tr("Shade"),
  { 2.0703, 1, 1.3359, 0 } },
{ "SONY",
  "DSLR-A330",
  QObject::tr("Cloudy"),
  { 1.8906, 1, 1.5430, 0 } },
{ "SONY",
  "DSLR-A330",
  QObject::tr("Tungsten"),
  { 1.0352, 1, 3.4219, 0 } },
{ "SONY",
  "DSLR-A330",
  QObject::tr("Fluorescent"),
  { 1.6172, 1, 2.4648, 0 } },
{ "SONY",
  "DSLR-A330",
  QObject::tr("Flash"),
  { 1.9375, 1, 1.5313, 0 } },

// SONY , DSLR-A350
{ "SONY",
  "DSLR-A350",
  QObject::tr("Daylight"),
  { 2.531250, 1, 1.648437, 0 } },
{ "SONY",
  "DSLR-A350",
  QObject::tr("Shade"),
  { 2.960937, 1, 1.281250, 0 } },
{ "SONY",
  "DSLR-A350",
  QObject::tr("Cloudy"),
  { 2.722656, 1, 1.476562, 0 } },
{ "SONY",
  "DSLR-A350",
  QObject::tr("Tungsten"),
  { 1.578125, 1, 3.289062, 0 } },
{ "SONY",
  "DSLR-A350",
  QObject::tr("Flash"),
  { 2.875000, 1, 1.355469, 0 } },
{ "SONY",
  "DSLR-A350",
  QObject::tr("CoolWhiteFluorescent"),
  { 2.226563, 1, 2.355469, 0 } },
{ "SONY",
  "DSLR-A350",
  QObject::tr("Fluorescent"),
  { 1.554687, 1, 3.984375, 0 } },
{ "SONY",
  "DSLR-A350",
  QObject::tr("WarmWhiteFluorescent"),
  { 1.816406, 1, 3.207031, 0 } },
{ "SONY",
  "DSLR-A350",
  QObject::tr("DayWhiteFluorescent"),
  { 2.511719, 1, 1.957031, 0 } },
{ "SONY",
  "DSLR-A350",
  QObject::tr("DaylightFluorescent"),
  { 3.023437, 1, 1.671875, 0 } },

// SONY , DSLR-A380
{ "SONY",
  "DSLR-A380",
  QObject::tr("Daylight"),
  { 2.562500, 1, 1.648438, 0 } },
{ "SONY",
  "DSLR-A380",
  QObject::tr("Shade"),
  { 3.019531, 1, 1.292969, 0 } },
{ "SONY",
  "DSLR-A380",
  QObject::tr("Cloudy"),
  { 2.765625, 1, 1.480469, 0 } },
{ "SONY",
  "DSLR-A380",
  QObject::tr("Tungsten"),
  { 1.550781, 1, 3.222656, 0 } },
{ "SONY",
  "DSLR-A380",
  QObject::tr("Fluorescent"),
  { 2.234375, 1, 2.335938, 0 } },
{ "SONY",
  "DSLR-A380",
  QObject::tr("Flash"),
  { 2.816406, 1, 1.453125, 0 } },

// SONY , DSLR-A390
{ "SONY",
  "DSLR-A390",
  QObject::tr("Daylight"),
  { 2.2188, 1, 1.3711, 0 } },
{ "SONY",
  "DSLR-A390",
  QObject::tr("Shade"),
  { 2.6055, 1, 1.0469, 0 } },
{ "SONY",
  "DSLR-A390",
  QObject::tr("Cloudy"),
  { 2.3906, 1, 1.2227, 0 } },
{ "SONY",
  "DSLR-A390",
  QObject::tr("Tungsten"),
  { 1.3633, 1, 2.8164, 0 } },
{ "SONY",
  "DSLR-A390",
  QObject::tr("Fluorescent"),
  { 1.9414, 1, 2.0039, 0 } },
{ "SONY",
  "DSLR-A390",
  QObject::tr("Flash"),
  { 2.4336, 1, 1.1953, 0 } },

// SONY , DSLR-A450
{ "SONY",
  "DSLR-A450",
  QObject::tr("Daylight"),
  { 2.296875, 1, 1.445312, 0 } },
{ "SONY",
  "DSLR-A450",
  QObject::tr("Shade"),
  { 2.691406, 1, 1.214844, 0 } },
{ "SONY",
  "DSLR-A450",
  QObject::tr("Cloudy"),
  { 2.464844, 1, 1.335938, 0 } },
{ "SONY",
  "DSLR-A450",
  QObject::tr("Tungsten"),
  { 1.437500, 1, 2.468750, 0 } },
{ "SONY",
  "DSLR-A450",
  QObject::tr("Fluorescent"),
  { 2.019531, 1, 2.003906, 0 } },
{ "SONY",
  "DSLR-A450",
  QObject::tr("Flash"),
  { 2.550781, 1, 1.304688, 0 } },

// SONY , DSLR-A550
{ "SONY",
  "DSLR-A550",
  QObject::tr("Daylight"),
  { 2.160156, 1, 1.496094, 0 } },
{ "SONY",
  "DSLR-A550",
  QObject::tr("Shade"),
  { 2.519531, 1, 1.234375, 0 } },
{ "SONY",
  "DSLR-A550",
  QObject::tr("Cloudy"),
  { 2.312500, 1, 1.375000, 0 } },
{ "SONY",
  "DSLR-A550",
  QObject::tr("Tungsten"),
  { 1.367188, 1, 2.632813, 0 } },
{ "SONY",
  "DSLR-A550",
  QObject::tr("Fluorescent"),
  { 1.902344, 1, 2.117188, 0 } },
{ "SONY",
  "DSLR-A550",
  QObject::tr("Flash"),
  { 2.390625, 1, 1.335938, 0 } },

// SONY , DSLR-A700
{ "SONY",
  "DSLR-A700",
  QObject::tr("Daylight"),
  { 2.101563, 1, 1.484375, 0 } },
{ "SONY",
  "DSLR-A700",
  QObject::tr("Shade"),
  { 2.445313, 1, 1.226563, 0 } },
{ "SONY",
  "DSLR-A700",
  QObject::tr("Cloudy"),
  { 2.250000, 1, 1.367188, 0 } },
{ "SONY",
  "DSLR-A700",
  QObject::tr("Tungsten"),
  { 1.335938, 1, 2.597656, 0 } },
{ "SONY",
  "DSLR-A700",
  QObject::tr("Fluorescent"),
  { 1.878906, 1, 2.152344, 0 } },
{ "SONY",
  "DSLR-A700",
  QObject::tr("Flash"),
  { 2.277344, 1, 1.312500, 0 } },

// SONY , DSLR-A850
{ "SONY",
  "DSLR-A850",
  QObject::tr("Daylight"),
  { 2.683594, 1, 1.359375, 0 } },
{ "SONY",
  "DSLR-A850",
  QObject::tr("Shade"),
  { 3.164062, 1, 1.121094, 0 } },
{ "SONY",
  "DSLR-A850",
  QObject::tr("Cloudy"),
  { 2.898438, 1, 1.250000, 0 } },
{ "SONY",
  "DSLR-A850",
  QObject::tr("Tungsten"),
  { 1.617188, 1, 2.421875, 0 } },
{ "SONY",
  "DSLR-A850",
  QObject::tr("Fluorescent"),
  { 2.324219, 1, 1.882813, 0 } },
{ "SONY",
  "DSLR-A850",
  QObject::tr("Flash"),
  { 3.015625, 1, 1.191406, 0 } },

// SONY , DSLR-A900
{ "SONY",
  "DSLR-A900",
  QObject::tr("Daylight"),
  { 2.585938, 1, 1.355469, 0 } },
{ "SONY",
  "DSLR-A900",
  QObject::tr("Shade"),
  { 3.054688, 1, 1.113281, 0 } },
{ "SONY",
  "DSLR-A900",
  QObject::tr("Cloudy"),
  { 2.792969, 1, 1.242187, 0 } },
{ "SONY",
  "DSLR-A900",
  QObject::tr("Tungsten"),
  { 1.546875, 1, 2.425781, 0 } },
{ "SONY",
  "DSLR-A900",
  QObject::tr("Fluorescent"),
  { 2.234375, 1, 1.882812, 0 } },
{ "SONY",
  "DSLR-A900",
  QObject::tr("Flash"),
  { 2.910156, 1, 1.183594, 0 } },

// SONY , NEX-3
{ "SONY",
  "NEX-3",
  QObject::tr("Daylight"),
  { 2.2539, 1, 1.4727, 0 } },
{ "SONY",
  "NEX-3",
  QObject::tr("Shade"),
  { 2.6289, 1, 1.2188, 0 } },
{ "SONY",
  "NEX-3",
  QObject::tr("Cloudy"),
  { 2.4141, 1, 1.3516, 0 } },
{ "SONY",
  "NEX-3",
  QObject::tr("Incandescent"),
  { 1.4336, 1, 2.5742, 0 } },
{ "SONY",
  "NEX-3",
  QObject::tr("Fluorescent"),
  { 1.9883, 1, 2.0742, 0 } },
{ "SONY",
  "NEX-3",
  QObject::tr("Flash"),
  { 2.7500, 1, 1.2383, 0 } },

// SONY , NEX-5
{ "SONY",
  "NEX-5",
  QObject::tr("Daylight"),
  { 2.3125, 1, 1.4453, 0 } },
{ "SONY",
  "NEX-5",
  QObject::tr("Shade"),
  { 2.6992, 1, 1.1914, 0 } },
{ "SONY",
  "NEX-5",
  QObject::tr("Cloudy"),
  { 2.4766, 1, 1.3281, 0 } },
{ "SONY",
  "NEX-5",
  QObject::tr("Incandescent"),
  { 1.4609, 1, 2.5703, 0 } },
{ "SONY",
  "NEX-5",
  QObject::tr("Fluorescent"),
  { 2.0352, 1, 2.0625, 0 } },
{ "SONY",
  "NEX-5",
  QObject::tr("Flash"),
  { 2.8281, 1, 1.2109, 0 } },

// SONY , NEX-7
{ "SONY",
  "NEX-7",
  QObject::tr("Daylight"),
  { 2.695313, 1, 1.425781, 0 } },
{ "SONY",
  "NEX-7",
  QObject::tr("Shade"),
  { 3.214844, 1, 1.1875,   0 } },
{ "SONY",
  "NEX-7",
  QObject::tr("Cloudy"),
  { 2.914063, 1, 1.316406, 0 } },
{ "SONY",
  "NEX-7",
  QObject::tr("Tungsten"),
  { 1.660156, 1, 2.445313, 0 } },
{ "SONY",
  "NEX-7",
  QObject::tr("Fluorescent"),
  { 2.421875, 1, 1.957031, 0 } },
{ "SONY",
  "NEX-7",
  QObject::tr("Flash"),
  { 3.050781, 1, 1.347656, 0 } },

// SONY , SLT-A55V
{ "SONY",
  "SLT-A55V",
  QObject::tr("Daylight"),
  { 2.5469, 1, 1.5117, 0 } },
{ "SONY",
  "SLT-A55V",
  QObject::tr("Shade"),
  { 2.9922, 1, 1.2500, 0 } },
{ "SONY",
  "SLT-A55V",
  QObject::tr("Cloudy"),
  { 2.7344, 1, 1.3906, 0 } },
{ "SONY",
  "SLT-A55V",
  QObject::tr("Incandescent"),
  { 1.5703, 1, 2.6523, 0 } },
{ "SONY",
  "SLT-A55V",
  QObject::tr("Fluorescent"),
  { 2.2305, 1, 2.1094, 0 } },
{ "SONY",
  "SLT-A55V",
  QObject::tr("Flash"),
  { 2.8437, 1, 1.3359, 0 } },

// SONY , SLT-A65V
{ "SONY",
  "SLT-A65V",
  QObject::tr("Daylight"),
  { 2.628906, 1, 1.433594, 0 } },
{ "SONY",
  "SLT-A65V",
  QObject::tr("Shade"),
  { 3.132813, 1, 1.191406, 0 } },
{ "SONY",
  "SLT-A65V",
  QObject::tr("Cloudy"),
  { 2.839844, 1, 1.316406, 0 } },
{ "SONY",
  "SLT-A65V",
  QObject::tr("Tungsten"),
  { 1.617188, 1, 2.488281, 0 } },
{ "SONY",
  "SLT-A65V",
  QObject::tr("Fluorescent"),
  { 2.363281, 1, 1.980469, 0 } },
{ "SONY",
  "SLT-A65V",
  QObject::tr("Flash"),
  { 3.070313, 1, 1.238281, 0 } },

// Terminator
{ "","","", {0.0,0.0,0.0} }

};
